@@ -1,7 +1,9 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "debug.h"
 #include "table/strings.h"
 #include "map.h"
+#include "tile.h"
 #include "window.h"
 #include "gui.h"
 #include "vehicle.h"
@@ -11,6 +13,7 @@
 #include "engine.h"
 #include "viewport.h"
 #include "player.h"
+#include "depot.h"
 
 
 void Set_DPARAM_Aircraft_Build_Window(uint16 engine_number)
@@ -33,7 +36,7 @@ void Set_DPARAM_Aircraft_Build_Window(uint16 engine_number)
 
 }
 
-static void DrawAircraftImage(Vehicle *v, int x, int y, VehicleID selection)
+static void DrawAircraftImage(const Vehicle *v, int x, int y, VehicleID selection)
 {
 	int image = GetAircraftImage(v, 6);
 	uint32 ormod = SPRITE_PALETTE(PLAYER_SPRITE_COLOR(v->owner));
@@ -148,11 +151,11 @@ static void NewAircraftWndProc(Window *w, WindowEvent *e)
 		break;
 
 	case WE_ON_EDIT_TEXT: {
-		byte *b = e->edittext.str;
-		if (*b == 0)
-			return;
-		memcpy(_decode_parameters, b, 32);
-		DoCommandP(0, WP(w,buildtrain_d).rename_engine, 0, NULL, CMD_RENAME_ENGINE | CMD_MSG(STR_A03A_CAN_T_RENAME_AIRCRAFT_TYPE));
+		if (e->edittext.str[0] != '\0') {
+			_cmd_text = e->edittext.str;
+			DoCommandP(0, WP(w, buildtrain_d).rename_engine, 0, NULL,
+				CMD_RENAME_ENGINE | CMD_MSG(STR_A03A_CAN_T_RENAME_AIRCRAFT_TYPE));
+		}
 	} break;
 
 	case WE_RESIZE:
@@ -202,64 +205,11 @@ static void ShowBuildAircraftWindow(uint tile)
 	}
 }
 
-#define MAX_REFIT 0xFF
-
-const byte _aircraft_refit_normal[] = {
-	CT_PASSENGERS,
-	CT_MAIL,
-	CT_GOODS,
-	CT_VALUABLES,
-	MAX_REFIT
-};
-
-const byte _aircraft_refit_arctic[] = {
-	CT_PASSENGERS,
-	CT_MAIL,
-	CT_GOODS,
-	CT_FOOD,
-	MAX_REFIT
-};
-
-const byte _aircraft_refit_desert[] = {
-	CT_PASSENGERS,
-	CT_MAIL,
-	CT_FRUIT,
-	CT_GOODS,
-	CT_DIAMONDS,
-	MAX_REFIT
-};
-
-const byte _aircraft_refit_candy[] = {
-	CT_PASSENGERS,
-	CT_SUGAR,
-	CT_TOYS,
-	CT_CANDY,
-	CT_COLA,
-	CT_COTTON_CANDY,
-	CT_BUBBLES,
-	CT_TOFFEE,
-	CT_BATTERIES,
-	CT_PLASTIC,
-	CT_FIZZY_DRINKS,
-	MAX_REFIT
-};
-
-const byte * const _aircraft_refit_types[4] = {
-	_aircraft_refit_normal, _aircraft_refit_arctic, _aircraft_refit_desert, _aircraft_refit_candy
-};
-
-#undef MAX_REFIT
-
 static void AircraftRefitWndProc(Window *w, WindowEvent *e)
 {
-	switch(e->event) {
+	switch (e->event) {
 	case WE_PAINT: {
-		Vehicle *v = GetVehicle(w->window_number);
-		const byte *b;
-		int sel;
-		int x,y;
-		byte color;
-		int cargo;
+		const Vehicle *v = GetVehicle(w->window_number);
 
 		SetDParam(0, v->string_id);
 		SetDParam(1, v->unitnumber);
@@ -268,72 +218,31 @@ static void AircraftRefitWndProc(Window *w, WindowEvent *e)
 		DrawString(1, 15, STR_A040_SELECT_CARGO_TYPE_TO_CARRY, 0);
 
 		/* TODO: Support for custom GRFSpecial-specified refitting! --pasky */
+		WP(w,refit_d).cargo = DrawVehicleRefitWindow(v, WP(w, refit_d).sel);
 
-		cargo = -1;
-		x = 6;
-		y = 25;
-		sel = WP(w,refit_d).sel;
-
-#define show_cargo(ctype) { \
-		color = 16; \
-		if (sel == 0) { \
-			cargo = ctype; \
-			color = 12; \
-		} \
-		sel--; \
-		DrawString(x, y, _cargoc.names_s[ctype], color); \
-		y += 10; \
-		}
-
-		if (_engine_refit_masks[v->engine_type]) {
-			uint32 mask = _engine_refit_masks[v->engine_type];
-			int cid = 0;
-
-			for (; mask; mask >>= 1, cid++) {
-				if (!(mask & 1)) // not this cid
-					continue;
-				if (!(_local_cargo_id_landscape[cid] & (1 << _opt.landscape))) // not in this landscape
-					continue;
-
-				show_cargo(_local_cargo_id_ctype[cid]);
-			}
-
-		} else { // generic refit list
-			b = _aircraft_refit_types[_opt.landscape];
-			do {
-				show_cargo(*b);
-			} while (*++b != 0xFF);
-		}
-
-#undef show_cargo
-
-		WP(w,refit_d).cargo = cargo;
-
-		if (cargo != -1) {
-			int32 cost = DoCommandByTile(v->tile, v->index, cargo, DC_QUERY_COST, CMD_REFIT_AIRCRAFT);
-			if (cost != CMD_ERROR) {
+		if (WP(w,refit_d).cargo != CT_INVALID) {
+			int32 cost = DoCommandByTile(v->tile, v->index, WP(w,refit_d).cargo, DC_QUERY_COST, CMD_REFIT_AIRCRAFT);
+			if (!CmdFailed(cost)) {
 				SetDParam(2, cost);
-				SetDParam(0, _cargoc.names_long_p[cargo]);
+				SetDParam(0, _cargoc.names_long_p[WP(w,refit_d).cargo]);
 				SetDParam(1, _aircraft_refit_capacity);
 				DrawString(1, 137, STR_A041_NEW_CAPACITY_COST_OF_REFIT, 0);
 			}
 		}
-
-		break;
-	}
+	}	break;
 
 	case WE_CLICK:
 		switch(e->click.widget) {
 		case 2: { /* listbox */
-				int y = e->click.pt.y - 25;
-				if (y >= 0) {
-					WP(w,refit_d).sel = y / 10;
-					SetWindowDirty(w);
-				}
-			} break;
+			int y = e->click.pt.y - 25;
+			if (y >= 0) {
+				WP(w,refit_d).sel = y / 10;
+				SetWindowDirty(w);
+			}
+		} break;
 		case 4: /* refit button */
-			if (WP(w,refit_d).cargo != 0xFF) {
-				Vehicle *v = GetVehicle(w->window_number);
+			if (WP(w,refit_d).cargo != CT_INVALID) {
+				const Vehicle *v = GetVehicle(w->window_number);
 				if (DoCommandP(v->tile, v->index, WP(w,refit_d).cargo, NULL, CMD_REFIT_AIRCRAFT | CMD_MSG(STR_A042_CAN_T_REFIT_AIRCRAFT)))
 					DeleteWindow(w);
 			}
@@ -375,11 +284,10 @@ static void ShowAircraftRefitWindow(Vehicle *v)
 
 static void AircraftDetailsWndProc(Window *w, WindowEvent *e)
 {
-	int mod;
-	Vehicle *v = GetVehicle(w->window_number), *u;
+	switch (e->event) {
+	case WE_PAINT: {
+		const Vehicle *v = GetVehicle(w->window_number);
 
-	switch(e->event) {
-	case WE_PAINT:
 		w->disabled_state = v->owner == _local_player ? 0 : (1 << 2);
 		if (!_patches.servint_aircraft) // disable service-scroller when interval is set to disabled
 			w->disabled_state |= (1 << 5) | (1 << 6);
@@ -437,6 +345,7 @@ static void AircraftDetailsWndProc(Window *w, WindowEvent *e)
 		DrawAircraftImage(v, 3, 57, INVALID_VEHICLE);
 
 		{
+			const Vehicle *u;
 			int y = 57;
 
 			do {
@@ -468,34 +377,32 @@ static void AircraftDetailsWndProc(Window *w, WindowEvent *e)
 				}
 			} while ( (v=v->next) != NULL);
 		}
-		break;
+	} break;
 
-	case WE_CLICK:
-		switch(e->click.widget) {
+	case WE_CLICK: {
+		int mod;
+		const Vehicle *v;
+		switch (e->click.widget) {
 		case 2: /* rename */
+			v = GetVehicle(w->window_number);
 			SetDParam(0, v->unitnumber);
 			ShowQueryString(v->string_id, STR_A030_NAME_AIRCRAFT, 31, 150, w->window_class, w->window_number);
 			break;
-
 		case 5: /* increase int */
 			mod = _ctrl_pressed? 5 : 10;
-			goto change_int;
+			goto do_change_service_int;
 		case 6: /* decrease int */
 			mod = _ctrl_pressed?- 5 : -10;
-change_int:
-			mod += v->service_interval;
+do_change_service_int:
+			v = GetVehicle(w->window_number);
 
-			/*	%-based service interval max 5%-90%
-					day-based service interval max 30-800 days */
-			mod = _patches.servint_ispercent ? clamp(mod, MIN_SERVINT_PERCENT, MAX_SERVINT_PERCENT) : clamp(mod, MIN_SERVINT_DAYS, MAX_SERVINT_DAYS+1);
-			if (mod == v->service_interval)
-				return;
+			mod = GetServiceIntervalClamped(mod + v->service_interval);
+			if (mod == v->service_interval) return;
 
-			DoCommandP(v->tile, v->index, mod, NULL,
-				CMD_CHANGE_AIRCRAFT_SERVICE_INT | CMD_MSG(STR_018A_CAN_T_CHANGE_SERVICING));
+			DoCommandP(v->tile, v->index, mod, NULL, CMD_CHANGE_AIRCRAFT_SERVICE_INT | CMD_MSG(STR_018A_CAN_T_CHANGE_SERVICING));
 			break;
 		}
-		break;
+	} break;
 
 	case WE_4:
 		if (FindWindowById(WC_VEHICLE_VIEW, w->window_number) == NULL)
@@ -503,11 +410,11 @@ change_int:
 		break;
 
 	case WE_ON_EDIT_TEXT: {
-		byte *b = e->edittext.str;
-		if (*b == 0)
-			return;
-		memcpy(_decode_parameters, b, 32);
-		DoCommandP(0, w->window_number, 0, NULL, CMD_NAME_VEHICLE | CMD_MSG(STR_A031_CAN_T_NAME_AIRCRAFT));
+		if (e->edittext.str[0] != '\0') {
+			_cmd_text = e->edittext.str;
+			DoCommandP(0, w->window_number, 0, NULL,
+				CMD_NAME_VEHICLE | CMD_MSG(STR_A031_CAN_T_NAME_AIRCRAFT));
+		}
 	} break;
 
 	}
@@ -553,18 +460,20 @@ static void ShowAircraftDetailsWindow(Vehicle *v)
 
 
 static const Widget _aircraft_view_widgets[] = {
-{    WWT_TEXTBTN,   RESIZE_NONE,    14,     0,    10,     0,    13, STR_00C5,	STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,   RESIZE_NONE,    14,    11,   237,     0,    13, STR_A00A,	STR_018C_WINDOW_TITLE_DRAG_THIS},
-{  WWT_STICKYBOX,   RESIZE_NONE,    14,   238,   249,     0,    13, 0x0,       STR_STICKY_BUTTON},
-{     WWT_IMGBTN,   RESIZE_NONE,    14,     0,   231,    14,   103, 0x0,				STR_NULL},
-{          WWT_6,   RESIZE_NONE,    14,     2,   229,    16,   101, 0x0,				STR_NULL},
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,    14,     0,   249,   104,   115, 0x0,				STR_A027_CURRENT_AIRCRAFT_ACTION},
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,    14,   232,   249,    14,    31, 0x2AB,			STR_A029_CENTER_MAIN_VIEW_ON_AIRCRAFT},
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,    14,   232,   249,    32,    49, 0x2AF,			STR_A02A_SEND_AIRCRAFT_TO_HANGAR},
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,    14,   232,   249,    50,    67, 0x2B4,			STR_A03B_REFIT_AIRCRAFT_TO_CARRY},
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,    14,   232,   249,    68,    85, 0x2B2,			STR_A028_SHOW_AIRCRAFT_S_ORDERS},
-{ WWT_PUSHIMGBTN,   RESIZE_NONE,    14,   232,   249,    86,   103, 0x2B3,			STR_A02B_SHOW_AIRCRAFT_DETAILS},
-{   WIDGETS_END},
+{ WWT_TEXTBTN,    RESIZE_NONE,  14,   0,  10,   0,  13, STR_00C5, STR_018B_CLOSE_WINDOW },
+{ WWT_CAPTION,    RESIZE_RIGHT, 14,  11, 237,   0,  13, STR_A00A, STR_018C_WINDOW_TITLE_DRAG_THIS },
+{ WWT_STICKYBOX,  RESIZE_LR,    14, 238, 249,   0,  13, 0x0,      STR_STICKY_BUTTON },
+{ WWT_IMGBTN,     RESIZE_RB,    14,   0, 231,  14, 103, 0x0,      STR_NULL },
+{ WWT_6,          RESIZE_RB,    14,   2, 229,  16, 101, 0x0,      STR_NULL },
+{ WWT_PUSHIMGBTN, RESIZE_RTB,   14,   0, 237, 104, 115, 0x0,      STR_A027_CURRENT_AIRCRAFT_ACTION },
+{ WWT_PUSHIMGBTN, RESIZE_LR,    14, 232, 249,  14,  31, 0x2AB,    STR_A029_CENTER_MAIN_VIEW_ON_AIRCRAFT },
+{ WWT_PUSHIMGBTN, RESIZE_LR,    14, 232, 249,  32,  49, 0x2AF,    STR_A02A_SEND_AIRCRAFT_TO_HANGAR },
+{ WWT_PUSHIMGBTN, RESIZE_LR,    14, 232, 249,  50,  67, 0x2B4,    STR_A03B_REFIT_AIRCRAFT_TO_CARRY },
+{ WWT_PUSHIMGBTN, RESIZE_LR,    14, 232, 249,  68,  85, 0x2B2,    STR_A028_SHOW_AIRCRAFT_S_ORDERS },
+{ WWT_PUSHIMGBTN, RESIZE_LR,    14, 232, 249,  86, 103, 0x2B3,    STR_A02B_SHOW_AIRCRAFT_DETAILS },
+{ WWT_PANEL,      RESIZE_LRB,   14, 232, 249, 104, 103, 0x0,      STR_NULL },
+{ WWT_RESIZEBOX,  RESIZE_LRTB,  14, 238, 249, 104, 115, 0x0,      STR_NULL },
+{ WIDGETS_END }
 };
 
 static void AircraftViewWndProc(Window *w, WindowEvent *e)
@@ -593,7 +502,8 @@ static void AircraftViewWndProc(Window *w, WindowEvent *e)
 		DrawWindowWidgets(w);
 
 		/* draw the flag */
-		DrawSprite((v->vehstatus & VS_STOPPED) ? 0xC12  : 0xC13, 2, 105);
+		DrawSprite(v->vehstatus & VS_STOPPED ? 0xC12 : 0xC13, 2,
+			w->widget[5].top + 1);
 
 		if (v->vehstatus & VS_CRASHED) {
 			str = STR_8863_CRASHED;
@@ -627,7 +537,8 @@ static void AircraftViewWndProc(Window *w, WindowEvent *e)
 			}
 		}
 
-		DrawStringCentered(125, 105, str, 0);
+		DrawStringCentered((w->widget[5].right - w->widget[5].left) / 2,
+			w->widget[5].top + 1, str, 0);
 		DrawWindowViewport(w);
 	} break;
 
@@ -656,6 +567,13 @@ static void AircraftViewWndProc(Window *w, WindowEvent *e)
 		}
 	} break;
 
+	case WE_RESIZE:
+		w->viewport->width  += e->sizing.diff.x;
+		w->viewport->height += e->sizing.diff.y;
+		w->viewport->virtual_width  += e->sizing.diff.x;
+		w->viewport->virtual_height += e->sizing.diff.y;
+		break;
+
 	case WE_DESTROY:
 		DeleteWindowById(WC_VEHICLE_ORDERS, w->window_number);
 		DeleteWindowById(WC_VEHICLE_REFIT, w->window_number);
@@ -668,7 +586,7 @@ static void AircraftViewWndProc(Window *w, WindowEvent *e)
 static const WindowDesc _aircraft_view_desc = {
 	-1,-1, 250, 116,
 	WC_VEHICLE_VIEW ,0,
-	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET | WDF_UNCLICK_BUTTONS | WDF_STICKY_BUTTON,
+	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET | WDF_UNCLICK_BUTTONS | WDF_STICKY_BUTTON | WDF_RESIZABLE,
 	_aircraft_view_widgets,
 	AircraftViewWndProc
 };
@@ -1011,7 +929,7 @@ static void PlayerAircraftWndProc(Window *w, WindowEvent *e)
 				w->widget[1].unkA = STR_A009_AIRCRAFT;
 			} else {
 				/* Station Name -- (###) Aircraft */
-				SetDParam(0, GetStation(station)->index);
+				SetDParam(0, station);
 				SetDParam(1, w->vscroll.count);
 				w->widget[1].unkA = STR_SCHEDULED_AIRCRAFT;
 			}
@@ -1020,8 +938,7 @@ static void PlayerAircraftWndProc(Window *w, WindowEvent *e)
 		/* draw sorting criteria string */
 		DrawString(85, 15, _vehicle_sort_listing[vl->sort_type], 0x10);
 		/* draw arrow pointing up/down for ascending/descending sorting */
-		DoDrawString(
-			vl->flags & VL_DESC ? "\xAA" : "\xA0", 69, 15, 0x10);
+		DoDrawString(vl->flags & VL_DESC ? "\xAA" : "\xA0", 69, 15, 0x10);
 
 		max = min(w->vscroll.pos + w->vscroll.cap, vl->list_length);
 		for (i = w->vscroll.pos; i < max; ++i) {
@@ -1060,6 +977,7 @@ static void PlayerAircraftWndProc(Window *w, WindowEvent *e)
 		case 3: /* Flip sorting method ascending/descending */
 			vl->flags ^= VL_DESC;
 			vl->flags |= VL_RESORT;
+			_sorting.aircraft.order = !!(vl->flags & VL_DESC);
 			SetWindowDirty(w);
 			break;
 
@@ -1122,18 +1040,19 @@ static void PlayerAircraftWndProc(Window *w, WindowEvent *e)
 			// value has changed -> resort
 			vl->flags |= VL_RESORT;
 			vl->sort_type = e->dropdown.index;
+			_sorting.aircraft.criteria = vl->sort_type;
 
 			// enable 'Sort By' if a sorter criteria is chosen
 			if (vl->sort_type != SORT_BY_UNSORTED)
-				w->disabled_state &= ~(1 << 3);
+				CLRBIT(w->disabled_state, 3);
 		}
 		SetWindowDirty(w);
 		break;
 
 	case WE_CREATE: /* set up resort timer */
 		vl->sort_list = NULL;
-		vl->flags = VL_REBUILD;
-		vl->sort_type = SORT_BY_UNSORTED;
+		vl->flags = VL_REBUILD | (_sorting.aircraft.order << (VL_DESC - 1));
+		vl->sort_type = _sorting.aircraft.criteria;
 		vl->resort_timer = DAY_TICKS * PERIODIC_RESORT_DAYS;
 		break;
 
