@@ -1,34 +1,44 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "table/strings.h"
+#include "map.h"
+#include "tile.h"
 #include "vehicle.h"
 #include "command.h"
 #include "news.h"
 #include "gfx.h"
 #include "station.h"
-#include "city.h"
+#include "town.h"
 #include "industry.h"
 #include "player.h"
 #include "airport_movement.h"
+#include "sound.h"
 
-static void DisasterClearSquare(uint tile)
+static void DisasterClearSquare(TileIndex tile)
 {
-	int type;
-		
 	if (!EnsureNoVehicle(tile))
 		return;
 
-	type = _map_type_and_height[tile] >> 4;
+	switch (GetTileType(tile)) {
+		case MP_RAILWAY:
+			if (IS_HUMAN_PLAYER(_map_owner[tile])) DoClearSquare(tile);
+			break;
 
-	if (type == MP_RAILWAY) {
-		if (IS_HUMAN_PLAYER(_map_owner[tile]))
+		case MP_HOUSE: {
+			byte p = _current_player;
+			_current_player = OWNER_NONE;
+			DoCommandByTile(tile, 0, 0, DC_EXEC, CMD_LANDSCAPE_CLEAR);
+			_current_player = p;
+			break;
+		}
+
+		case MP_TREES:
+		case MP_CLEAR:
 			DoClearSquare(tile);
-	} else if (type == MP_HOUSE) {
-		byte p = _current_player;
-		_current_player = OWNER_NONE;
-		DoCommandByTile(tile, 0, 0, DC_EXEC, CMD_LANDSCAPE_CLEAR);
-		_current_player = p;
-	} else if (type == MP_TREES || type == MP_CLEAR) {
-		DoClearSquare(tile);
+			break;
+
+		default:
+			break;
 	}
 }
 
@@ -77,7 +87,9 @@ static void InitializeDisasterVehicle(Vehicle *v, int x, int y, byte z, byte dir
 	v->owner = OWNER_NONE;
 	v->vehstatus = VS_UNCLICKABLE;
 	v->u.disaster.image_override = 0;
-	v->next_order = 0;
+	v->current_order.type = OT_NOTHING;
+	v->current_order.flags = 0;
+	v->current_order.station = 0;
 
 	DisasterVehicleUpdateImage(v);
 	VehiclePositionChanged(v);
@@ -107,7 +119,7 @@ static void SetDisasterVehiclePos(Vehicle *v, int x, int y, byte z)
 
 	if ( (u=v->next) != NULL) {
 		BeginVehicleMove(u);
-	
+
 		u->x_pos = x;
 		u->y_pos = yt = y - 1 - (max(z - GetSlopeZ(x, y-1), 0) >> 3);
 		u->z_pos = GetSlopeZ(x,yt);
@@ -139,56 +151,58 @@ static void DisasterTick_Zeppeliner(Vehicle *v)
 
 	++v->tick_counter;
 
-	if (v->next_order < 2) {
+	if (v->current_order.station < 2) {
 		if (v->tick_counter&1)
 			return;
 
 		GetNewVehiclePos(v, &gp);
-		
+
 		SetDisasterVehiclePos(v, gp.x, gp.y, v->z_pos);
 
-		if (v->next_order == 1) {
+		if (v->current_order.station == 1) {
 			if (++v->age == 38) {
-				v->next_order = 2;
+				v->current_order.station = 2;
 				v->age = 0;
 			}
 
 			if ((v->tick_counter&7)==0) {
-				CreateEffectVehicleRel(v, 0, -17, 2, EV_SMOKE_3);
+				CreateEffectVehicleRel(v, 0, -17, 2, EV_SMOKE);
 			}
-		} else if (v->next_order == 0) {
+		} else if (v->current_order.station == 0) {
 			tile = v->tile; /**/
 
-			if (IS_TILETYPE(tile, MP_STATION) &&
+			if (IsValidTile(tile) &&
+					IsTileType(tile, MP_STATION) &&
 				IS_BYTE_INSIDE(_map5[tile], 8, 0x43) &&
 				IS_HUMAN_PLAYER(_map_owner[tile])) {
-				
-				v->next_order = 1;
+
+				v->current_order.station = 1;
 				v->age = 0;
 
-				SET_DPARAM16(0, _map2[tile]);
+				SetDParam(0, _map2[tile]);
 				AddNewsItem(STR_B000_ZEPPELIN_DISASTER_AT,
 					NEWS_FLAGS(NM_THIN, NF_VIEWPORT|NF_VEHICLE, NT_ACCIDENT, 0),
 					v->index,
 					0);
 			}
 		}
-		if (v->y_pos >= (TILES_Y+9) * 16 - 1)
+		if (v->y_pos >= ((int)MapSizeY() + 9) * 16 - 1)
 			DeleteDisasterVeh(v);
 		return;
-	} 
-	
-	if (v->next_order > 2) {
+	}
+
+	if (v->current_order.station > 2) {
 		if (++v->age <= 13320)
 			return;
-		
+
 		tile = v->tile; /**/
 
-		if (IS_TILETYPE(tile, MP_STATION) &&
+		if (IsValidTile(tile) &&
+				IsTileType(tile, MP_STATION) &&
 			IS_BYTE_INSIDE(_map5[tile], 8, 0x43) &&
 			IS_HUMAN_PLAYER(_map_owner[tile])) {
 
-			st = DEREF_STATION(_map2[tile]);
+			st = GetStation(_map2[tile]);
 			CLRBITS(st->airport_flags, RUNWAY_IN_block);
 		}
 
@@ -205,8 +219,8 @@ static void DisasterTick_Zeppeliner(Vehicle *v)
 	SetDisasterVehiclePos(v, x, y, z);
 
 	if (++v->age == 1) {
-		CreateEffectVehicleRel(v, 0, 7, 8, EV_CRASHED_SMOKE);
-		SndPlayVehicleFx(0x10, v);
+		CreateEffectVehicleRel(v, 0, 7, 8, EV_EXPLOSION_LARGE);
+		SndPlayVehicleFx(SND_12_EXPLOSION, v);
 		v->u.disaster.image_override = 0xF42;
 	} else if (v->age == 70) {
 		v->u.disaster.image_override = 0xF43;
@@ -218,19 +232,20 @@ static void DisasterTick_Zeppeliner(Vehicle *v)
 				-7 + (r&0xF),
 				-7 + (r>>4&0xF),
 				5 + (r>>8&0x7),
-				EV_DEMOLISH);
+				EV_EXPLOSION_SMALL);
 		}
 	} else if (v->age == 350) {
-		v->next_order = 3;
+		v->current_order.station = 3;
 		v->age = 0;
 	}
 
 	tile = v->tile;/**/
-	if (IS_TILETYPE(tile, MP_STATION) &&
+	if (IsValidTile(tile) &&
+			IsTileType(tile, MP_STATION) &&
 		IS_BYTE_INSIDE(_map5[tile], 8, 0x43) &&
 		IS_HUMAN_PLAYER(_map_owner[tile])) {
 
-		st = DEREF_STATION(_map2[tile]);
+		st = GetStation(_map2[tile]);
 		SETBITS(st->airport_flags, RUNWAY_IN_block);
 	}
 }
@@ -245,11 +260,11 @@ static void DisasterTick_UFO(Vehicle *v)
 	byte z;
 
 	v->u.disaster.image_override = (++v->tick_counter & 8) ? 0xF45 : 0xF44;
-	
-	if (v->next_order == 0) {
-// fly around randomly 
-		int x = GET_TILE_X(v->dest_tile)*16;
-		int y = GET_TILE_Y(v->dest_tile)*16;
+
+	if (v->current_order.station == 0) {
+// fly around randomly
+		int x = TileX(v->dest_tile) * 16;
+		int y = TileY(v->dest_tile) * 16;
 		if (abs(x - v->x_pos) +	abs(y - v->y_pos) >= 16) {
 			v->direction = GetDirectionTowards(v, x, y);
 			GetNewVehiclePos(v, &gp);
@@ -260,7 +275,7 @@ static void DisasterTick_UFO(Vehicle *v)
 			v->dest_tile = TILE_MASK(Random());
 			return;
 		}
-		v->next_order = 1;
+		v->current_order.station = 1;
 
 		FOR_ALL_VEHICLES(u) {
 			if (u->type == VEH_Road && IS_HUMAN_PLAYER(u->owner)) {
@@ -273,7 +288,7 @@ static void DisasterTick_UFO(Vehicle *v)
 		DeleteDisasterVeh(v);
 	} else {
 // target a vehicle
-		u = &_vehicles[v->dest_tile];
+		u = GetVehicle(v->dest_tile);
 		if (u->type != VEH_Road) {
 			DeleteDisasterVeh(v);
 			return;
@@ -288,11 +303,11 @@ static void DisasterTick_UFO(Vehicle *v)
 
 		v->direction = GetDirectionTowards(v, u->x_pos, u->y_pos);
 		GetNewVehiclePos(v, &gp);
-		
+
 		z = v->z_pos;
 		if (dist <= 16 && z > u->z_pos) z--;
 		SetDisasterVehiclePos(v, gp.x, gp.y, z);
-			
+
 		if (z <= u->z_pos && (u->vehstatus&VS_HIDDEN)==0) {
 			v->age++;
 			if (u->u.road.crashed_ctr == 0) {
@@ -308,8 +323,8 @@ static void DisasterTick_UFO(Vehicle *v)
 
 // destroy?
 		if (v->age > 50) {
-			CreateEffectVehicleRel(v, 0, 7, 8, EV_CRASHED_SMOKE);
-			SndPlayVehicleFx(0x10, v);
+			CreateEffectVehicleRel(v, 0, 7, 8, EV_EXPLOSION_LARGE);
+			SndPlayVehicleFx(SND_12_EXPLOSION, v);
 			DeleteDisasterVeh(v);
 		}
 	}
@@ -318,10 +333,9 @@ static void DisasterTick_UFO(Vehicle *v)
 static void DestructIndustry(Industry *i)
 {
 	uint tile;
-	byte index = i - _industries;
 
-	for(tile=0; tile != TILES_X*TILES_Y; tile++) {
-		if (IS_TILETYPE(tile, MP_INDUSTRY) &&	_map2[tile] == index) {
+	for(tile=0; tile != MapSize(); tile++) {
+		if (IsTileType(tile, MP_INDUSTRY) &&	_map2[tile] == i->index) {
 			_map_owner[tile] = 0;
 			MarkTileDirtyByTile(tile);
 		}
@@ -332,10 +346,11 @@ static void DestructIndustry(Industry *i)
 static void DisasterTick_2(Vehicle *v)
 {
 	GetNewVehiclePosResult gp;
-	
+
 	v->tick_counter++;
-	v->u.disaster.image_override = (v->next_order == 1 && v->tick_counter&4) ? 0xF4F : 0;
-	
+	v->u.disaster.image_override =
+		(v->current_order.station == 1 && v->tick_counter&4) ? 0xF4F : 0;
+
 	GetNewVehiclePos(v, &gp);
 	SetDisasterVehiclePos(v, gp.x, gp.y, v->z_pos);
 
@@ -344,37 +359,37 @@ static void DisasterTick_2(Vehicle *v)
 		return;
 	}
 
-	if (v->next_order == 2) {
+	if (v->current_order.station == 2) {
 		if (!(v->tick_counter&3)) {
-			Industry *i = DEREF_INDUSTRY(v->dest_tile);
-			int x = GET_TILE_X(i->xy)*16;
-			int y = GET_TILE_Y(i->xy)*16;
+			Industry *i = GetIndustry(v->dest_tile);
+			int x = TileX(i->xy) * 16;
+			int y = TileY(i->xy) * 16;
 			uint32 r = Random();
 
 			CreateEffectVehicleAbove(
 				x + (r & 0x3F),
 				y + (r >> 6 & 0x3F),
 				(r >> 12 & 0xF),
-				EV_DEMOLISH);
+				EV_EXPLOSION_SMALL);
 
 			if (++v->age >= 55)
-				v->next_order = 3;
+				v->current_order.station = 3;
 		}
-	} else if (v->next_order == 1) {
+	} else if (v->current_order.station == 1) {
 		if (++v->age == 112) {
 			Industry *i;
 
-			v->next_order = 2;
+			v->current_order.station = 2;
 			v->age = 0;
 
-			i = DEREF_INDUSTRY(v->dest_tile);
+			i = GetIndustry(v->dest_tile);
 			DestructIndustry(i);
 
-			SET_DPARAM16(0, i->city->index);
+			SetDParam(0, i->town->index);
 			AddNewsItem(STR_B002_OIL_REFINERY_EXPLOSION, NEWS_FLAGS(NM_THIN,NF_VIEWPORT|NF_TILE,NT_ACCIDENT,0), i->xy, 0);
-			SndPlayTileFx(0x10, i->xy);
+			SndPlayTileFx(SND_12_EXPLOSION, i->xy);
 		}
-	} else if (v->next_order == 0) {
+	} else if (v->current_order.station == 0) {
 		int x,y;
 		uint tile;
 		int ind;
@@ -382,17 +397,17 @@ static void DisasterTick_2(Vehicle *v)
 		x = v->x_pos - 15*16;
 		y = v->y_pos;
 
-		if ( (uint)x > (TILES_X-1) * 16-1)
+		if ( (uint)x > MapMaxX() * 16-1)
 			return;
 
 		tile = TILE_FROM_XY(x,y);
-		if (!IS_TILETYPE(tile, MP_INDUSTRY))	
+		if (!IsTileType(tile, MP_INDUSTRY))
 			return;
 
 		v->dest_tile = ind = _map2[tile];
 
-		if (DEREF_INDUSTRY(ind)->type == IT_OIL_REFINERY) {
-			v->next_order = 1;
+		if (GetIndustry(ind)->type == IT_OIL_REFINERY) {
+			v->current_order.station = 1;
 			v->age = 0;
 		}
 	}
@@ -404,47 +419,48 @@ static void DisasterTick_3(Vehicle *v)
 	GetNewVehiclePosResult gp;
 
 	v->tick_counter++;
-	v->u.disaster.image_override = (v->next_order == 1 && v->tick_counter&4) ? 0xF53 : 0;
+	v->u.disaster.image_override =
+		(v->current_order.station == 1 && v->tick_counter&4) ? 0xF53 : 0;
 
 	GetNewVehiclePos(v, &gp);
 	SetDisasterVehiclePos(v, gp.x, gp.y, v->z_pos);
 
-	if (gp.x > TILES_X * 16 + 9*16 - 1) {
+	if (gp.x > (int)MapSizeX() * 16 + 9*16 - 1) {
 		DeleteDisasterVeh(v);
 		return;
 	}
 
-	if (v->next_order == 2) {
+	if (v->current_order.station == 2) {
 		if (!(v->tick_counter&3)) {
-			Industry *i = DEREF_INDUSTRY(v->dest_tile);
-			int x = GET_TILE_X(i->xy)*16;
-			int y = GET_TILE_Y(i->xy)*16;
+			Industry *i = GetIndustry(v->dest_tile);
+			int x = TileX(i->xy) * 16;
+			int y = TileY(i->xy) * 16;
 			uint32 r = Random();
 
 			CreateEffectVehicleAbove(
 				x + (r & 0x3F),
 				y + (r >> 6 & 0x3F),
 				(r >> 12 & 0xF),
-				EV_DEMOLISH);
+				EV_EXPLOSION_SMALL);
 
 			if (++v->age >= 55)
-				v->next_order = 3;
+				v->current_order.station = 3;
 		}
-	} else if (v->next_order == 1) {
+	} else if (v->current_order.station == 1) {
 		if (++v->age == 112) {
 			Industry *i;
 
-			v->next_order = 2;
+			v->current_order.station = 2;
 			v->age = 0;
 
-			i = DEREF_INDUSTRY(v->dest_tile);
+			i = GetIndustry(v->dest_tile);
 			DestructIndustry(i);
 
-			SET_DPARAM16(0, i->city->index);
+			SetDParam(0, i->town->index);
 			AddNewsItem(STR_B003_FACTORY_DESTROYED_IN_SUSPICIOUS, NEWS_FLAGS(NM_THIN,NF_VIEWPORT|NF_TILE,NT_ACCIDENT,0), i->xy, 0);
-			SndPlayTileFx(0x10, i->xy);
+			SndPlayTileFx(SND_12_EXPLOSION, i->xy);
 		}
-	} else if (v->next_order == 0) {
+	} else if (v->current_order.station == 0) {
 		int x,y;
 		uint tile;
 		int ind;
@@ -452,17 +468,17 @@ static void DisasterTick_3(Vehicle *v)
 		x = v->x_pos - 15*16;
 		y = v->y_pos;
 
-		if ( (uint)x > (TILES_X-1) * 16-1)
+		if ( (uint)x > MapMaxX() * 16-1)
 			return;
 
 		tile = TILE_FROM_XY(x,y);
-		if (!IS_TILETYPE(tile, MP_INDUSTRY))	
+		if (!IsTileType(tile, MP_INDUSTRY))
 			return;
 
 		v->dest_tile = ind = _map2[tile];
 
-		if (DEREF_INDUSTRY(ind)->type == IT_FACTORY) {
-			v->next_order = 1;
+		if (GetIndustry(ind)->type == IT_FACTORY) {
+			v->current_order.station = 1;
 			v->age = 0;
 		}
 	}
@@ -489,14 +505,14 @@ static void DisasterTick_4(Vehicle *v)
 	GetNewVehiclePosResult gp;
 	byte z;
 	Vehicle *u,*w;
-	City *city;
+	Town *t;
 	uint tile,tile_org;
 
 	v->tick_counter++;
 
-	if (v->next_order == 1) {
-		int x = GET_TILE_X(v->dest_tile)*16 + 8;
-		int y = GET_TILE_Y(v->dest_tile)*16 + 8;
+	if (v->current_order.station == 1) {
+		int x = TileX(v->dest_tile) * 16 + 8;
+		int y = TileY(v->dest_tile) * 16 + 8;
 		if (abs(v->x_pos - x) + abs(v->y_pos - y) >= 8) {
 			v->direction = GetDirectionTowards(v, x, y);
 
@@ -511,7 +527,7 @@ static void DisasterTick_4(Vehicle *v)
 			return;
 		}
 
-		v->next_order = 2;
+		v->current_order.station = 2;
 
 		FOR_ALL_VEHICLES(u) {
 			if (u->type == VEH_Train || u->type == VEH_Road) {
@@ -522,8 +538,8 @@ static void DisasterTick_4(Vehicle *v)
 			}
 		}
 
-		city = ClosestCityFromTile(v->dest_tile, (uint)-1);
-		SET_DPARAM16(0, city->index);
+		t = ClosestTownFromTile(v->dest_tile, (uint)-1);
+		SetDParam(0, t->index);
 		AddNewsItem(STR_B004_UFO_LANDS_NEAR,
 			NEWS_FLAGS(NM_THIN, NF_VIEWPORT|NF_TILE, NT_ACCIDENT, 0),
 			v->tile,
@@ -545,10 +561,10 @@ static void DisasterTick_4(Vehicle *v)
 		u->next = w;
 		InitializeDisasterVehicle(w, -6*16, v->y_pos, 0, 5, 12);
 		w->vehstatus |= VS_DISASTER;
-	} else if (v->next_order < 1) {
-		
-		int x = GET_TILE_X(v->dest_tile)*16;
-		int y = GET_TILE_Y(v->dest_tile)*16;
+	} else if (v->current_order.station < 1) {
+
+		int x = TileX(v->dest_tile) * 16;
+		int y = TileY(v->dest_tile) * 16;
 		if (abs(x - v->x_pos) +	abs(y - v->y_pos) >= 16) {
 			v->direction = GetDirectionTowards(v, x, y);
 			GetNewVehiclePos(v, &gp);
@@ -560,11 +576,11 @@ static void DisasterTick_4(Vehicle *v)
 			v->dest_tile = TILE_MASK(Random());
 			return;
 		}
-		v->next_order = 1;
+		v->current_order.station = 1;
 
 		tile_org = tile = TILE_MASK(Random());
 		do {
-			if (IS_TILETYPE(tile, MP_RAILWAY) &&
+			if (IsTileType(tile, MP_RAILWAY) &&
 					(_map5[tile]&~3)!=0xC0 &&	IS_HUMAN_PLAYER(_map_owner[tile]))
 						break;
 			tile = TILE_MASK(tile+1);
@@ -587,19 +603,19 @@ static void DisasterTick_4b(Vehicle *v)
 	GetNewVehiclePos(v, &gp);
 	SetDisasterVehiclePos(v, gp.x, gp.y, v->z_pos);
 
-	if (gp.x > TILES_X * 16 + 9*16 - 1) {
+	if (gp.x > (int)MapSizeX() * 16 + 9*16 - 1) {
 		DeleteDisasterVeh(v);
 		return;
 	}
 
-	if (v->next_order == 0) {
-		u = &_vehicles[v->u.disaster.unk2];	
+	if (v->current_order.station == 0) {
+		u = GetVehicle(v->u.disaster.unk2);
 		if (abs(v->x_pos - u->x_pos) > 16)
 			return;
-		v->next_order = 1;
+		v->current_order.station = 1;
 
-		CreateEffectVehicleRel(u, 0, 7, 8, EV_CRASHED_SMOKE);
-		SndPlayVehicleFx(0x10, u);
+		CreateEffectVehicleRel(u, 0, 7, 8, EV_EXPLOSION_LARGE);
+		SndPlayVehicleFx(SND_12_EXPLOSION, u);
 
 		DeleteDisasterVeh(u);
 
@@ -609,7 +625,7 @@ static void DisasterTick_4b(Vehicle *v)
 				v->x_pos-32+(r&0x3F),
 				v->y_pos-32+(r>>5&0x3F),
 				0,
-				EV_DEMOLISH);
+				EV_EXPLOSION_SMALL);
 		}
 
 		BEGIN_TILE_LOOP(tile,6,6,v->tile - TILE_XY(3,3))
@@ -639,9 +655,9 @@ static void DisasterTick_5_and_6(Vehicle *v)
 	if (!(v->tick_counter&1))
 		return;
 
-	tile = v->tile + _tileoffs_by_dir[v->direction >> 1];
+	tile = v->tile + TileOffsByDir(v->direction >> 1);
 	if (IsValidTile(tile) &&
-			(r=GetTileTrackStatus(tile,4),(byte)(r+(r >> 8)) == 0x3F) &&
+			(r=GetTileTrackStatus(tile,TRANSPORT_WATER),(byte)(r+(r >> 8)) == 0x3F) &&
 			!CHANCE16(1,90)) {
 		GetNewVehiclePos(v, &gp);
 		SetDisasterVehiclePos(v, gp.x, gp.y, v->z_pos);
@@ -682,10 +698,10 @@ void OnNewDay_DisasterVehicle(Vehicle *v)
 	// not used
 }
 
-typedef void DisasterInitProc();
+typedef void DisasterInitProc(void);
 
 // Zeppeliner which crashes on a small airport
-static void Disaster0_Init() 
+static void Disaster0_Init(void)
 {
 	Vehicle *v = ForceAllocateSpecialVehicle(), *u;
 	Station *st;
@@ -694,16 +710,15 @@ static void Disaster0_Init()
 	if (v == NULL)
 		return;
 
-	for(st=_stations;;) {
-		if (st->xy && st->airport_tile != 0 && 
+	/* Pick a random place, unless we find
+	    a small airport */
+	x = TileX(Random()) * 16 + 8;
+
+	FOR_ALL_STATIONS(st) {
+		if (st->xy && st->airport_tile != 0 &&
 				st->airport_type <= 1 &&
 				IS_HUMAN_PLAYER(st->owner)) {
-			x = (GET_TILE_X(st->xy) + 2) * 16;
-			break;
-		}
-
-		if (++st == endof(_stations)) {
-			x = (GET_TILE_X(Random())) * 16 + 8;
+			x = (TileX(st->xy) + 2) * 16;
 			break;
 		}
 	}
@@ -714,12 +729,12 @@ static void Disaster0_Init()
 	u = ForceAllocateSpecialVehicle();
 	if (u != NULL) {
 		v->next = u;
-		InitializeDisasterVehicle(u,x,0,0,3,1); 
+		InitializeDisasterVehicle(u, x, 0, 0, 3, 1);
 		u->vehstatus |= VS_DISASTER;
 	}
 }
 
-static void Disaster1_Init() 
+static void Disaster1_Init(void)
 {
 	Vehicle *v = ForceAllocateSpecialVehicle(), *u;
 	int x;
@@ -727,30 +742,32 @@ static void Disaster1_Init()
 	if (v == NULL)
 		return;
 
-	x = (GET_TILE_X(Random())) * 16 + 8;
+	x = TileX(Random()) * 16 + 8;
 
 	InitializeDisasterVehicle(v, x, 0, 135, 3, 2);
-	v->dest_tile = TILE_XY(TILES_X/2,TILES_Y/2);
+	v->dest_tile = TILE_XY(MapSizeX() / 2, MapSizeY() / 2);
 	v->age = 0;
 
 	// Allocate shadow too?
 	u = ForceAllocateSpecialVehicle();
 	if (u != NULL) {
 		v->next = u;
-		InitializeDisasterVehicle(u,x,0,0,3,3); 
+		InitializeDisasterVehicle(u,x,0,0,3,3);
 		u->vehstatus |= VS_DISASTER;
 	}
 }
 
-static void Disaster2_Init() 
+static void Disaster2_Init(void)
 {
 	Industry *i, *found;
 	Vehicle *v,*u;
 	int x,y;
 
-	for(found=NULL,i=_industries; i != endof(_industries); i++) {
+	found = NULL;
+
+	FOR_ALL_INDUSTRIES(i) {
 		if (i->xy != 0 &&
-				i->type == IT_OIL_REFINERY && 
+				i->type == IT_OIL_REFINERY &&
 				(found==NULL || CHANCE16(1,2))) {
 			found = i;
 		}
@@ -763,28 +780,30 @@ static void Disaster2_Init()
 	if (v == NULL)
 		return;
 
-	x = (TILES_X+9) * 16 - 1;
-	y = GET_TILE_Y(found->xy)*16 + 37;
+	x = (MapSizeX() + 9) * 16 - 1;
+	y = TileY(found->xy) * 16 + 37;
 
 	InitializeDisasterVehicle(v,x,y, 135,1,4);
 
 	u = ForceAllocateSpecialVehicle();
 	if (u != NULL) {
 		v->next = u;
-		InitializeDisasterVehicle(u,x,y,0,3,5); 
+		InitializeDisasterVehicle(u,x,y,0,3,5);
 		u->vehstatus |= VS_DISASTER;
 	}
 }
 
-static void Disaster3_Init() 
+static void Disaster3_Init(void)
 {
 	Industry *i, *found;
 	Vehicle *v,*u,*w;
 	int x,y;
 
-	for(found=NULL,i=_industries; i != endof(_industries); i++) {
+	found = NULL;
+
+	FOR_ALL_INDUSTRIES(i) {
 		if (i->xy != 0 &&
-				i->type == IT_FACTORY && 
+				i->type == IT_FACTORY &&
 				(found==NULL || CHANCE16(1,2))) {
 			found = i;
 		}
@@ -798,25 +817,25 @@ static void Disaster3_Init()
 		return;
 
 	x = -16 * 16;
-	y = GET_TILE_Y(found->xy)*16 + 37;
+	y = TileY(found->xy) * 16 + 37;
 
 	InitializeDisasterVehicle(v,x,y, 135,5,6);
 
 	u = ForceAllocateSpecialVehicle();
 	if (u != NULL) {
 		v->next = u;
-		InitializeDisasterVehicle(u,x,y,0,5,7); 
+		InitializeDisasterVehicle(u,x,y,0,5,7);
 		u->vehstatus |= VS_DISASTER;
 
 		w = ForceAllocateSpecialVehicle();
 		if (w != NULL) {
 			u->next = w;
-			InitializeDisasterVehicle(w,x,y,140,5,8); 
+			InitializeDisasterVehicle(w,x,y,140,5,8);
 		}
 	}
 }
 
-static void Disaster4_Init() 
+static void Disaster4_Init(void)
 {
 	Vehicle *v = ForceAllocateSpecialVehicle(), *u;
 	int x,y;
@@ -824,24 +843,24 @@ static void Disaster4_Init()
 	if (v == NULL)
 		return;
 
-	x = (GET_TILE_X(Random())) * 16 + 8;
+	x = TileX(Random()) * 16 + 8;
 
-	y = (TILES_X-1)*16-1;
+	y = MapMaxX() * 16 - 1;
 	InitializeDisasterVehicle(v, x, y, 135, 7, 9);
-	v->dest_tile = TILE_XY(TILES_X/2,TILES_Y/2);
+	v->dest_tile = TILE_XY(MapSizeX() / 2, MapSizeY() / 2);
 	v->age = 0;
 
 	// Allocate shadow too?
 	u = ForceAllocateSpecialVehicle();
 	if (u != NULL) {
 		v->next = u;
-		InitializeDisasterVehicle(u,x,y,0,7,10); 
+		InitializeDisasterVehicle(u,x,y,0,7,10);
 		u->vehstatus |= VS_DISASTER;
 	}
 }
 
 // Submarine type 1
-static void Disaster5_Init() 
+static void Disaster5_Init(void)
 {
 	Vehicle *v = ForceAllocateSpecialVehicle();
 	int x,y;
@@ -852,17 +871,17 @@ static void Disaster5_Init()
 		return;
 
 	r = Random();
-	x = (GET_TILE_X(r)) * 16 + 8;
+	x = TileX(r) * 16 + 8;
 
 	y = 8;
 	dir = 3;
-	if (r & 0x80000000) { y = (TILES_X-1) * 16 - 8 - 1; dir = 7; }
+	if (r & 0x80000000) { y = MapMaxX() * 16 - 8 - 1; dir = 7; }
 	InitializeDisasterVehicle(v, x, y, 0, dir,13);
 	v->age = 0;
 }
 
 // Submarine type 2
-static void Disaster6_Init() 
+static void Disaster6_Init(void)
 {
 	Vehicle *v = ForceAllocateSpecialVehicle();
 	int x,y;
@@ -873,32 +892,32 @@ static void Disaster6_Init()
 		return;
 
 	r = Random();
-	x = (GET_TILE_X(r)) * 16 + 8;
+	x = TileX(r) * 16 + 8;
 
 	y = 8;
 	dir = 3;
-	if (r & 0x80000000) { y = (TILES_X-1) * 16 - 8 - 1;	dir = 7; }
+	if (r & 0x80000000) { y = MapMaxX() * 16 - 8 - 1; dir = 7; }
 	InitializeDisasterVehicle(v, x, y, 0, dir,14);
 	v->age = 0;
 }
 
-static void Disaster7_Init() 
+static void Disaster7_Init(void)
 {
 	Industry *i;
 	int maxloop = 15;
 	int index = Random() & 0xF;
-	
+
 	do {
-		for(i=_industries; i != endof(_industries); i++) {
+		FOR_ALL_INDUSTRIES(i) {
 			if (i->xy != 0 && i->type == IT_COAL_MINE	&& --index < 0) {
-				
-				SET_DPARAM16(0, i->city->index);
-				AddNewsItem(STR_B005_COAL_MINE_SUBSIDENCE_LEAVES, 
+
+				SetDParam(0, i->town->index);
+				AddNewsItem(STR_B005_COAL_MINE_SUBSIDENCE_LEAVES,
 					NEWS_FLAGS(NM_THIN,NF_VIEWPORT|NF_TILE,NT_ACCIDENT,0), i->xy + TILE_XY(1,1), 0);
-				
+
 				{
 					uint tile = i->xy;
-					int step = _tileoffs_by_dir[Random() & 3];
+					TileIndexDiff step = TileOffsByDir(Random() & 3);
 					int count = 30;
 					do {
 						DisasterClearSquare(tile);
@@ -939,7 +958,7 @@ static const DisasterYears _dis_years[8] = {
 };
 
 
-void DoDisaster()
+static void DoDisaster(void)
 {
 	byte buf[8];
 	byte year = _cur_year;
@@ -958,12 +977,12 @@ void DoDisaster()
 }
 
 
-static void ResetDisasterDelay()
+static void ResetDisasterDelay(void)
 {
 	_disaster_delay = (int)(Random() & 0x1FF) + 730;
 }
 
-void DisasterDailyLoop()
+void DisasterDailyLoop(void)
 {
 	if (--_disaster_delay != 0)
 		return;
@@ -974,7 +993,8 @@ void DisasterDailyLoop()
 		DoDisaster();
 }
 
-void StartupDisasters() {
+void StartupDisasters(void)
+{
 	ResetDisasterDelay();
 }
 
