@@ -1,5 +1,6 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "strings.h" // XXX InjectDParam()
 #include "table/strings.h"
 #include "map.h"
 #include "news.h"
@@ -107,7 +108,7 @@ int64 CalculateCompanyValue(Player *p) {
 
 	value += p->money64 - p->current_loan; // add real money value
 
-	return max(value, 1);
+	return max64(value, 1);
 }
 
 // if update is set to true, the economy is updated with this score
@@ -122,7 +123,7 @@ int UpdateCompanyRatingAndValue(Player *p, bool update)
 /* Count vehicles */
 	{
 		Vehicle *v;
-		int32 min_profit = MAX_INT;
+		int32 min_profit = score_info[SCORE_MIN_PROFIT].needed;
 		uint num = 0;
 
 		FOR_ALL_VEHICLES(v) {
@@ -141,8 +142,7 @@ int UpdateCompanyRatingAndValue(Player *p, bool update)
 		}
 
 		_score_part[owner][SCORE_VEHICLES] = num;
-
-		if (min_profit != MAX_INT && min_profit > 0)
+		if (min_profit > 0)
 			_score_part[owner][SCORE_MIN_PROFIT] = min_profit;
 	}
 
@@ -259,45 +259,44 @@ int UpdateCompanyRatingAndValue(Player *p, bool update)
 }
 
 // use 255 as new_player to delete the player.
-void ChangeOwnershipOfPlayerItems(byte old_player, byte new_player)
+void ChangeOwnershipOfPlayerItems(PlayerID old_player, PlayerID new_player)
 {
-	byte old = _current_player;
+	PlayerID old = _current_player;
 	_current_player = old_player;
 
-	if (new_player == 255) {
+	if (new_player == OWNER_SPECTATOR) {
 		Subsidy *s;
 
-		for(s=_subsidies; s != endof(_subsidies); s++) {
+		for (s = _subsidies; s != endof(_subsidies); s++) {
 			if (s->cargo_type != 0xff && s->age >= 12) {
-				Station *st = GetStation(s->to);
-				if (st->owner == old_player)
+				if (GetStation(s->to)->owner == old_player)
 					s->cargo_type = 0xff;
 			}
 		}
 	}
 
-	// take care of rating in towns
-	{
-		Town *t;
+	/* Take care of rating in towns */
+	{ Town *t;
+		if (new_player != OWNER_SPECTATOR) {
+			FOR_ALL_TOWNS(t) {
+				/* If a player takes over, give the ratings to that player. */
+				if (IsValidTown(t) && HASBIT(t->have_ratings, old_player)) {
+					if (HASBIT(t->have_ratings, new_player)) {
+						// use max of the two ratings.
+						t->ratings[new_player] = max(t->ratings[new_player], t->ratings[old_player]);
+					} else {
+						SETBIT(t->have_ratings, new_player);
+						t->ratings[new_player] = t->ratings[old_player];
+					}
+				}
 
-		// if a player takes over, give the ratings to that player.
-		if (new_player != 255) {
-			FOR_ALL_TOWNS(t) if (t->xy && HASBIT(t->have_ratings, old_player)) {
-				if (HASBIT(t->have_ratings, new_player)) {
-					// use max of the two ratings.
-					t->ratings[new_player] = max(t->ratings[new_player], t->ratings[old_player]);
-				} else {
-					SETBIT(t->have_ratings, new_player);
-					t->ratings[new_player] = t->ratings[old_player];
+				/* Reset ratings for the town */
+				if (IsValidTown(t)) {
+					t->ratings[old_player] = 500;
+					CLRBIT(t->have_ratings, old_player);
 				}
 			}
 		}
-
-		// reset rating for the towns.
-		FOR_ALL_TOWNS(t) if (t->xy) {
-			t->ratings[old_player] = 500;
-			CLRBIT(t->have_ratings, old_player);
-		}
 	}
 
 	{
@@ -310,20 +309,27 @@ void ChangeOwnershipOfPlayerItems(byte old_player, byte new_player)
 		// Determine Ids for the new vehicles
 		FOR_ALL_VEHICLES(v) {
 			if (v->owner == new_player) {
-				if (v->type == VEH_Train && v->subtype == TS_Front_Engine)
-					num_train++;
-				else if (v->type == VEH_Road)
-					num_road++;
-				else if (v->type == VEH_Ship)
-					num_ship++;
-				else if (v->type == VEH_Aircraft && v->subtype <= 2)
-					num_aircraft++;
+				switch (v->type) {
+					case VEH_Train:
+						if (v->subtype == TS_Front_Engine) num_train++;
+						break;
+					case VEH_Road:
+						num_road++;
+						break;
+					case VEH_Ship:
+						num_ship++;
+						break;
+					case VEH_Aircraft:
+						if (v->subtype <= 2) num_aircraft++;
+						break;
+					default: break;
+				}
 			}
 		}
 
 		FOR_ALL_VEHICLES(v) {
 			if (v->owner == old_player && IS_BYTE_INSIDE(v->type, VEH_Train, VEH_Aircraft+1) ) {
-				if (new_player == 255) {
+				if (new_player == OWNER_SPECTATOR) {
 					DeleteWindowById(WC_VEHICLE_VIEW, v->index);
 					DeleteWindowById(WC_VEHICLE_DETAILS, v->index);
 					DeleteWindowById(WC_VEHICLE_ORDERS, v->index);
@@ -345,16 +351,16 @@ void ChangeOwnershipOfPlayerItems(byte old_player, byte new_player)
 
 	// Change ownership of tiles
 	{
-		uint tile = 0;
+		TileIndex tile = 0;
 		do {
 			ChangeTileOwner(tile, old_player, new_player);
 		} while (++tile != MapSize());
 	}
 
 	// Change color of existing windows
-	if (new_player != 255) {
+	if (new_player != OWNER_SPECTATOR) {
 		Window *w;
-		for(w=_windows; w != _last_window; w++) {
+		for (w = _windows; w != _last_window; w++) {
 			if (w->caption_color == old_player)
 				w->caption_color = new_player;
 		}
@@ -366,7 +372,7 @@ void ChangeOwnershipOfPlayerItems(byte old_player, byte new_player)
 
 		/* Check for shares */
 		FOR_ALL_PLAYERS(p) {
-			for(i = 0; i < 4; i++) {
+			for (i = 0; i < 4; i++) {
 				/* 'Sell' the share if this player has any */
 				if (p->share_owners[i] == _current_player)
 					p->share_owners[i] = 0xFF;
@@ -374,7 +380,7 @@ void ChangeOwnershipOfPlayerItems(byte old_player, byte new_player)
 		}
 		p = DEREF_PLAYER(_current_player);
 		/* Sell all the shares that people have on this company */
-		for(i = 0; i < 4; i++)
+		for (i = 0; i < 4; i++)
 			p->share_owners[i] = 0xFF;
 	}
 
@@ -896,7 +902,7 @@ static void FindSubsidyPassengerRoute(FoundRoute *fr)
 	if (from==to || to->xy == 0 || to->population < 400 || to->pct_pass_transported > 42)
 		return;
 
-	fr->distance = GetTileDist(from->xy, to->xy);
+	fr->distance = DistanceManhattan(from->xy, to->xy);
 }
 
 static void FindSubsidyCargoRoute(FoundRoute *fr)
@@ -937,7 +943,7 @@ static void FindSubsidyCargoRoute(FoundRoute *fr)
 		// Only want big towns
 		if (t->xy == 0 || t->population < 900)
 			return;
-		fr->distance = GetTileDist(i->xy, t->xy);
+		fr->distance = DistanceManhattan(i->xy, t->xy);
 		fr->to = t;
 	} else {
 		// The destination is an industry
@@ -949,7 +955,7 @@ static void FindSubsidyCargoRoute(FoundRoute *fr)
 				cargo != i2->accepts_cargo[1] &&
 				cargo != i2->accepts_cargo[2]))
 			return;
-		fr->distance = GetTileDist(i->xy, i2->xy);
+		fr->distance = DistanceManhattan(i->xy, i2->xy);
 		fr->to = i2;
 	}
 }
@@ -1132,7 +1138,7 @@ static void DeliverGoodsToIndustry(TileIndex xy, byte cargo_type, int num_pieces
 				 == ind->accepts_cargo[1] || cargo_type == ind->accepts_cargo[2]) &&
 				 ind->produced_cargo[0] != 0xFF &&
 				 ind->produced_cargo[0] != cargo_type &&
-				 (t=GetTileDist(ind->xy, xy)) < u) {
+				 (t = DistanceManhattan(ind->xy, xy)) < 2 * u) {
 			u = t;
 			best = ind;
 		}
@@ -1171,7 +1177,7 @@ static bool CheckSubsidised(Station *from, Station *to, byte cargo_type)
 			} else {
 				xy = (GetIndustry(s->from))->xy;
 			}
-			if (GetTileDist1D(xy, from->xy) > 9)
+			if (DistanceMax(xy, from->xy) > 9)
 				continue;
 
 			/* Check distance from dest */
@@ -1181,7 +1187,7 @@ static bool CheckSubsidised(Station *from, Station *to, byte cargo_type)
 				xy = (GetIndustry(s->to))->xy;
 			}
 
-			if (GetTileDist1D(xy, to->xy) > 9)
+			if (DistanceMax(xy, to->xy) > 9)
 				continue;
 
 			/* Found a subsidy, change the values to indicate that it's in use */
@@ -1191,7 +1197,7 @@ static bool CheckSubsidised(Station *from, Station *to, byte cargo_type)
 
 			/* Add a news item */
 			pair = SetupSubsidyDecodeParam(s, 0);
-			INJECT_DPARAM(2);
+			InjectDParam(2);
 
 			p = DEREF_PLAYER(_current_player);
 			SetDParam(0, p->name_1);
@@ -1208,7 +1214,7 @@ static bool CheckSubsidised(Station *from, Station *to, byte cargo_type)
 	return false;
 }
 
-static int32 DeliverGoods(int num_pieces, byte cargo_type, byte source, byte dest, byte days_in_transit)
+static int32 DeliverGoods(int num_pieces, byte cargo_type, uint16 source, uint16 dest, byte days_in_transit)
 {
 	bool subsidised;
 	Station *s_from, *s_to;
@@ -1234,7 +1240,7 @@ static int32 DeliverGoods(int num_pieces, byte cargo_type, byte source, byte des
 	{
 		Town *t = s_to->town;
 		if (cargo_type == CT_FOOD) t->new_act_food += num_pieces;
-		if (cargo_type == CT_STEEL)  t->new_act_water += num_pieces;
+		if (cargo_type == CT_WATER)  t->new_act_water += num_pieces;
 	}
 
 	// Give the goods to the industry.
@@ -1242,7 +1248,7 @@ static int32 DeliverGoods(int num_pieces, byte cargo_type, byte source, byte des
 
 	// Determine profit
 	{
-		int t = GetTileDist(s_from->xy, s_to->xy);
+		int t = DistanceManhattan(s_from->xy, s_to->xy);
 		int r = num_pieces;
 		profit = 0;
 		do {
@@ -1335,7 +1341,7 @@ int LoadUnloadVehicle(Vehicle *v)
 	int unloading_time = 20;
 	Vehicle *u = v;
 	int result = 0;
-	int last_visited;
+	uint16 last_visited;
 	Station *st;
 	GoodsEntry *ge;
 	int t;
@@ -1346,6 +1352,7 @@ int LoadUnloadVehicle(Vehicle *v)
 	assert(v->current_order.type == OT_LOADING);
 
 	v->cur_speed = 0;
+
 	old_player = _current_player;
 	_current_player = v->owner;
 
@@ -1358,8 +1365,10 @@ int LoadUnloadVehicle(Vehicle *v)
 
 		/* unload? */
 		if (v->cargo_count != 0) {
-			if (v->cargo_source != (byte)last_visited && ge->waiting_acceptance & 0x8000) {
+			if (v->cargo_source != last_visited && ge->waiting_acceptance & 0x8000) {
 				// deliver goods to the station
+				st->time_since_unload = 0;
+
 				unloading_time += v->cargo_count; /* TTDBUG: bug in original TTD */
 				profit += DeliverGoods(v->cargo_count, v->cargo_type, v->cargo_source, last_visited, v->cargo_days);
 				result |= 1;
@@ -1376,7 +1385,7 @@ int LoadUnloadVehicle(Vehicle *v)
 					// Goods already waiting at station. Set counters to the worst value.
 					if (v->cargo_days >= ge->enroute_time)
 						ge->enroute_time = v->cargo_days;
-					if ((byte)last_visited != ge->enroute_from)
+					if (last_visited != ge->enroute_from)
 						ge->enroute_from = v->cargo_source;
 				}
 				// Update amount of waiting cargo
@@ -1477,7 +1486,7 @@ int LoadUnloadVehicle(Vehicle *v)
 void PlayersMonthlyLoop(void)
 {
 	PlayersGenStatistics();
-	if (_patches.inflation)
+	if (_patches.inflation && _cur_year < MAX_YEAR_END)
 		AddInflation();
 	PlayersPayInterest();
 	// Reset the _current_player flag
@@ -1524,41 +1533,40 @@ static void DoAcquireCompany(Player *p)
 
 extern int GetAmountOwnedBy(Player *p, byte owner);
 
+/** Acquire shares in an opposing company.
+ * @param x,y unused
+ * @param p1 player to buy the shares from
+ * @param p2 unused
+ */
 int32 CmdBuyShareInCompany(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
 	Player *p;
 	int64 cost;
-	byte *b;
-	int i;
+
+	/* Check if buying shares is allowed (protection against modified clients */
+	if (p1 >= MAX_PLAYERS || !_patches.allow_shares) return CMD_ERROR;
 
 	SET_EXPENSES_TYPE(EXPENSES_OTHER);
 	p = DEREF_PLAYER(p1);
 
-
-	if (_cur_year - p->inaugurated_year < 6) {
-		_error_message = STR_7080_PROTECTED;
-		return CMD_ERROR;
-	}
-
-	/* Check if buying shares is allowed (protection against modified clients */
-	if (!_patches.allow_shares)
-		return CMD_ERROR;
+	/* Protect new companies from hostile takeovers */
+	if (_cur_year - p->inaugurated_year < 6) return_cmd_error(STR_7080_PROTECTED);
 
 	/* Those lines are here for network-protection (clients can be slow) */
-	if (GetAmountOwnedBy(p, OWNER_SPECTATOR) == 0)
-		return 0;
+	if (GetAmountOwnedBy(p, OWNER_SPECTATOR) == 0) return 0;
 
-	/* We can not buy out a real player in networking */
-	if (GetAmountOwnedBy(p, OWNER_SPECTATOR) == 1 && !p->is_ai)
-		return 0;
+	/* We can not buy out a real player (temporarily). TODO: well, enable it obviously */
+	if (GetAmountOwnedBy(p, OWNER_SPECTATOR) == 1 && !p->is_ai) return 0;
 
 	cost = CalculateCompanyValue(p) >> 2;
 	if (flags & DC_EXEC) {
-		b = p->share_owners;
+		int i;
+		byte *b = p->share_owners;
+
 		while (*b != 0xFF) b++; /* share owners is guaranteed to contain at least one 0xFF */
 		*b = _current_player;
 
-		for(i=0;p->share_owners[i] == _current_player;) {
+		for (i = 0; p->share_owners[i] == _current_player;) {
 			if (++i == 4) {
 				p->bankrupt_value = 0;
 				DoAcquireCompany(p);
@@ -1570,29 +1578,31 @@ int32 CmdBuyShareInCompany(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	return cost;
 }
 
+/** Sell shares in an opposing company.
+ * @param x,y unused
+ * @param p1 player to sell the shares from
+ * @param p2 unused
+ */
 int32 CmdSellShareInCompany(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
 	Player *p;
 	int64 cost;
-	byte *b;
+
+	/* Check if buying shares is allowed (protection against modified clients */
+	if (p1 >= MAX_PLAYERS || !_patches.allow_shares) return CMD_ERROR;
 
 	SET_EXPENSES_TYPE(EXPENSES_OTHER);
 	p = DEREF_PLAYER(p1);
 
-	/* Check if selling shares is allowed (protection against modified clients */
-	if (!_patches.allow_shares)
-		return CMD_ERROR;
-
 	/* Those lines are here for network-protection (clients can be slow) */
-	if (GetAmountOwnedBy(p, _current_player) == 0)
-		return 0;
+	if (GetAmountOwnedBy(p, _current_player) == 0) return 0;
 
 	/* adjust it a little to make it less profitable to sell and buy */
 	cost = CalculateCompanyValue(p) >> 2;
 	cost = -(cost - (cost >> 7));
 
 	if (flags & DC_EXEC) {
-		b = p->share_owners;
+		byte *b = p->share_owners;
 		while (*b != _current_player) b++; /* share owners is guaranteed to contain player */
 		*b = 0xFF;
 		InvalidateWindow(WC_COMPANY, (int)p1);
@@ -1600,12 +1610,27 @@ int32 CmdSellShareInCompany(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	return cost;
 }
 
+/** Buy up another company.
+ * When a competing company is gone bankrupt you get the chance to purchase
+ * that company.
+ * @todo currently this only works for AI players
+ * @param x,y unused
+ * @param p1 player/company to buy up
+ * @param p2 unused
+ */
 int32 CmdBuyCompany(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
 	Player *p;
+
+	/* Disable takeovers in multiplayer games */
+	if (p1 >= MAX_PLAYERS || _networking) return CMD_ERROR;
+
 	SET_EXPENSES_TYPE(EXPENSES_OTHER);
 	p = DEREF_PLAYER(p1);
-	if (flags & 1) {
+
+	if (!p->is_ai) return CMD_ERROR;
+
+	if (flags & DC_EXEC) {
 		DoAcquireCompany(p);
 	}
 	return p->bankrupt_value;
@@ -1647,7 +1672,3 @@ const ChunkHandler _economy_chunk_handlers[] = {
 	{ 'SUBS', Save_SUBS,			Load_SUBS, CH_ARRAY},
 	{ 'ECMY', SaveLoad_ECMY, SaveLoad_ECMY, CH_RIFF | CH_LAST},
 };
-
-
-
-
