@@ -1,15 +1,17 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "spritecache.h"
+#include "strings.h"
 #include "gfx.h"
 #include "table/palettes.h"
 #include "hal.h"
 
-static void GfxMainBlitter(byte *sprite, int x, int y, int mode);
+static void GfxMainBlitter(const Sprite *sprite, int x, int y, int mode);
 
 static int _stringwidth_out;
 static byte _cursor_backup[64*64];
 static Rect _invalid_rect;
-static byte *_color_remap_ptr;
+static const byte *_color_remap_ptr;
 static byte _string_colorremap[3];
 
 #define DIRTY_BYTES_PER_LINE (MAX_SCREEN_WIDTH/64)
@@ -104,7 +106,7 @@ void GfxScroll(int left, int top, int width, int height, int xo, int yo) {
 
 void GfxFillRect(int left, int top, int right, int bottom, int color) {
 	DrawPixelInfo *dpi = _cur_dpi;
-	byte *dst,*ctab;
+	byte *dst;
 	const int otop = top;
 	const int oleft = left;
 
@@ -142,7 +144,8 @@ void GfxFillRect(int left, int top, int right, int bottom, int color) {
 			} while (--bottom);
 		} else {
 			/* use colortable mode */
-			ctab = GetSpritePtr(color & 0x3FFF) + 1;
+			const byte* ctab = GetNonSprite(color & 0x3FFF) + 1;
+
 			do {
 				int i;
 				for(i=0; i!=right;i++)
@@ -241,8 +244,6 @@ void GfxDrawLine(int x, int y, int x2, int y2, int color)
 
 
 enum {
-	ASCII_LETTERSTART = 32,
-
 	ASCII_SETX = 1,
 	ASCII_SETXY = 2,
 
@@ -257,29 +258,31 @@ enum {
 /* returns right coordinate */
 int DrawString(int x, int y, uint16 str, uint16 color)
 {
-	GetString(str_buffr, str);
-	assert(strlen(str_buffr) < sizeof(str_buffr) - 1);
-	return DoDrawString(str_buffr, x, y, color);
+	char buffer[512];
+
+	GetString(buffer, str);
+	return DoDrawString(buffer, x, y, color);
 }
 
 
 void DrawStringRightAligned(int x, int y, uint16 str, uint16 color)
 {
-	GetString(str_buffr, str);
-	assert(strlen(str_buffr) < sizeof(str_buffr) - 1);
-	DoDrawString(str_buffr, x - GetStringWidth(str_buffr), y, color);
+	char buffer[512];
+
+	GetString(buffer, str);
+	DoDrawString(buffer, x - GetStringWidth(buffer), y, color);
 }
 
 
 int DrawStringCentered(int x, int y, uint16 str, uint16 color)
 {
+	char buffer[512];
 	int w;
 
-	GetString(str_buffr, str);
-	assert(strlen(str_buffr) < sizeof(str_buffr) - 1);
+	GetString(buffer, str);
 
-	w = GetStringWidth(str_buffr);
-	DoDrawString(str_buffr, x - (w>>1), y, color);
+	w = GetStringWidth(buffer);
+	DoDrawString(buffer, x - w / 2, y, color);
 
 	return w;
 }
@@ -290,12 +293,12 @@ void DrawStringCenterUnderline(int x, int y, uint16 str, uint16 color)
 	GfxFillRect(x-(w>>1), y+10, x-(w>>1)+w, y+10, _string_colorremap[1]);
 }
 
-static uint32 FormatStringLinebreaks(byte *str, int maxw)
+static uint32 FormatStringLinebreaks(char *str, int maxw)
 {
 	int num = 0;
 	int base = _stringwidth_base;
 	int w;
-	byte *last_space;
+	char *last_space;
 	byte c;
 
 	for(;;) {
@@ -304,10 +307,10 @@ static uint32 FormatStringLinebreaks(byte *str, int maxw)
 
 		for(;;) {
 			c = *str++;
-			if (c == ' ') last_space = str;
+			if (c == ASCII_LETTERSTART) last_space = str;
 
 			if (c >= ASCII_LETTERSTART) {
-				w += _stringwidth_table[base + ((byte)c) - 0x20];
+				w += GetCharacterWidth(base + (byte)c);
 				if (w > maxw) {
 					str = last_space;
 					if (str == NULL)
@@ -333,15 +336,15 @@ static uint32 FormatStringLinebreaks(byte *str, int maxw)
 
 void DrawStringMultiCenter(int x, int y, uint16 str, int maxw)
 {
+	char buffer[512];
 	uint32 tmp;
 	int num, w, mt, t;
-	byte *src;
+	const char *src;
 	byte c;
 
-	GetString(str_buffr, str);
-	assert(strlen(str_buffr) < sizeof(str_buffr) - 1);
+	GetString(buffer, str);
 
-	tmp = FormatStringLinebreaks(str_buffr, maxw);
+	tmp = FormatStringLinebreaks(buffer, maxw);
 	num = (uint16)tmp;
 	t = tmp >> 16;
 
@@ -353,7 +356,7 @@ void DrawStringMultiCenter(int x, int y, uint16 str, int maxw)
 
 	y -= (mt >> 1) * num;
 
-	src = str_buffr;
+	src = buffer;
 
 	for(;;) {
 		w = GetStringWidth(src);
@@ -379,15 +382,15 @@ void DrawStringMultiCenter(int x, int y, uint16 str, int maxw)
 }
 
 void DrawStringMultiLine(int x, int y, uint16 str, int maxw) {
+	char buffer[512];
 	uint32 tmp;
 	int num, w, mt, t;
-	byte *src;
+	const char *src;
 	byte c;
 
-	GetString(str_buffr, str);
-	assert(strlen(str_buffr) < sizeof(str_buffr) - 1);
+	GetString(buffer, str);
 
-	tmp = FormatStringLinebreaks(str_buffr, maxw);
+	tmp = FormatStringLinebreaks(buffer, maxw);
 	num = (uint16)tmp;
 	t = tmp >> 16;
 	mt = 10;
@@ -396,7 +399,7 @@ void DrawStringMultiLine(int x, int y, uint16 str, int maxw) {
 		if (t != 244) mt = 18;
 	}
 
-	src = str_buffr;
+	src = buffer;
 
 	for(;;) {
 		w = GetStringWidth(src);
@@ -421,17 +424,14 @@ void DrawStringMultiLine(int x, int y, uint16 str, int maxw) {
 	}
 }
 
-int GetStringWidth(const byte *str) {
-	int w = -1;
+int GetStringWidth(const char *str)
+{
+	int w = 0;
 	byte c;
 	int base = _stringwidth_base;
-
-	for(;;) {
-		c = *str++;
-		if (c == 0)
-			return w;
+	for (c = *str; c != '\0'; c = *(++str)) {
 		if (c >= ASCII_LETTERSTART) {
-			w += _stringwidth_table[base + c - ASCII_LETTERSTART];
+			w += GetCharacterWidth(base + c);
 		} else {
 			if (c == ASCII_SETX) str++;
 			else if (c == ASCII_SETXY) str += 2;
@@ -439,6 +439,7 @@ int GetStringWidth(const byte *str) {
 			else if (c == ASCII_BIGFONT) base = 448;
 		}
 	}
+	return w;
 }
 
 void DrawFrameRect(int left, int top, int right, int bottom, int ctab, int flags) {
@@ -474,7 +475,8 @@ void DrawFrameRect(int left, int top, int right, int bottom, int ctab, int flags
 	}
 }
 
-int DoDrawString(const byte *string, int x, int y, uint16 real_color) {
+int DoDrawString(const char *string, int x, int y, uint16 real_color)
+{
 	DrawPixelInfo *dpi = _cur_dpi;
 	int base = _stringwidth_base;
 	byte c;
@@ -522,9 +524,9 @@ skip_cont:;
 		if (c >= ASCII_LETTERSTART) {
 			if (x >= dpi->left + dpi->width) goto skip_char;
 			if (x + 26 >= dpi->left) {
-				GfxMainBlitter(GetSpritePtr(base + 2 + c - ASCII_LETTERSTART), x, y, 1);
+				GfxMainBlitter(GetSprite(base + 2 + c - ASCII_LETTERSTART), x, y, 1);
 			}
-			x += _stringwidth_table[base + c - ' '];
+			x += GetCharacterWidth(base + c);
 		} else if (c == ASCII_NL) { // newline = {}
 			x = xo;
 			y += 10;
@@ -538,10 +540,10 @@ skip_cont:;
 			color = (byte)(c - ASCII_COLORSTART);
 			goto switch_color;
 		} else if (c == ASCII_SETX) { // {SETX}
-			x = xo + *string++;
+			x = xo + (byte)*string++;
 		} else if (c == ASCII_SETXY) {// {SETXY}
-			x = xo + *string++;
-			y = yo + *string++;
+			x = xo + (byte)*string++;
+			y = yo + (byte)*string++;
 		} else if (c == ASCII_TINYFONT) { // {TINYFONT}
 			base = 0xE0;
 		} else if (c == ASCII_BIGFONT) { // {BIGFONT}
@@ -554,19 +556,20 @@ skip_cont:;
 
 void DrawSprite(uint32 img, int x, int y) {
 	if (img & 0x8000) {
-		_color_remap_ptr = GetSpritePtr(img >> 16) + 1;
-		GfxMainBlitter(GetSpritePtr(img & 0x3FFF), x, y, 1);
+		_color_remap_ptr = GetNonSprite(img >> 16) + 1;
+		GfxMainBlitter(GetSprite(img & 0x3FFF), x, y, 1);
 	} else if (img & 0x4000) {
-		_color_remap_ptr = GetSpritePtr(img >> 16) + 1;
-		GfxMainBlitter(GetSpritePtr(img & 0x3FFF), x, y, 2);
+		_color_remap_ptr = GetNonSprite(img >> 16) + 1;
+		GfxMainBlitter(GetSprite(img & 0x3FFF), x, y, 2);
 	} else {
-		GfxMainBlitter(GetSpritePtr(img & 0x3FFF), x, y, 0);
+		GfxMainBlitter(GetSprite(img & 0x3FFF), x, y, 0);
 	}
 }
 
 typedef struct BlitterParams {
 	int start_x, start_y;
-	byte *sprite, *sprite_org;
+	const byte* sprite;
+	const byte* sprite_org;
 	byte *dst;
 	int mode;
 	int width, height;
@@ -577,10 +580,12 @@ typedef struct BlitterParams {
 
 static void GfxBlitTileZoomIn(BlitterParams *bp)
 {
-	byte *src_o = bp->sprite, *src;
+	const byte* src_o = bp->sprite;
+	const byte* src;
 	int num, skip;
 	byte done;
-	byte *dst, *ctab;
+	byte *dst;
+	const byte* ctab;
 
 	if (bp->mode & 1) {
 		src_o += READ_LE_UINT16(src_o + bp->start_y * 2);
@@ -719,7 +724,7 @@ static void GfxBlitTileZoomIn(BlitterParams *bp)
 
 static void GfxBlitZoomInUncomp(BlitterParams *bp)
 {
-	byte *src = bp->sprite;
+	const byte *src = bp->sprite;
 	byte *dst = bp->dst;
 	int height = bp->height;
 	int width = bp->width;
@@ -730,7 +735,7 @@ static void GfxBlitZoomInUncomp(BlitterParams *bp)
 
 	if (bp->mode & 1) {
 		if (bp->info & 1) {
-			byte *ctab = _color_remap_ptr;
+			const byte *ctab = _color_remap_ptr;
 			byte b;
 
 			do {
@@ -744,7 +749,8 @@ static void GfxBlitZoomInUncomp(BlitterParams *bp)
 		}
 	} else if (bp->mode & 2) {
 		if (bp->info & 1) {
-			byte *ctab = _color_remap_ptr;
+			const byte *ctab = _color_remap_ptr;
+
 			do {
 				for(i=0; i!=width; i++)
 					if (src[i])
@@ -791,10 +797,12 @@ static void GfxBlitZoomInUncomp(BlitterParams *bp)
 
 static void GfxBlitTileZoomMedium(BlitterParams *bp)
 {
-	byte *src_o = bp->sprite, *src;
+	const byte* src_o = bp->sprite;
+	const byte* src;
 	int num, skip;
 	byte done;
-	byte *dst, *ctab;
+	byte *dst;
+	const byte* ctab;
 
 	if (bp->mode & 1) {
 		src_o += READ_LE_UINT16(src_o + bp->start_y * 2);
@@ -967,7 +975,7 @@ static void GfxBlitTileZoomMedium(BlitterParams *bp)
 
 static void GfxBlitZoomMediumUncomp(BlitterParams *bp)
 {
-	byte *src = bp->sprite;
+	const byte *src = bp->sprite;
 	byte *dst = bp->dst;
 	int height = bp->height;
 	int width = bp->width;
@@ -978,7 +986,9 @@ static void GfxBlitZoomMediumUncomp(BlitterParams *bp)
 
 	if (bp->mode & 1) {
 		if (bp->info & 1) {
-			byte *ctab = _color_remap_ptr,b;
+			const byte *ctab = _color_remap_ptr;
+			byte b;
+
 			height >>= 1;
 			if (height)	do {
 				for(i=0; i!=width>>1; i++)
@@ -990,7 +1000,8 @@ static void GfxBlitZoomMediumUncomp(BlitterParams *bp)
 		}
 	} else if (bp->mode & 2) {
 		if (bp->info & 1) {
-			byte *ctab = _color_remap_ptr;
+			const byte *ctab = _color_remap_ptr;
+
 			height >>= 1;
 			if (height)	do {
 				for(i=0; i!=width>>1; i++)
@@ -1016,10 +1027,12 @@ static void GfxBlitZoomMediumUncomp(BlitterParams *bp)
 
 static void GfxBlitTileZoomOut(BlitterParams *bp)
 {
-	byte *src_o = bp->sprite, *src;
+	const byte* src_o = bp->sprite;
+	const byte* src;
 	int num, skip;
 	byte done;
-	byte *dst, *ctab;
+	byte *dst;
+	const byte* ctab;
 
 	if (bp->mode & 1) {
 		src_o += READ_LE_UINT16(src_o + bp->start_y * 2);
@@ -1258,7 +1271,7 @@ static void GfxBlitTileZoomOut(BlitterParams *bp)
 
 static void GfxBlitZoomOutUncomp(BlitterParams *bp)
 {
-	byte *src = bp->sprite;
+	const byte* src = bp->sprite;
 	byte *dst = bp->dst;
 	int height = bp->height;
 	int width = bp->width;
@@ -1269,7 +1282,9 @@ static void GfxBlitZoomOutUncomp(BlitterParams *bp)
 
 	if (bp->mode & 1) {
 		if (bp->info & 1) {
-			byte *ctab = _color_remap_ptr,b;
+			const byte *ctab = _color_remap_ptr;
+			byte b;
+
 			height >>= 2;
 			if (height)	do {
 				for(i=0; i!=width>>2; i++)
@@ -1281,7 +1296,8 @@ static void GfxBlitZoomOutUncomp(BlitterParams *bp)
 		}
 	} else if (bp->mode & 2) {
 		if (bp->info & 1) {
-			byte *ctab = _color_remap_ptr;
+			const byte *ctab = _color_remap_ptr;
+
 			height >>= 2;
 			if (height)	do {
 				for(i=0; i!=width>>2; i++)
@@ -1307,7 +1323,7 @@ static void GfxBlitZoomOutUncomp(BlitterParams *bp)
 
 typedef void (*BlitZoomFunc)(BlitterParams *bp);
 
-static void GfxMainBlitter(byte *sprite, int x, int y, int mode)
+static void GfxMainBlitter(const Sprite* sprite, int x, int y, int mode)
 {
 	DrawPixelInfo *dpi = _cur_dpi;
 	int start_x, start_y;
@@ -1329,13 +1345,13 @@ static void GfxMainBlitter(byte *sprite, int x, int y, int mode)
 	};
 
 	/* decode sprite header */
-	x += (int16)READ_LE_UINT16(&((SpriteHdr*)sprite)->x_offs);
-	y += (int16)READ_LE_UINT16(&((SpriteHdr*)sprite)->y_offs);
-	bp.width_org = bp.width = READ_LE_UINT16(&((SpriteHdr*)sprite)->width);
-	bp.height_org = bp.height = ((SpriteHdr*)sprite)->height;
-	info = ((SpriteHdr*)sprite)->info;
+	x += sprite->x_offs;
+	y += sprite->y_offs;
+	bp.width_org = bp.width = sprite->width;
+	bp.height_org = bp.height = sprite->height;
+	info = sprite->info;
 	bp.info = info;
-	bp.sprite_org = bp.sprite = sprite + sizeof(SpriteHdr);
+	bp.sprite_org = bp.sprite = sprite->data;
 	bp.dst = dpi->dst_ptr;
 	bp.mode = mode;
 	bp.pitch = dpi->pitch;
@@ -1432,7 +1448,7 @@ static void GfxMainBlitter(byte *sprite, int x, int y, int mode)
 		if (info&2) {
 			int totpix = bp.height_org * bp.width_org;
 			byte *dst = (byte*)alloca(totpix);
-			byte *src = bp.sprite_org;
+			const byte *src = bp.sprite_org;
 			signed char b;
 
 			bp.sprite = dst + (bp.sprite - bp.sprite_org);
@@ -1622,15 +1638,15 @@ void LoadStringWidthTable(void)
 
 	// 2 equals space.
 	for(i=2; i != 0xE2; i++) {
-		*b++ = (byte)((i < 93 || i >= 129 || i == 98) ? GetSpritePtr(i)[2] : 0);
+		*b++ = (byte)((i < 93 || i >= 129 || i == 98) ? GetSprite(i)->width : 0);
 	}
 
 	for(i=0xE2; i != 0x1C2; i++) {
-		*b++ = (byte)((i < 317 || i >= 353) ? GetSpritePtr(i)[2]+1 : 0);
+		*b++ = (byte)((i < 317 || i >= 353) ? GetSprite(i)->width + 1 : 0);
 	}
 
 	for(i=0x1C2; i != 0x2A2; i++) {
-		*b++ = (byte)((i < 541 || i >= 577) ? GetSpritePtr(i)[2]+1 : 0);
+		*b++ = (byte)((i < 541 || i >= 577) ? GetSprite(i)->width + 1 : 0);
 	}
 }
 
@@ -1900,17 +1916,17 @@ bool FillDrawPixelInfo(DrawPixelInfo *n, DrawPixelInfo *o, int left, int top, in
 static void SetCursorSprite(uint cursor)
 {
 	CursorVars *cv = &_cursor;
-	byte *p;
+	const Sprite *p;
 
 	if (cv->sprite == cursor)
 		return;
 
-	p =	GetSpritePtr(cursor & 0x3FFF);
+	p =	GetSprite(cursor & 0x3FFF);
 	cv->sprite = cursor;
-	cv->size.y = *(byte*)(p+1);
-	cv->size.x = READ_LE_UINT16(p+2);
-	cv->offs.x = (int16)READ_LE_UINT16(p+4);
-	cv->offs.y = (int16)READ_LE_UINT16(p+6);
+	cv->size.y = p->height;
+	cv->size.x = p->width;
+	cv->offs.x = p->x_offs;
+	cv->offs.y = p->y_offs;
 
 	cv->dirty = true;
 }
@@ -1963,15 +1979,18 @@ bool ChangeResInGame(int w, int h)
 	return true;
 }
 
-void ToggleFullScreen(const bool full_screen)
+void ToggleFullScreen(bool fs) {_video_driver->toggle_fullscreen(fs);}
+
+static int CDECL compare_res(const void *pa, const void *pb)
 {
-	_fullscreen = full_screen;
-	/* use preset resolutions, not _screen.height and _screen.width. On windows for example
-	   if Desktop-size is 1280x1024, and gamesize is also 1280x1024, _screen.height will be
-		 only 1000 because of possible start-bar. For this reason you cannot switch to
-		 fullscreen mode from this resolution. Use of preset resolution will fix this */
-	if (!_video_driver->change_resolution(_cur_resolution[0], _cur_resolution[1]))
-		_fullscreen ^= true; // switching resolution failed, put back full_screen to original status
+	int x = ((const uint16*)pa)[0] - ((const uint16*)pb)[0];
+	if (x != 0) return x;
+	return ((const uint16*)pa)[1] - ((const uint16*)pb)[1];
+}
+
+void SortResolutions(int count)
+{
+	qsort(_resolutions, count, sizeof(_resolutions[0]), compare_res);
 }
 
 uint16 GetDrawStringPlayerColor(byte player)
