@@ -1,6 +1,11 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "debug.h"
+#include "spritecache.h"
+#include "strings.h"
+#include "table/sprites.h"
 #include "table/strings.h"
+#include "table/tree_land.h"
 #include "map.h"
 #include "window.h"
 #include "gui.h"
@@ -12,13 +17,14 @@
 #include "town.h"
 #include "sound.h"
 #include "network.h"
+#include "string.h"
 
 #include "hal.h" // for file list
 
-bool _query_string_active;
+static bool _fios_path_changed;
+static bool _savegame_sort_dirty;
 
-/* Now this is what I call dirty.. the edit-box needs to be rewritten! */
-static bool _do_edit_on_text_even_when_no_change_to_edit_box;
+bool _query_string_active;
 
 typedef struct LandInfoData {
 	Town *town;
@@ -170,10 +176,10 @@ static const char *credits[] = {
 	"Original graphics by Simon Foster",
 	"",
 	"The OpenTTD team (in alphabetical order):",
-	"  Bjarni Corfitzen (Bjarni) - MacOS port",
+	"  Matthijs Kooijman (blathijs) - Pathfinder-god",
+	"  Bjarni Corfitzen (Bjarni) - MacOSX port, coder",
 	"  Victor Fischer (Celestar) - Programming everywhere you need him to",
 	"  Tamas Farag (Darkvater) - Lead coder",
-	"  Dominik Scherer (dominik81) - Lead coder",
 	"  Kerekes Miham (MiHaMiX) - Translator system, and Nightlies host",
 	"  Owen Rudge (orudge) - Forum- and masterserver host, OS/2 port",
 	"  Christoph Mallon (Tron) - Programmer, code correctness police",
@@ -182,13 +188,14 @@ static const char *credits[] = {
 	"Retired Developers:",
 	"  Ludvig Strigeus (ludde) - OpenTTD author, main coder (0.1 - 0.3.3)",
 	"  Serge Paquet (vurlix) - Assistant project manager, coder (0.1 - 0.3.3)",
+	"  Dominik Scherer (dominik81) - Lead programmer, GUI expert (0.3.0 - 0.3.6)",
 	"",
 	"Special thanks go out to:",
 	"  Josef Drexler - For his great work on TTDPatch",
 	"  Marcin Grzegorczyk - For his documentation of TTD internals",
-	"  blathijs - For his many patches, suggestions and major work on maprewrite",
-	"  pasky - Many patches, newgrf support",
+	"  Petr Baudis (pasky) - Many patches, newgrf support",
 	"  Stefan Meiner (sign_de) - For his work on the console",
+	"  Simon Sasburg (HackyKid) - Many bugfixes he has blessed us with (and PBS)",
 	"  Cian Duffy (MYOB) - BeOS port / manual writing",
 	"  Christian Rosentreter (tokaiz) - MorphOS / AmigaOS port",
 	"",
@@ -197,7 +204,6 @@ static const char *credits[] = {
 	"  Marcin Grzegorczyk - Foundations for Tracks on Slopes",
 	"  All Translators - Who made OpenTTD a truly international game",
 	"  Bug Reporters - Without whom OpenTTD would still be full of bugs!",
-	NULL,
 	"",
 	"",
 	"And last but not least:",
@@ -212,8 +218,6 @@ static void AboutWindowProc(Window *w, WindowEvent *e)
 		WP(w, scroller_d).height = w->height - 40;
 		break;
 	case WE_PAINT: {
-		const char *str;
-		char buffer[100];
 		uint i;
 		int y = WP(w, scroller_d).height;
 		DrawWindowWidgets(w);
@@ -225,16 +229,7 @@ static void AboutWindowProc(Window *w, WindowEvent *e)
 		// Show all scrolling credits
 		for (i = 0; i < lengthof(credits); i++) {
 			if (y >= 50 && y < (w->height - 40)) {
-				str = credits[i];
-				/* Hack-Alert: Translated by is a dynamic string as it changes
-				 * with the language chosen. So the special value of NULL is used
-				 * to identify this for the moment */
-				if (str == NULL) {
-					GetString(buffer, STR_TRANSLATED_BY);
-					str = buffer;
-				}
-
-			 DoDrawString(str, 10, y, 0x10);
+				DoDrawString(credits[i], 10, y, 0x10);
 			}
 			y += 10;
 		}
@@ -278,8 +273,6 @@ void ShowAboutWindow(void)
 
 static int _tree_to_plant;
 
-static const byte _tree_base_by_landscape[4] = {0, 12, 20, 32};
-static const byte _tree_count_by_landscape[4] = {12, 8, 12, 9};
 static const uint32 _tree_sprites[] = {
 	0x655,0x663,0x678,0x62B,0x647,0x639,0x64E,0x632,0x67F,0x68D,0x69B,0x6A9,
 	0x6AF,0x6D2,0x6D9,0x6C4,0x6CB,0x6B6,0x6BD,0x6E0,
@@ -629,6 +622,7 @@ static void TooltipsWndProc(Window *w, WindowEvent *e)
 
 void GuiShowTooltips(StringID string_id)
 {
+	char buffer[512];
 	Window *w;
 	int right,bottom;
 	int x,y;
@@ -643,10 +637,9 @@ void GuiShowTooltips(StringID string_id)
 		DeleteWindow(w);
 	}
 
-	GetString(str_buffr, string_id);
-	assert(strlen(str_buffr) < sizeof(str_buffr) - 1);
+	GetString(buffer, string_id);
 
-	right = GetStringWidth(str_buffr) + 4;
+	right = GetStringWidth(buffer) + 4;
 
 	bottom = 14;
 	if (right > 200) {
@@ -673,36 +666,27 @@ void GuiShowTooltips(StringID string_id)
 }
 
 
-static void DrawStationCoverageText(const uint *accepts, int str_x, int str_y, uint mask)
+static void DrawStationCoverageText(const AcceptedCargo accepts,
+	int str_x, int str_y, uint mask)
 {
+	char *b = _userstring;
 	int i;
-	byte *b;
 
-	b = _userstring;
-	b[0] = 0x81;
-	b[1] = STR_000D_ACCEPTS;
-	b[2] = STR_000D_ACCEPTS >> 8;
-	b += 3;
+	b = InlineString(b, STR_000D_ACCEPTS);
 
-	for(i=0; i!=NUM_CARGO; i++,mask>>=1) {
-		if (accepts[i] >= 8 && (mask&1) ) {
-			StringID id = _cargoc.names_s[i];
-			b[0] = 0x81;
-			b[1] = (byte)(id & 255);
-			b[2] = (byte)(id >> 8);
-			b[3] = ',';
-			b[4] = ' ';
-			b += 5;
+	for (i = 0; i != NUM_CARGO; i++, mask >>= 1) {
+		if (accepts[i] >= 8 && mask & 1) {
+			b = InlineString(b, _cargoc.names_s[i]);
+			*b++ = ',';
+			*b++ = ' ';
 		}
 	}
 
-	if (b == (byte*)&_userstring[3]) {
-		b[0] = 0x81;
-		b[1] = STR_00D0_NOTHING;
-		b[2] = STR_00D0_NOTHING >> 8;
-		b[3] = 0;
+	if (b == &_userstring[3]) {
+		b = InlineString(b, STR_00D0_NOTHING);
+		*b++ = '\0';
 	} else {
-		b[-2] = 0;
+		b[-2] = '\0';
 	}
 
 	DrawStringMultiLine(str_x, str_y, STR_SPEC_USERSTRING, 144);
@@ -773,77 +757,140 @@ void SetHScrollCount(Window *w, int num)
 	if (num < w->hscroll.pos) w->hscroll.pos = num;
 }
 
-/* Get the count of characters in the string as well as the width in pixels
- * [IN]buf: string to be checked
- * [OUT]count: gets set to the count of characters
- * [OUT]width: gets set to the pixels width */
-static void GetCurrentStringSize(const byte *buf, int *count, int *width)
+static void DelChar(Textbuf *tb)
 {
-	*count = 0;
-	*width = -1;
+	tb->width -= GetCharacterWidth((byte)tb->buf[tb->caretpos]);
+	memmove(tb->buf + tb->caretpos, tb->buf + tb->caretpos + 1, tb->length - tb->caretpos);
+	tb->length--;
+}
 
-	do {
-		if (*++buf == 0)
-			break;
-		(*count)++;
-		(*width) += _stringwidth_table[*buf - 32];
-	} while (1);
+/**
+ * Delete a character from a textbuffer, either with 'Delete' or 'Backspace'
+ * The character is delete from the position the caret is at
+ * @param tb @Textbuf type to be changed
+ * @param delmode Type of deletion, either @WKC_BACKSPACE or @WKC_DELETE
+ * @return Return true on successfull change of Textbuf, or false otherwise
+ */
+bool DeleteTextBufferChar(Textbuf *tb, int delmode)
+{
+	if (delmode == WKC_BACKSPACE && tb->caretpos != 0) {
+		tb->caretpos--;
+		tb->caretxoffs -= GetCharacterWidth((byte)tb->buf[tb->caretpos]);
+
+		DelChar(tb);
+		return true;
+	} else if (delmode == WKC_DELETE && tb->caretpos < tb->length) {
+		DelChar(tb);
+		return true;
+	}
+
+	return false;
+}
+
+/**
+ * Insert a character to a textbuffer. If maxlength is zero, we don't care about
+ * the screenlength but only about the physical length of the string
+ * @param tb @Textbuf type to be changed
+ * @param key Character to be inserted
+ * @return Return true on successfull change of Textbuf, or false otherwise
+ */
+bool InsertTextBufferChar(Textbuf *tb, byte key)
+{
+	const byte charwidth = GetCharacterWidth(key);
+	if (tb->length < tb->maxlength && (tb->maxwidth == 0 || tb->width + charwidth <= tb->maxwidth)) {
+		memmove(tb->buf + tb->caretpos + 1, tb->buf + tb->caretpos, (tb->length - tb->caretpos) + 1);
+		tb->buf[tb->caretpos] = key;
+		tb->length++;
+		tb->width += charwidth;
+
+		tb->caretpos++;
+		tb->caretxoffs += charwidth;
+		return true;
+	}
+	return false;
+}
+
+/**
+ * Handle text navigation with arrow keys left/right.
+ * This defines where the caret will blink and the next characer interaction will occur
+ * @param tb @Textbuf type where navigation occurs
+ * @param navmode Direction in which navigation occurs @WKC_LEFT, @WKC_RIGHT, @WKC_END, @WKC_HOME
+ * @return Return true on successfull change of Textbuf, or false otherwise
+ */
+bool MoveTextBufferPos(Textbuf *tb, int navmode)
+{
+	switch (navmode) {
+	case WKC_LEFT:
+		if (tb->caretpos != 0) {
+			tb->caretpos--;
+			tb->caretxoffs -= GetCharacterWidth((byte)tb->buf[tb->caretpos]);
+			return true;
+		}
+		break;
+	case WKC_RIGHT:
+		if (tb->caretpos < tb->length) {
+			tb->caretxoffs += GetCharacterWidth((byte)tb->buf[tb->caretpos]);
+			tb->caretpos++;
+			return true;
+		}
+		break;
+	case WKC_HOME:
+		tb->caretpos = 0;
+		tb->caretxoffs = 0;
+		return true;
+	case WKC_END:
+		tb->caretpos = tb->length;
+		tb->caretxoffs = tb->width;
+		return true;
+	}
+
+	return false;
+}
+
+/**
+ * Update @Textbuf type with its actual physical character and screenlength
+ * Get the count of characters in the string as well as the width in pixels.
+ * Useful when copying in a larger amount of text at once
+ * @param tb @Textbuf type which length is calculated
+ */
+void UpdateTextBufferSize(Textbuf *tb)
+{
+	char *buf;
+	tb->length = 0;
+	tb->width = 0;
+
+	for (buf = tb->buf; *buf != '\0' && tb->length <= tb->maxlength; buf++) {
+		tb->length++;
+		tb->width += GetCharacterWidth((byte)*buf);
+	}
+
+	tb->caretpos = tb->length;
+	tb->caretxoffs = tb->width;
 }
 
 int HandleEditBoxKey(Window *w, int wid, WindowEvent *we)
 {
-	int width,count;
-	int key = we->keypress.ascii;
-
 	we->keypress.cont = false;
 
-	if (we->keypress.keycode == WKC_ESC) {
-		return 2;
-	} else if (we->keypress.keycode == WKC_RETURN) {
-		return 1;
-#ifdef WIN32
-	} else if (we->keypress.keycode == (WKC_CTRL | 'V')) {
-		if (IsClipboardFormatAvailable(CF_TEXT)) {
-			const byte* data;
-			HGLOBAL cbuf;
-
-			OpenClipboard(NULL);
-			cbuf = GetClipboardData(CF_TEXT);
-			data = GlobalLock(cbuf); // clipboard data
-
-			GetCurrentStringSize(WP(w,querystr_d).buf - 1, &count, &width);
-
-			/* IS_INT_INSIDE = filter for ascii-function codes like BELL and so on [we need an special filter here later] */
-			for (; (IS_INT_INSIDE(*data, ' ', 256)) && // valid ASCII char
-					(count < WP(w,querystr_d).maxlen - 1 && // max charcount; always allow for terminating '\0'
-					width + _stringwidth_table[(int)(*data) - 32] <= WP(w,querystr_d).maxwidth); ++data) { // max screensize
-
-				// append data and update size parameters
-				WP(w,querystr_d).buf[count] = *data;
-				count++;
-				width += _stringwidth_table[*data - 32];
-			}
-			WP(w,querystr_d).buf[count + 1] = '\0';
-
-			GlobalUnlock(cbuf);
-			CloseClipboard();
+	switch (we->keypress.keycode) {
+	case WKC_ESC: return 2;
+	case WKC_RETURN: case WKC_NUM_ENTER: return 1;
+	case (WKC_CTRL | 'V'):
+		if (InsertTextBufferClipboard(&WP(w, querystr_d).text))
 			InvalidateWidget(w, wid);
-		}
-#endif
-	} else {
-		GetCurrentStringSize(WP(w,querystr_d).buf - 1, &count, &width);
-
-		if (we->keypress.keycode == WKC_BACKSPACE) {
-			if (count != 0) {
-				WP(w,querystr_d).buf[count-1] = 0;
+		break;
+	case WKC_BACKSPACE: case WKC_DELETE:
+		if (DeleteTextBufferChar(&WP(w, querystr_d).text, we->keypress.keycode))
+			InvalidateWidget(w, wid);
+		break;
+	case WKC_LEFT: case WKC_RIGHT: case WKC_END: case WKC_HOME:
+		if (MoveTextBufferPos(&WP(w, querystr_d).text, we->keypress.keycode))
+			InvalidateWidget(w, wid);
+  	break;
+	default:
+		if (IsValidAsciiChar(we->keypress.ascii)) {
+			if (InsertTextBufferChar(&WP(w, querystr_d).text, we->keypress.ascii))
 				InvalidateWidget(w, wid);
-			}
-		} else if (IS_INT_INSIDE((key = we->keypress.ascii), 32, 256)) {
-			if (count < WP(w,querystr_d).maxlen && width + _stringwidth_table[key - 32] <= WP(w,querystr_d).maxwidth) {
-				WP(w,querystr_d).buf[count] = key;
-				WP(w,querystr_d).buf[count + 1] = '\0';
-				InvalidateWidget(w, wid);
-			}
 		} else // key wasn't caught
 			we->keypress.cont = true;
 	}
@@ -851,54 +898,56 @@ int HandleEditBoxKey(Window *w, int wid, WindowEvent *we)
 	return 0;
 }
 
+bool HandleCaret(Textbuf *tb)
+{
+	/* caret changed? */
+	bool b = !!(_caret_timer & 0x20);
+
+	if (b != tb->caret) {
+		tb->caret = b;
+		return true;
+	}
+	return false;
+}
+
 void HandleEditBox(Window *w, int wid)
 {
-	bool b;
-
-	/* caret changed? */
-	b = !!(_caret_timer & 0x20);
-	if (b != WP(w,querystr_d).caret) {
-		WP(w,querystr_d).caret = b;
+	if (HandleCaret(&WP(w, querystr_d).text))
 		InvalidateWidget(w, wid);
-	}
 }
 
 void DrawEditBox(Window *w, int wid)
 {
 	const Widget *wi = w->widget + wid;
-	int x;
+	const Textbuf *tb = &WP(w,querystr_d).text;
 
 	GfxFillRect(wi->left+1, wi->top+1, wi->right-1, wi->bottom-1, 215);
-	x = DoDrawString(WP(w,querystr_d).buf, wi->left+2, wi->top+1, 8);
-	if (WP(w,querystr_d).caret)
-		DoDrawString("_", x, wi->top+1, 12);
+	DoDrawString(tb->buf, wi->left+2, wi->top+1, 8);
+	if (tb->caret)
+		DoDrawString("_", wi->left + 2 + tb->caretxoffs, wi->top + 1, 12);
 }
 
-
-#define MAX_QUERYSTR_LEN 64
-
 static void QueryStringWndProc(Window *w, WindowEvent *e)
 {
 	static bool closed = false;
 	switch(e->event) {
-	case WE_PAINT: {
-//		int x;
-
+	case WE_PAINT:
 		SetDParam(0, WP(w,querystr_d).caption);
 		DrawWindowWidgets(w);
 
 		DrawEditBox(w, 5);
-	} break;
+		break;
 
 	case WE_CLICK:
 		switch(e->click.widget) {
 		case 3: DeleteWindow(w); break;
 		case 4:
 press_ok:;
-			if (str_eq(WP(w,querystr_d).buf, WP(w,querystr_d).buf + MAX_QUERYSTR_LEN) && !_do_edit_on_text_even_when_no_change_to_edit_box) {
+			if (WP(w, querystr_d).orig != NULL &&
+					strcmp(WP(w, querystr_d).text.buf, WP(w, querystr_d).orig) == 0) {
 				DeleteWindow(w);
 			} else {
-				byte *buf = WP(w,querystr_d).buf;
+				char *buf = WP(w,querystr_d).text.buf;
 				WindowClass wnd_class = WP(w,querystr_d).wnd_class;
 				WindowNumber wnd_num = WP(w,querystr_d).wnd_num;
 				Window *parent;
@@ -953,6 +1002,7 @@ press_ok:;
 			}
 		}
 		_query_string_active = false;
+		CLRBIT(_no_scroll, SCROLL_EDIT);
 		break;
 	}
 }
@@ -975,43 +1025,41 @@ static const WindowDesc _query_string_desc = {
 	QueryStringWndProc
 };
 
-static byte _edit_str_buf[MAX_QUERYSTR_LEN*2];
+static char _edit_str_buf[64];
+static char _orig_str_buf[lengthof(_edit_str_buf)];
 
-void ShowQueryString(StringID str, StringID caption, int maxlen, int maxwidth, byte window_class, uint16 window_number)
+void ShowQueryString(StringID str, StringID caption, uint maxlen, uint maxwidth, WindowClass window_class, WindowNumber window_number)
 {
 	Window *w;
+	uint realmaxlen = maxlen & ~0x1000;
 
-#define _orig_edit_str_buf (_edit_str_buf+MAX_QUERYSTR_LEN)
+	assert(realmaxlen < lengthof(_edit_str_buf));
 
 	DeleteWindowById(WC_QUERY_STRING, 0);
 	DeleteWindowById(WC_SAVELOAD, 0);
 
-	if (str == 0xFFFF) {
-		memcpy(_orig_edit_str_buf, str_buffr, MAX_QUERYSTR_LEN);
-	} else {
-		GetString(_orig_edit_str_buf, str);
-	}
+	w = AllocateWindowDesc(&_query_string_desc);
+	SETBIT(_no_scroll, SCROLL_EDIT);
+
+	GetString(_edit_str_buf, str);
+	_edit_str_buf[realmaxlen] = '\0';
 
 	if (maxlen & 0x1000) {
-		_do_edit_on_text_even_when_no_change_to_edit_box = true;
-		maxlen &= ~0x1000;
-	} else
-		_do_edit_on_text_even_when_no_change_to_edit_box = false;
-
-	_orig_edit_str_buf[maxlen] = 0;
-
-	memcpy(_edit_str_buf, _orig_edit_str_buf, MAX_QUERYSTR_LEN);
-
-	w = AllocateWindowDesc(&_query_string_desc);
+		WP(w, querystr_d).orig = NULL;
+	} else {
+		strcpy(_orig_str_buf, _edit_str_buf);
+		WP(w, querystr_d).orig = _orig_str_buf;
+	}
 
 	w->click_state = 1 << 5;
-	WP(w,querystr_d).caption = caption;
-	WP(w,querystr_d).wnd_class = window_class;
-	WP(w,querystr_d).wnd_num = window_number;
-	WP(w,querystr_d).caret = 0;
-	WP(w,querystr_d).maxlen = maxlen;
-	WP(w,querystr_d).maxwidth = maxwidth;
-	WP(w,querystr_d).buf = _edit_str_buf;
+	WP(w, querystr_d).caption = caption;
+	WP(w, querystr_d).wnd_class = window_class;
+	WP(w, querystr_d).wnd_num = window_number;
+	WP(w, querystr_d).text.caret = false;
+	WP(w, querystr_d).text.maxlength = realmaxlen - 1;
+	WP(w, querystr_d).text.maxwidth = maxwidth;
+	WP(w, querystr_d).text.buf = _edit_str_buf;
+	UpdateTextBufferSize(&WP(w, querystr_d).text);
 
 	_query_string_active = true;
 }
@@ -1079,31 +1127,30 @@ static const Widget _save_dialog_scen_widgets[] = {
 
 void BuildFileList(void)
 {
+	_fios_path_changed = true;
 	FiosFreeSavegameList();
-	if(_saveload_mode==SLD_NEW_GAME || _saveload_mode==SLD_LOAD_SCENARIO || _saveload_mode==SLD_SAVE_SCENARIO)
+	if (_saveload_mode == SLD_NEW_GAME || _saveload_mode == SLD_LOAD_SCENARIO || _saveload_mode == SLD_SAVE_SCENARIO) {
 		_fios_list = FiosGetScenarioList(&_fios_num, _saveload_mode);
-	else
+	} else
 		_fios_list = FiosGetSavegameList(&_fios_num, _saveload_mode);
 }
 
 static void DrawFiosTexts(void)
 {
-	const char *path;
-	StringID str;
+	static const char *path = NULL;
+	static StringID str = STR_4006_UNABLE_TO_READ_DRIVE;
+	static uint32 tot = 0;
 
-	str = FiosGetDescText(&path);
-	if (str != 0)
-		DrawString(2, 37, str, 0);
+	if (_fios_path_changed) {
+		str = FiosGetDescText(&path, &tot);
+		_fios_path_changed = false;
+	}
+
+	if (str != STR_4006_UNABLE_TO_READ_DRIVE) SetDParam(0, tot);
+	DrawString(2, 37, str, 0);
 	DoDrawString(path, 2, 27, 16);
 }
 
-#if defined(_WIN32)
-	extern int CDECL compare_FiosItems (const void *a, const void *b);
-#else
-	extern int compare_FiosItems (const void *a, const void *b);
-#endif
-
-
 static void MakeSortedSaveGameList(void)
 {
 	/*	Directories are always above the files (FIOS_TYPE_DIR)
@@ -1213,13 +1260,14 @@ static void SaveLoadDlgWndProc(Window *w, WindowEvent *e)
 					strcpy(_file_to_saveload.name, name);
 					DeleteWindow(w);
 				} else if (_saveload_mode == SLD_LOAD_SCENARIO) {
-					_switch_mode = SM_LOAD_SCENARIO;
+					_switch_mode = (_game_mode == GM_MENU) ? SM_LOAD : SM_LOAD_SCENARIO;
 					SetFiosType(file->type);
 					strcpy(_file_to_saveload.name, name);
 					DeleteWindow(w);
 				} else {
 					// SLD_SAVE_GAME, SLD_SAVE_SCENARIO copy clicked name to editbox
-					strcpy(WP(w,querystr_d).buf, file->title[0] ? file->title : file->name);
+					ttd_strlcpy(WP(w, querystr_d).text.buf, (file->title[0] != '\0') ? file->title : file->name, WP(w, querystr_d).text.maxlength);
+					UpdateTextBufferSize(&WP(w, querystr_d).text);
 					InvalidateWidget(w, 9);
 				}
 			} else {
@@ -1245,25 +1293,28 @@ static void SaveLoadDlgWndProc(Window *w, WindowEvent *e)
 		break;
 	case WE_TIMEOUT:
 		if (HASBIT(w->click_state, 10)) { /* Delete button clicked */
-			FiosDelete(WP(w,querystr_d).buf);
+			FiosDelete(WP(w,querystr_d).text.buf);
 			SetWindowDirty(w);
 			BuildFileList();
-			if (_saveload_mode == SLD_SAVE_GAME)
+			if (_saveload_mode == SLD_SAVE_GAME) {
 				GenerateFileName(); /* Reset file name to current date */
+				UpdateTextBufferSize(&WP(w, querystr_d).text);
+			}
 		} else if (HASBIT(w->click_state, 11)) { /* Save button clicked */
 			_switch_mode = SM_SAVE;
-			FiosMakeSavegameName(_file_to_saveload.name, WP(w,querystr_d).buf);
+			FiosMakeSavegameName(_file_to_saveload.name, WP(w,querystr_d).text.buf);
 
 			/* In the editor set up the vehicle engines correctly (date might have changed) */
 			if (_game_mode == GM_EDITOR) StartupEngines();
 		}
 		break;
 	case WE_DESTROY:
-		// pause is only used in single-player, non-editor mode
-		if(!_networking && (_game_mode != GM_EDITOR))
+		// pause is only used in single-player, non-editor mode, non menu mode
+		if(!_networking && (_game_mode != GM_EDITOR) && (_game_mode != GM_MENU))
 			DoCommandP(0, 0, 0, NULL, CMD_PAUSE);
 		_query_string_active = false;
 		FiosFreeSavegameList();
+		CLRBIT(_no_scroll, SCROLL_SAVE);
 		break;
 	case WE_RESIZE: {
 		/* Widget 2 and 3 have to go with halve speed, make it so obiwan */
@@ -1332,25 +1383,31 @@ void ShowSaveLoadDialog(int mode)
 	DeleteWindowById(WC_SAVELOAD, 0);
 
 	_saveload_mode = mode;
+	SETBIT(_no_scroll, SCROLL_SAVE);
+
+	switch (mode) {
+	case SLD_SAVE_GAME:
+		GenerateFileName();
+		break;
+	case SLD_SAVE_SCENARIO:
+		strcpy(_edit_str_buf, "UNNAMED");
+		break;
+	}
 
 	w = AllocateWindowDesc(_saveload_dialogs[mode]);
 	w->vscroll.cap = 24;
 	w->resize.step_width = 2;
 	w->resize.step_height = 10;
 	w->resize.height = w->height - 14 * 10; // Minimum of 10 items
-	w->click_state |= (1 << 6);
-	WP(w,querystr_d).caret = 0;
-	WP(w,querystr_d).maxlen = MAX_QUERYSTR_LEN;
-	WP(w,querystr_d).maxwidth = 240;
-	WP(w,querystr_d).buf = _edit_str_buf;
+	SETBIT(w->click_state, 6);
+	WP(w,querystr_d).text.caret = false;
+	WP(w,querystr_d).text.maxlength = lengthof(_edit_str_buf) - 1;
+	WP(w,querystr_d).text.maxwidth = 240;
+	WP(w,querystr_d).text.buf = _edit_str_buf;
+	UpdateTextBufferSize(&WP(w, querystr_d).text);
 
-	if (mode == SLD_SAVE_GAME) {
-		GenerateFileName();
-	} else if (mode == SLD_SAVE_SCENARIO) {
-		strcpy(_edit_str_buf, "UNNAMED");
-	}
-
-	// pause is only used in single-player, non-editor mode
+	// pause is only used in single-player, non-editor mode, non-menu mode. It
+	// will be unpaused in the WE_DESTROY event handler.
 	if(_game_mode != GM_MENU && !_networking && _game_mode != GM_EDITOR)
 		DoCommandP(0, 1, 0, NULL, CMD_PAUSE);
 
@@ -1424,8 +1481,8 @@ static void SelectScenarioWndProc(Window *w, WindowEvent *e) {
 			break;
 
 		case 6: /* Click the listbox */
-			if(e->click.pt.y < list_start)
-				DoCommandP(0, Random(), InteractiveRandom(), NULL, CMD_GEN_RANDOM_NEW_GAME);
+			if (e->click.pt.y < list_start)
+				GenRandomNewGame(Random(), InteractiveRandom());
 			else {
 				char *name;
 				int y = (e->click.pt.y - list_start) / 10;
@@ -1440,7 +1497,7 @@ static void SelectScenarioWndProc(Window *w, WindowEvent *e) {
 					SetFiosType(file->type);
 					strcpy(_file_to_saveload.name, name);
 					DeleteWindow(w);
-					DoCommandP(0, Random(), InteractiveRandom(), NULL, CMD_START_SCENARIO);
+					StartScenarioEditor(Random(), InteractiveRandom());
 				}
 			}
 			break;
@@ -1549,8 +1606,18 @@ static int32 ClickChangeDateCheat(int32 p1, int32 p2)
 
 typedef int32 CheckButtonClick(int32, int32);
 
+typedef enum ce_type {
+	CE_BOOL = 0,
+	CE_UINT8 = 1,
+	CE_INT16 = 2,
+	CE_UINT16 = 3,
+	CE_INT32 = 4,
+	CE_BYTE = 5,
+	CE_CLICK = 6,
+} ce_type;
+
 typedef struct CheatEntry {
-	byte type;    // type of selector
+	ce_type type;    // type of selector
 	byte flags;		// selector flags
 	StringID str; // string with descriptive text
 	void *variable; // pointer to the variable
@@ -1560,19 +1627,6 @@ typedef struct CheatEntry {
 	uint16 step;   // step for spinbox
 } CheatEntry;
 
-enum {
-	CE_BOOL = 0,
-	CE_UINT8 = 1,
-	CE_INT16 = 2,
-	CE_UINT16 = 3,
-	CE_INT32 = 4,
-	CE_BYTE = 5,
-	CE_CLICK = 6,
-
-	CF_0ISDIS = 1,
-	CF_NOCOMMA = 2,
-};
-
 static int32 ReadCE(const CheatEntry*ce)
 {
 	switch(ce->type) {
@@ -1614,6 +1668,7 @@ static const CheatEntry _cheats_ui[] = {
 	{CE_BOOL, 0, STR_CHEAT_CROSSINGTUNNELS,	&_cheats.crossing_tunnels.value,&_cheats.crossing_tunnels.been_used,NULL,											0, 0, 0},
 	{CE_BOOL, 0, STR_CHEAT_BUILD_IN_PAUSE,	&_cheats.build_in_pause.value,	&_cheats.build_in_pause.been_used,	NULL,											0, 0, 0},
 	{CE_BOOL, 0, STR_CHEAT_NO_JETCRASH,			&_cheats.no_jetcrash.value,			&_cheats.no_jetcrash.been_used,			NULL,											0, 0, 0},
+	{CE_BOOL, 0, STR_CHEAT_SETUP_PROD,			&_cheats.setup_prod.value,			&_cheats.setup_prod.been_used,			NULL,											0, 0, 0},
 	{CE_UINT8, 0, STR_CHEAT_SWITCH_CLIMATE, &_opt.landscape, 								&_cheats.switch_climate.been_used,	&ClickChangeClimateCheat,-1, 4, 1},
 	{CE_UINT8, 0, STR_CHEAT_CHANGE_DATE,		&_cur_year,											&_cheats.change_date.been_used,			&ClickChangeDateCheat,	 -1, 1, 1},
 };
@@ -1622,8 +1677,8 @@ static const CheatEntry _cheats_ui[] = {
 static const Widget _cheat_widgets[] = {
 {   WWT_CLOSEBOX,   RESIZE_NONE,    14,     0,    10,     0,    13, STR_00C5,		STR_018B_CLOSE_WINDOW},
 {    WWT_CAPTION,   RESIZE_NONE,    14,    11,   399,     0,    13, STR_CHEATS,	STR_018C_WINDOW_TITLE_DRAG_THIS},
-{      WWT_PANEL,   RESIZE_NONE,    14,     0,   399,    14,   147, 0x0,					STR_NULL},
-{     WWT_IMGBTN,   RESIZE_NONE,    14,     0,   399,    14,   147, 0x0,					STR_CHEATS_TIP},
+{      WWT_PANEL,   RESIZE_NONE,    14,     0,   399,    14,   159, 0x0,					STR_NULL},
+{     WWT_IMGBTN,   RESIZE_NONE,    14,     0,   399,    14,   159, 0x0,					STR_CHEATS_TIP},
 {   WIDGETS_END},
 };
 
@@ -1758,7 +1813,7 @@ static void CheatsWndProc(Window *w, WindowEvent *e)
 	}
 }
 static const WindowDesc _cheats_desc = {
-	240, 22, 400, 148,
+	240, 22, 400, 160,
 	WC_CHEATS,0,
 	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET | WDF_UNCLICK_BUTTONS,
 	_cheat_widgets,
