@@ -1,5 +1,6 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "table/sprites.h"
 #include "table/strings.h"
 #include "window.h"
 #include "gui.h"
@@ -167,11 +168,11 @@ static void PlayerFinancesWndProc(Window *w, WindowEvent *e)
 		} break;
 
 		case 6: /* increase loan */
-			DoCommandP(0, w->window_number, _ctrl_pressed, NULL, CMD_INCREASE_LOAN | CMD_MSG(STR_702C_CAN_T_BORROW_ANY_MORE_MONEY));
+			DoCommandP(0, 0, _ctrl_pressed, NULL, CMD_INCREASE_LOAN | CMD_MSG(STR_702C_CAN_T_BORROW_ANY_MORE_MONEY));
 			break;
 
 		case 7: /* repay loan */
-			DoCommandP(0, w->window_number, _ctrl_pressed, NULL, CMD_DECREASE_LOAN | CMD_MSG(STR_702F_CAN_T_REPAY_LOAN));
+			DoCommandP(0, 0, _ctrl_pressed, NULL, CMD_DECREASE_LOAN | CMD_MSG(STR_702F_CAN_T_REPAY_LOAN));
 			break;
 		}
 		break;
@@ -284,7 +285,7 @@ static void SelectPlayerColorWndProc(Window *w, WindowEvent *e)
 
 			for(i=0; i!=16; i++) {
 				if (!(used_colors & 1) && --item < 0) {
-					DoCommandP(0, w->window_number, i, NULL, CMD_SET_PLAYER_COLOR);
+					DoCommandP(0, 0, i, NULL, CMD_SET_PLAYER_COLOR);
 					DeleteWindow(w);
 					break;
 				}
@@ -326,7 +327,7 @@ static void SelectPlayerFaceWndProc(Window *w, WindowEvent *e)
 		switch(e->click.widget) {
 		case 3: DeleteWindow(w); break;
 		case 4: /* ok click */
-			DoCommandP(0, w->window_number, WP(w,facesel_d).face, NULL, CMD_SET_PLAYER_FACE);
+			DoCommandP(0, 0, WP(w,facesel_d).face, NULL, CMD_SET_PLAYER_FACE);
 			DeleteWindow(w);
 			break;
 		case 5: /* male click */
@@ -508,14 +509,19 @@ static void PlayerCompanyWndProc(Window *w, WindowEvent *e)
 
 			if (!_networking) SETBIT(w->hidden_state, 11); // hide company-password widget
 		} else {
+			if (p->location_of_house == 0) SETBIT(dis, 7);
+
 			if (_patches.allow_shares) { /* shares are allowed */
+				/* If all shares are owned by someone (none by nobody), disable buy button */
 				if (GetAmountOwnedBy(p, OWNER_SPECTATOR) == 0) SETBIT(dis, 9);
 
-				/* We cannot buy out real players in a network game */
+				/* Only 25% left to buy. If the player is human, disable buying it up.. TODO issues! */
 				if (GetAmountOwnedBy(p, OWNER_SPECTATOR) == 1 && !p->is_ai) SETBIT(dis, 9);
 
+				/* If the player doesn't own any shares, disable sell button */
 				if (GetAmountOwnedBy(p, _local_player) == 0) SETBIT(dis, 10);
 
+				/* Spectators cannot do anything of course */
 				if (_local_player == OWNER_SPECTATOR) dis |= (1 << 9) | (1 << 10);
 			} else /* shares are not allowed, disable buy/sell buttons */
 				dis |= (1 << 9) | (1 << 10);
@@ -625,9 +631,9 @@ static void PlayerCompanyWndProc(Window *w, WindowEvent *e)
 		break;
 
 	case WE_PLACE_OBJ: {
-		// you cannot destroy a HQ, only relocate it. So build_HQ is called, just with different flags
+		/* You cannot destroy a HQ, only relocate it. So build_HQ is called, just with different flags */
 		TileIndex tile = DEREF_PLAYER(w->window_number)->location_of_house;
-		if (DoCommandP(e->place.tile, (tile == 0) ? 0 : (1 << 16) | w->window_number, 0, NULL, CMD_BUILD_COMPANY_HQ | CMD_AUTO | CMD_NO_WATER | CMD_MSG(STR_7071_CAN_T_BUILD_COMPANY_HEADQUARTERS)))
+		if (DoCommandP(e->place.tile, (tile == 0) ? 0 : 1 | w->window_number, 0, NULL, CMD_BUILD_COMPANY_HQ | CMD_AUTO | CMD_NO_WATER | CMD_MSG(STR_7071_CAN_T_BUILD_COMPANY_HEADQUARTERS)))
 			ResetObjectToPlace();
 		break;
 	}
@@ -639,23 +645,23 @@ static void PlayerCompanyWndProc(Window *w, WindowEvent *e)
 		break;
 
 	case WE_ON_EDIT_TEXT: {
-		char *b = (char*)e->edittext.str;
+		char *b = e->edittext.str;
 
 		if (*b == 0 && WP(w,def_d).byte_1 != 2) // empty string is allowed for password
 			return;
 
-		memcpy(_decode_parameters, b, 32);
+		_cmd_text = b;
 		switch (WP(w,def_d).byte_1) {
 		case 0: /* Change president name */
-			DoCommandP(0, w->window_number, 0, NULL, CMD_CHANGE_PRESIDENT_NAME | CMD_MSG(STR_700D_CAN_T_CHANGE_PRESIDENT));
+			DoCommandP(0, 0, 0, NULL, CMD_CHANGE_PRESIDENT_NAME | CMD_MSG(STR_700D_CAN_T_CHANGE_PRESIDENT));
 			break;
 		case 1: /* Change company name */
-			DoCommandP(0, w->window_number, 0, NULL, CMD_CHANGE_COMPANY_NAME | CMD_MSG(STR_700C_CAN_T_CHANGE_COMPANY_NAME));
+			DoCommandP(0, 0, 0, NULL, CMD_CHANGE_COMPANY_NAME | CMD_MSG(STR_700C_CAN_T_CHANGE_COMPANY_NAME));
 			break;
 		#ifdef ENABLE_NETWORK
 		case 2: /* Change company password */
 			if (*b == 0) *b = '*'; // empty password is a '*' because of console argument
-			NetworkChangeCompanyPassword(b);
+			NetworkChangeCompanyPassword(1, &b);
 		#endif
 		}
 	} break;
