@@ -1,5 +1,10 @@
+/** @file,
+ * sjdlfkasjdf
+ */
 #include "stdafx.h"
 #include "ttd.h"
+#include "string.h"
+#include "strings.h"
 #include "table/strings.h"
 #include "map.h"
 #include "player.h"
@@ -215,7 +220,7 @@ static void SubtractMoneyFromAnyPlayer(Player *p, int32 cost)
 
 void SubtractMoneyFromPlayer(int32 cost)
 {
-	uint pid = _current_player;
+	PlayerID pid = _current_player;
 	if (pid < MAX_PLAYERS)
 		SubtractMoneyFromAnyPlayer(DEREF_PLAYER(pid), cost);
 }
@@ -436,7 +441,7 @@ restart:;
 			if (pp->is_active && p != pp) {
 				SetDParam(0, pp->president_name_2);
 				GetString(buffer2, pp->president_name_1);
-				if (str_eq(buffer2, buffer))
+				if (strcmp(buffer2, buffer) == 0)
 					goto restart;
 			}
 		}
@@ -607,10 +612,12 @@ void DeletePlayerWindows(int pi)
 	DeleteWindowById(WC_COMPANY, pi);
 	DeleteWindowById(WC_FINANCES, pi);
 	DeleteWindowById(WC_STATION_LIST, pi);
-	DeleteWindowById(WC_TRAINS_LIST, pi);
-	DeleteWindowById(WC_ROADVEH_LIST, pi);
-	DeleteWindowById(WC_SHIPS_LIST, pi);
-	DeleteWindowById(WC_AIRCRAFT_LIST, pi);
+	/* The vehicle list windows also have station in the window_number
+	 * A stationindex of -1 means the global vehicle list */
+	DeleteWindowById(WC_TRAINS_LIST, (-1 << 16) | pi);
+	DeleteWindowById(WC_ROADVEH_LIST, (-1 << 16) | pi);
+	DeleteWindowById(WC_SHIPS_LIST, (-1 << 16) | pi);
+	DeleteWindowById(WC_AIRCRAFT_LIST, (-1 << 16) | pi);
 	DeleteWindowById(WC_BUY_COMPANY, pi);
 }
 
@@ -626,23 +633,36 @@ static void DeletePlayerStuff(int pi)
 	p->president_name_1 = 0;
 }
 
-// functionality.
-// 0 - make new player
-// 1 - make new AI player
-// 2 - delete player (p2)
-// 3 - join player (p1 >> 8) & 0xFF with (p1 >> 16) & 0xFF
+/** Control the players: add, delete, etc.
+ * @param x,y unused
+ * @param p1 various functionality
+ * - p1 = 0 - create a new player, Which player (network) it will be is in p2
+ * - p1 = 1 - create a new AI player
+ * - p1 = 2 - delete a player. Player is identified by p2
+ * - p1 = 3 - merge two companies together. Player to merge #1 with player #2. Identified by p2
+ * @param p2 various functionality, dictated by p1
+ * - p1 = 0 - ClientID of the newly created player
+ * - p1 = 2 - PlayerID of the that is getting deleted
+ * - p1 = 3 - #1 p2 = (bit  0-15) - player to merge (p2 & 0xFFFF)
+ *          - #2 p2 = (bit 16-31) - player to be merged into ((p2>>16)&0xFFFF)
+ * @todo In the case of p1=0, create new player, the clientID of the new player is in parameter
+ * p2. This parameter is passed in at function DEF_SERVER_RECEIVE_COMMAND(PACKET_CLIENT_COMMAND)
+ * on the server itself. First of all this is unbelievably ugly; second of all, well,
+ * it IS ugly! <b>Someone fix this up :)</b> So where to fix?@n
+ * @arg - network_server.c:838 DEF_SERVER_RECEIVE_COMMAND(PACKET_CLIENT_COMMAND)@n
+ * @arg - network_client.c:536 DEF_CLIENT_RECEIVE_COMMAND(PACKET_SERVER_MAP) from where the map has been received
+ */
 int32 CmdPlayerCtrl(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
-	int pi;
-	Player *p;
+	if (flags & DC_EXEC) _current_player = OWNER_NONE;
 
-	if (!(flags & DC_EXEC))
-		return 0;
+	switch (p1) {
+	case 0: { /* Create a new player */
+		Player *p;
+		PlayerID pid = p2;
 
-	_current_player = OWNER_NONE;
+		if (!(flags & DC_EXEC) || pid >= MAX_PLAYERS) return 0;
 
-	switch(p1 & 0xff) {
-	case 0: // make new player
 		p = DoStartupNewPlayer(false);
 
 #ifdef ENABLE_NETWORK
@@ -661,63 +681,91 @@ int32 CmdPlayerCtrl(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 			}
 #ifdef ENABLE_NETWORK
 			if (_network_server) {
-				NetworkClientInfo *ci;
-				// UGLY! p2 is mis-used to fetch the client-id
-				ci = &_network_client_info[p2];
+				/* XXX - UGLY! p2 (pid) is mis-used to fetch the client-id, done at server-side
+				 * in network_server.c:838, function DEF_SERVER_RECEIVE_COMMAND(PACKET_CLIENT_COMMAND) */
+				NetworkClientInfo *ci = &_network_client_info[pid];
 				ci->client_playas = p->index + 1;
 				NetworkUpdateClientInfo(ci->client_index);
 
 				if (ci->client_playas != 0 && ci->client_playas <= MAX_PLAYERS) {
+					PlayerID player_backup = _local_player;
 					_network_player_info[p->index].months_empty = 0;
 
-					memcpy(_decode_parameters, ci->client_name, 32);
-					/* XXX - What are the consequents of this? It is needed, but is it bad? */
-					_docommand_recursive = 0;
-					DoCommandP(0, ci->client_playas-1, 0, NULL, CMD_CHANGE_PRESIDENT_NAME | CMD_MSG(STR_700D_CAN_T_CHANGE_PRESIDENT));
+					/* XXX - When a client joins, we automatically set its name to the
+					 * player's name (for some reason). As it stands now only the server
+					 * knows the client's name, so it needs to send out a "broadcast" to
+					 * do this. To achieve this we send a network command. However, it
+					 * uses _local_player to execute the command as.  To prevent abuse
+					 * (eg. only yourself can change your name/company), we 'cheat' by
+					 * impersonation _local_player as the server. Not the best solution;
+					 * but it works.
+					 * TODO: Perhaps this could be improved by when the client is ready
+					 * with joining to let it send itself the command, and not the server?
+					 * For example in network_client.c:534? */
+					_cmd_text = ci->client_name;
+					_local_player = ci->client_playas - 1;
+					NetworkSend_Command(0, 0, 0, CMD_CHANGE_PRESIDENT_NAME, NULL);
+					_local_player = player_backup;
 				}
 			}
-		} else {
-			if (_network_server) {
-				NetworkClientInfo *ci;
-				// UGLY! p2 is mis-used to fetch the client-id
-				ci = &_network_client_info[p2];
-				ci->client_playas = OWNER_SPECTATOR;
-				NetworkUpdateClientInfo(ci->client_index);
-			}
-#endif /* ENABLE_NETWORK */
+		} else if (_network_server) {
+				/* XXX - UGLY! p2 (pid) is mis-used to fetch the client-id, done at server-side
+				* in network_server.c:838, function DEF_SERVER_RECEIVE_COMMAND(PACKET_CLIENT_COMMAND) */
+			NetworkClientInfo *ci = &_network_client_info[pid];
+			ci->client_playas = OWNER_SPECTATOR;
+			NetworkUpdateClientInfo(ci->client_index);
 		}
-		break;
-	case 1: // make new ai player
+#else
+		}
+#endif /* ENABLE_NETWORK */
+	} break;
+
+	case 1: /* Make a new AI player */
+		if (!(flags & DC_EXEC)) return 0;
+
 		DoStartupNewPlayer(true);
 		break;
-	case 2: // delete player
+
+	case 2: { /* Delete a player */
+		Player *p;
+
+		if (p2 >= MAX_PLAYERS) return CMD_ERROR;
+
+		if (!(flags & DC_EXEC)) return 0;
+
 		p = DEREF_PLAYER(p2);
 
 		/* Only allow removal of HUMAN companies */
-		if (IS_HUMAN_PLAYER(p2)) {
+		if (IS_HUMAN_PLAYER(p->index)) {
 			/* Delete any open window of the company */
-			DeletePlayerWindows(p2);
+			DeletePlayerWindows(p->index);
 
 			/* Show the bankrupt news */
 			SetDParam(0, p->name_1);
 			SetDParam(1, p->name_2);
-			AddNewsItem( (StringID)(p2 + 16*3), NEWS_FLAGS(NM_CALLBACK, 0, NT_COMPANY_INFO, DNC_BANKRUPCY),0,0);
+			AddNewsItem( (StringID)(p->index + 16*3), NEWS_FLAGS(NM_CALLBACK, 0, NT_COMPANY_INFO, DNC_BANKRUPCY),0,0);
 
 			/* Remove the company */
-			ChangeOwnershipOfPlayerItems(p2, 255);
-			p->money64 = p->player_money = 100000000;
+			ChangeOwnershipOfPlayerItems(p->index, OWNER_SPECTATOR);
+			p->money64 = p->player_money = 100000000; // XXX - wtf?
 			p->is_active = false;
 		}
-		break;
+	} break;
 
-	case 3: // join player
-		pi = (byte)(p1 >> 8);
-		ChangeOwnershipOfPlayerItems(pi, (byte)(p1 >> 16));
-		DeletePlayerStuff(pi);
-		break;
+	case 3: { /* Merge a company (#1) into another company (#2), elimination company #1 */
+		PlayerID pid_old = p2 & 0xFFFF;
+		PlayerID pid_new = (p2 >> 16) & 0xFFFF;
+
+		if (pid_old >= MAX_PLAYERS || pid_new >= MAX_PLAYERS) return CMD_ERROR;
+
+		if (!(flags & DC_EXEC)) return CMD_ERROR;
+
+		ChangeOwnershipOfPlayerItems(pid_old, pid_new);
+		DeletePlayerStuff(pid_old);
+	} break;
+	default: return CMD_ERROR;
 	}
 
-
 	return 0;
 }
 
@@ -773,7 +821,7 @@ int8 SaveHighScoreValue(const Player *p)
 	for (i = 0; i < lengthof(_highscore_table[0]); i++) {
 		/* You are in the TOP5. Move all values one down and save us there */
 		if (hs[i].score <= score) {
-			byte buf[sizeof(hs[i].company)];
+			char buf[sizeof(hs[i].company)];
 
 			// move all elements one down starting from the replaced one
 			memmove(&hs[i + 1], &hs[i], sizeof(HighScore) * (lengthof(_highscore_table[0]) - i - 1));
@@ -818,7 +866,6 @@ int8 SaveHighScoreValueNetwork(void)
 
 	{
 		HighScore *hs;
-		byte buf[sizeof(_highscore_table[0]->company)];
 		Player* const *p_cur = &player_sort[0];
 		uint8 i;
 
@@ -826,6 +873,8 @@ int8 SaveHighScoreValueNetwork(void)
 
 		/* Copy over Top5 companies */
 		for (i = 0; i < lengthof(_highscore_table[LAST_HS_ITEM]) && i < (uint8)count; i++) {
+			char buf[sizeof(_highscore_table[0]->company)];
+
 			hs = &_highscore_table[LAST_HS_ITEM][i];
 			SetDParam(0, (*p_cur)->president_name_1);
 			SetDParam(1, (*p_cur)->president_name_2);
@@ -923,8 +972,10 @@ static const byte _player_desc[] = {
 	SLE_VAR(Player,block_preview,		SLE_UINT8),
 
 	SLE_VAR(Player,cargo_types,			SLE_UINT16),
-	SLE_VAR(Player,location_of_house,SLE_UINT16),
-	SLE_VAR(Player,last_build_coordinate,SLE_UINT16),
+	SLE_CONDVAR(Player, location_of_house,     SLE_FILE_U16 | SLE_VAR_U32, 0, 5),
+	SLE_CONDVAR(Player, location_of_house,     SLE_UINT32, 6, 255),
+	SLE_CONDVAR(Player, last_build_coordinate, SLE_FILE_U16 | SLE_VAR_U32, 0, 5),
+	SLE_CONDVAR(Player, last_build_coordinate, SLE_UINT32, 6, 255),
 	SLE_VAR(Player,inaugurated_year,SLE_UINT8),
 
 	SLE_ARR(Player,share_owners,		SLE_UINT8, 4),
@@ -967,7 +1018,8 @@ static const byte _player_economy_desc[] = {
 static const byte _player_ai_desc[] = {
 	SLE_VAR(PlayerAI,state,							SLE_UINT8),
 	SLE_VAR(PlayerAI,tick,							SLE_UINT8),
-	SLE_VAR(PlayerAI,state_counter,			SLE_UINT16),
+	SLE_CONDVAR(PlayerAI,state_counter, SLE_FILE_U16 | SLE_VAR_U32, 0, 12),
+	SLE_CONDVAR(PlayerAI,state_counter, SLE_UINT32, 13, 255),
 	SLE_VAR(PlayerAI,timeout_counter,		SLE_UINT16),
 
 	SLE_VAR(PlayerAI,state_mode,				SLE_UINT8),
@@ -983,13 +1035,17 @@ static const byte _player_ai_desc[] = {
 
 	SLE_VAR(PlayerAI,route_type_mask,		SLE_UINT8),
 
-	SLE_VAR(PlayerAI,start_tile_a,			SLE_UINT16),
-	SLE_VAR(PlayerAI,cur_tile_a,				SLE_UINT16),
+	SLE_CONDVAR(PlayerAI, start_tile_a, SLE_FILE_U16 | SLE_VAR_U32, 0, 5),
+	SLE_CONDVAR(PlayerAI, start_tile_a, SLE_UINT32, 6, 255),
+	SLE_CONDVAR(PlayerAI, cur_tile_a,   SLE_FILE_U16 | SLE_VAR_U32, 0, 5),
+	SLE_CONDVAR(PlayerAI, cur_tile_a,   SLE_UINT32, 6, 255),
 	SLE_VAR(PlayerAI,start_dir_a,				SLE_UINT8),
 	SLE_VAR(PlayerAI,cur_dir_a,					SLE_UINT8),
 
-	SLE_VAR(PlayerAI,start_tile_b,			SLE_UINT16),
-	SLE_VAR(PlayerAI,cur_tile_b,				SLE_UINT16),
+	SLE_CONDVAR(PlayerAI, start_tile_b, SLE_FILE_U16 | SLE_VAR_U32, 0, 5),
+	SLE_CONDVAR(PlayerAI, start_tile_b, SLE_UINT32, 6, 255),
+	SLE_CONDVAR(PlayerAI, cur_tile_b,   SLE_FILE_U16 | SLE_VAR_U32, 0, 5),
+	SLE_CONDVAR(PlayerAI, cur_tile_b,   SLE_UINT32, 6, 255),
 	SLE_VAR(PlayerAI,start_dir_b,				SLE_UINT8),
 	SLE_VAR(PlayerAI,cur_dir_b,					SLE_UINT8),
 
@@ -1004,8 +1060,10 @@ static const byte _player_ai_desc[] = {
 };
 
 static const byte _player_ai_build_rec_desc[] = {
-	SLE_VAR(AiBuildRec,spec_tile,			SLE_UINT16),
-	SLE_VAR(AiBuildRec,use_tile,			SLE_UINT16),
+	SLE_CONDVAR(AiBuildRec,spec_tile, SLE_FILE_U16 | SLE_VAR_U32, 0, 5),
+	SLE_CONDVAR(AiBuildRec,spec_tile, SLE_UINT32, 6, 255),
+	SLE_CONDVAR(AiBuildRec,use_tile,  SLE_FILE_U16 | SLE_VAR_U32, 0, 5),
+	SLE_CONDVAR(AiBuildRec,use_tile,  SLE_UINT32, 6, 255),
 	SLE_VAR(AiBuildRec,rand_rng,			SLE_UINT8),
 	SLE_VAR(AiBuildRec,cur_building_rule,SLE_UINT8),
 	SLE_VAR(AiBuildRec,unk6,					SLE_UINT8),
