@@ -1,5 +1,8 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "string.h"
+#include "strings.h" // XXX GetCurrentCurrencyRate()
+#include "table/sprites.h"
 #include "table/strings.h"
 #include "window.h"
 #include "gui.h"
@@ -67,7 +70,7 @@ static int GetCurRes(void)
 
 static inline bool RoadVehiclesAreBuilt(void)
 {
-	Vehicle *v;
+	const Vehicle *v;
 	FOR_ALL_VEHICLES(v) {
 		if (v->type == VEH_Road) return true;
 	}
@@ -76,17 +79,17 @@ static inline bool RoadVehiclesAreBuilt(void)
 
 static void GameOptionsWndProc(Window *w, WindowEvent *e)
 {
-	switch(e->event) {
+	switch (e->event) {
 	case WE_PAINT: {
 		int i;
 		StringID str = STR_02BE_DEFAULT;
 		w->disabled_state = (_vehicle_design_names & 1) ? (++str, 0) : (1 << 21);
 		SetDParam(0, str);
-		SetDParam(1, _currency_string_list[_opt_mod_ptr->currency]);
-		SetDParam(2, _opt_mod_ptr->kilometers + STR_0139_IMPERIAL_MILES);
-		SetDParam(3, STR_02E9_DRIVE_ON_LEFT + _opt_mod_ptr->road_side);
-		SetDParam(4, STR_TOWNNAME_ORIGINAL_ENGLISH + _opt_mod_ptr->town_name);
-		SetDParam(5, _autosave_dropdown[_opt_mod_ptr->autosave]);
+		SetDParam(1, _currency_string_list[_opt_ptr->currency]);
+		SetDParam(2, _opt_ptr->kilometers + STR_0139_IMPERIAL_MILES);
+		SetDParam(3, STR_02E9_DRIVE_ON_LEFT + _opt_ptr->road_side);
+		SetDParam(4, STR_TOWNNAME_ORIGINAL_ENGLISH + _opt_ptr->town_name);
+		SetDParam(5, _autosave_dropdown[_opt_ptr->autosave]);
 		SetDParam(6, SPECSTR_LANGUAGE_START + _dynlang.curr);
 		i = GetCurRes();
 		SetDParam(7, i == _num_resolutions ? STR_RES_OTHER : SPECSTR_RESOLUTION_START + i);
@@ -98,60 +101,60 @@ static void GameOptionsWndProc(Window *w, WindowEvent *e)
 	}	break;
 
 	case WE_CLICK:
-		switch(e->click.widget) {
-		case 5: /* Setup currencies dropdown */
-			ShowDropDownMenu(w, _currency_string_list, _opt_mod_ptr->currency, e->click.widget, _game_mode == GM_MENU ? 0 : ~GetMaskOfAllowedCurrencies(), 0);
+		switch (e->click.widget) {
+		case 4: case 5: /* Setup currencies dropdown */
+			ShowDropDownMenu(w, _currency_string_list, _opt_ptr->currency, 5, _game_mode == GM_MENU ? 0 : ~GetMaskOfAllowedCurrencies(), 0);
 			return;
-		case 8: /* Setup distance unit dropdown */
-			ShowDropDownMenu(w, _distances_dropdown, _opt_mod_ptr->kilometers, e->click.widget, 0, 0);
+		case 7: case 8: /* Setup distance unit dropdown */
+			ShowDropDownMenu(w, _distances_dropdown, _opt_ptr->kilometers, 8, 0, 0);
 			return;
-		case 11: { /* Setup road-side dropdown */
+		case 10: case 11: { /* Setup road-side dropdown */
 			int i = 0;
 
 			/* You can only change the drive side if you are in the menu or ingame with
 			 * no vehicles present. In a networking game only the server can change it */
 			if ((_game_mode != GM_MENU && RoadVehiclesAreBuilt()) || (_networking && !_network_server))
-				i = (-1) ^ (1 << _opt_mod_ptr->road_side); // disable the other value
+				i = (-1) ^ (1 << _opt_ptr->road_side); // disable the other value
 
-			ShowDropDownMenu(w, _driveside_dropdown, _opt_mod_ptr->road_side, e->click.widget, i, 0);
+			ShowDropDownMenu(w, _driveside_dropdown, _opt_ptr->road_side, 11, i, 0);
 		} return;
-		case 14: { /* Setup townname dropdown */
-			int i = _opt_mod_ptr->town_name;
-			ShowDropDownMenu(w, BuildDynamicDropdown(STR_TOWNNAME_ORIGINAL_ENGLISH, SPECSTR_TOWNNAME_LAST - SPECSTR_TOWNNAME_START + 1), i, e->click.widget, (_game_mode == GM_MENU) ? 0 : (-1) ^ (1 << i), 0);
+		case 13: case 14: { /* Setup townname dropdown */
+			int i = _opt_ptr->town_name;
+			ShowDropDownMenu(w, BuildDynamicDropdown(STR_TOWNNAME_ORIGINAL_ENGLISH, SPECSTR_TOWNNAME_LAST - SPECSTR_TOWNNAME_START + 1), i, 14, (_game_mode == GM_MENU) ? 0 : (-1) ^ (1 << i), 0);
 			return;
 		}
-		case 17: /* Setup autosave dropdown */
-			ShowDropDownMenu(w, _autosave_dropdown, _opt_mod_ptr->autosave, e->click.widget, 0, 0);
+		case 16: case 17: /* Setup autosave dropdown */
+			ShowDropDownMenu(w, _autosave_dropdown, _opt_ptr->autosave, 17, 0, 0);
 			return;
-		case 20: /* Setup customized vehicle-names dropdown */
-			ShowDropDownMenu(w, _designnames_dropdown, (_vehicle_design_names&1)?1:0, e->click.widget, (_vehicle_design_names&2)?0:2, 0);
+		case 19: case 20: /* Setup customized vehicle-names dropdown */
+			ShowDropDownMenu(w, _designnames_dropdown, (_vehicle_design_names & 1) ? 1 : 0, 20, (_vehicle_design_names & 2) ? 0 : 2, 0);
 			return;
 		case 21: /* Save customized vehicle-names to disk */
 			return;
-		case 24: /* Setup interface language dropdown */
-			ShowDropDownMenu(w, _dynlang.dropdown, _dynlang.curr, e->click.widget, 0, 0);
+		case 23: case 24: /* Setup interface language dropdown */
+			ShowDropDownMenu(w, _dynlang.dropdown, _dynlang.curr, 24, 0, 0);
 			return;
-		case 27: /* Setup resolution dropdown */
-			ShowDropDownMenu(w, BuildDynamicDropdown(SPECSTR_RESOLUTION_START, _num_resolutions), GetCurRes(), e->click.widget, 0, 0);
+		case 26: case 27: /* Setup resolution dropdown */
+			ShowDropDownMenu(w, BuildDynamicDropdown(SPECSTR_RESOLUTION_START, _num_resolutions), GetCurRes(), 27, 0, 0);
 			return;
 		case 28: /* Click fullscreen on/off */
 			(_fullscreen) ? CLRBIT(w->click_state, 28) : SETBIT(w->click_state, 28);
 			ToggleFullScreen(!_fullscreen); // toggle full-screen on/off
 			SetWindowDirty(w);
 			return;
-		case 31: /* Setup screenshot format dropdown */
-			ShowDropDownMenu(w, BuildDynamicDropdown(SPECSTR_SCREENSHOT_START, _num_screenshot_formats), _cur_screenshot_format, e->click.widget, 0, 0);
+		case 30: case 31: /* Setup screenshot format dropdown */
+			ShowDropDownMenu(w, BuildDynamicDropdown(SPECSTR_SCREENSHOT_START, _num_screenshot_formats), _cur_screenshot_format, 31, 0, 0);
 			return;
 		}
 		break;
 
 	case WE_DROPDOWN_SELECT:
-		switch(e->dropdown.button) {
+		switch (e->dropdown.button) {
 		case 20: /* Vehicle design names */
 			if (e->dropdown.index == 0) {
 				DeleteCustomEngineNames();
 				MarkWholeScreenDirty();
-			} else if (!(_vehicle_design_names&1)) {
+			} else if (!(_vehicle_design_names & 1)) {
 				LoadCustomEngineNames();
 				MarkWholeScreenDirty();
 			}
@@ -159,25 +162,27 @@ static void GameOptionsWndProc(Window *w, WindowEvent *e)
 		case 5: /* Currency */
 			if (e->dropdown.index == 23)
 				ShowCustCurrency();
-			_opt_mod_ptr->currency = _opt.currency = e->dropdown.index;
+			_opt_ptr->currency = e->dropdown.index;
 			MarkWholeScreenDirty();
 			break;
 		case 8: /* Distance units */
-			_opt_mod_ptr->kilometers = e->dropdown.index;
+			_opt_ptr->kilometers = e->dropdown.index;
 			MarkWholeScreenDirty();
 			break;
 		case 11: /* Road side */
-			if (_opt_mod_ptr->road_side != e->dropdown.index) { // only change if setting changed
-				DoCommandP(0, e->dropdown.index, 0, NULL, CMD_SET_ROAD_DRIVE_SIDE | CMD_MSG(STR_EMPTY));
+			if (_opt_ptr->road_side != e->dropdown.index) { // only change if setting changed
+				DoCommandP(0, e->dropdown.index, 0, NULL, CMD_SET_ROAD_DRIVE_SIDE | CMD_MSG(STR_00B4_CAN_T_DO_THIS));
 				MarkWholeScreenDirty();
 			}
 			break;
 		case 14: /* Town names */
-			if (_game_mode == GM_MENU)
-				DoCommandP(0, e->dropdown.index, 0, NULL, CMD_SET_TOWN_NAME_TYPE | CMD_MSG(STR_EMPTY));
+			if (_game_mode == GM_MENU) {
+				_opt_ptr->town_name = e->dropdown.index;
+				InvalidateWindow(WC_GAME_OPTIONS, 0);
+			}
 			break;
 		case 17: /* Autosave options */
-			_opt_mod_ptr->autosave = e->dropdown.index;
+			_opt_ptr->autosave = e->dropdown.index;
 			SetWindowDirty(w);
 			break;
 		case 24: /* Change interface language */
@@ -202,25 +207,24 @@ static void GameOptionsWndProc(Window *w, WindowEvent *e)
 
 }
 
+/** Change the side of the road vehicles drive on (server only).
+ * @param x,y unused
+ * @param p1 the side of the road; 0 = left side and 1 = right side
+ * @param p2 unused
+ */
 int32 CmdSetRoadDriveSide(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
+	/* Check boundaries and you can only change this if NO vehicles have been built yet,
+	 * except in the intro-menu where of course it's always possible to do so. */
+	if (p1 > 1 || (_game_mode != GM_MENU && RoadVehiclesAreBuilt())) return CMD_ERROR;
+
 	if (flags & DC_EXEC) {
-		_opt_mod_ptr->road_side = p1;
+		_opt_ptr->road_side = p1;
 		InvalidateWindow(WC_GAME_OPTIONS,0);
 	}
 	return 0;
 }
 
-int32 CmdSetTownNameType(int x, int y, uint32 flags, uint32 p1, uint32 p2)
-{
-	if (flags & DC_EXEC) {
-		_opt_mod_ptr->town_name = p1;
-		InvalidateWindow(WC_GAME_OPTIONS,0);
-	}
-	return 0;
-}
-
-
 static const Widget _game_options_widgets[] = {
 {   WWT_CLOSEBOX,   RESIZE_NONE,    14,     0,    10,     0,    13, STR_00C5,								STR_018B_CLOSE_WINDOW},
 {    WWT_CAPTION,   RESIZE_NONE,    14,    11,   369,     0,    13, STR_00B1_GAME_OPTIONS,		STR_018C_WINDOW_TITLE_DRAG_THIS},
@@ -285,37 +289,58 @@ typedef struct {
 } GameSettingData;
 
 static const GameSettingData _game_setting_info[] = {
-	{0,7,1,0},
-	{0,3,1,STR_6830_IMMEDIATE},
-	{0,2,1,STR_6816_LOW},
-	{0,3,1,STR_26816_NONE},
-	{100,500,50,0},
-	{2,4,1,0},
-	{0,2,1,STR_6820_LOW},
-	{0,4,1,STR_681B_VERY_SLOW},
-	{0,2,1,STR_6820_LOW},
-	{0,2,1,STR_6823_NONE},
-	{0,3,1,STR_6826_X1_5},
-	{0,2,1,STR_6820_LOW},
-	{0,3,1,STR_682A_VERY_FLAT},
-	{0,3,1,STR_VERY_LOW},
-	{0,1,1,STR_682E_STEADY},
-	{0,1,1,STR_6834_AT_END_OF_LINE_AND_AT_STATIONS},
-	{0,1,1,STR_6836_OFF},
-	{0,2,1,STR_6839_PERMISSIVE},
+	{  0,   7,  1, STR_NULL},
+	{  0,   3,  1, STR_6830_IMMEDIATE},
+	{  0,   2,  1, STR_6816_LOW},
+	{  0,   3,  1, STR_26816_NONE},
+	{100, 500, 50, STR_NULL},
+	{  2,   4,  1, STR_NULL},
+	{  0,   2,  1, STR_6820_LOW},
+	{  0,   4,  1, STR_681B_VERY_SLOW},
+	{  0,   2,  1, STR_6820_LOW},
+	{  0,   2,  1, STR_6823_NONE},
+	{  0,   3,  1, STR_6826_X1_5},
+	{  0,   2,  1, STR_6820_LOW},
+	{  0,   3,  1, STR_682A_VERY_FLAT},
+	{  0,   3,  1, STR_VERY_LOW},
+	{  0,   1,  1, STR_682E_STEADY},
+	{  0,   1,  1, STR_6834_AT_END_OF_LINE_AND_AT_STATIONS},
+	{  0,   1,  1, STR_6836_OFF},
+	{  0,   2,  1, STR_6839_PERMISSIVE},
 };
 
 static inline bool GetBitAndShift(uint32 *b)
 {
 	uint32 x = *b;
 	*b >>= 1;
-	return (x&1) != 0;
+	return HASBIT(x, 0);
 }
 
-static const int16 _default_game_diff[3][GAME_DIFFICULTY_NUM] = {
-	{2, 2, 1, 3, 300, 2, 0, 2, 0, 1, 2, 0, 1, 0, 0, 0, 0, 0},
-	{4, 1, 1, 2, 150, 3, 1, 3, 1, 2, 1, 1, 2, 1, 1, 1, 1, 1},
-	{7, 0, 2, 2, 100, 4, 1, 3, 2, 2, 0, 2, 3, 2, 1, 1, 1, 2},
+/*
+	A: competitors
+	B: start time in months / 3
+	C: town count (2 = high, 0 = low)
+	D: industry count (3 = high, 0 = none)
+	E: inital loan / 1000 (in GBP)
+	F: interest rate
+	G: running costs (0 = low, 2 = high)
+	H: construction speed of competitors (0 = very slow, 4 = very fast)
+	I: intelligence (0-2)
+	J: breakdowns(0 = off, 2 = normal)
+	K: subsidy multiplier (0 = 1.5, 3 = 4.0)
+	L: construction cost (0-2)
+	M: terrain type (0 = very flat, 3 = mountainous)
+	N: amount of water (0 = very low, 3 = high)
+	O: economy (0 = steady, 1 = fluctuating)
+	P: Train reversing (0 = end of line + stations, 1 = end of line)
+	Q: disasters
+	R: area restructuring (0 = permissive, 2 = hostile)
+*/
+static const int16 _default_game_diff[3][GAME_DIFFICULTY_NUM] = { /*
+	 A, B, C, D,   E, F, G, H, I, J, K, L, M, N, O, P, Q, R*/
+	{2, 2, 1, 3, 300, 2, 0, 2, 0, 1, 2, 0, 1, 0, 0, 0, 0, 0},	//easy
+	{4, 1, 1, 2, 150, 3, 1, 3, 1, 2, 1, 1, 2, 1, 1, 1, 1, 1},	//medium
+	{7, 0, 2, 2, 100, 4, 1, 3, 2, 2, 0, 2, 3, 2, 1, 1, 1, 2},	//hard
 };
 
 void SetDifficultyLevel(int mode, GameOptions *gm_opt)
@@ -325,7 +350,7 @@ void SetDifficultyLevel(int mode, GameOptions *gm_opt)
 
 	gm_opt->diff_level = mode;
 	if (mode != 3) { // not custom
-		for(i = 0; i != GAME_DIFFICULTY_NUM; i++)
+		for (i = 0; i != GAME_DIFFICULTY_NUM; i++)
 			((int*)&gm_opt->diff)[i] = _default_game_diff[mode][i];
 	}
 }
@@ -337,74 +362,78 @@ enum {
 	GAMEDIFF_WND_ROWSIZE    = 9
 };
 
+// Temporary holding place of values in the difficulty window until 'Save' is clicked
+static GameOptions _opt_mod_temp;
+// 0x383E = (1 << 13) | (1 << 12) | (1 << 11) | (1 << 5) | (1 << 4) | (1 << 3) | (1 << 2) | (1 << 1)
+#define DIFF_INGAME_DISABLED_BUTTONS 0x383E
+
 static void GameDifficultyWndProc(Window *w, WindowEvent *e)
 {
-	switch(e->event) {
-	case WE_PAINT: {
-		uint32 click_a, click_b, disabled;
-		int i;
-		int x,y,value;
-
-		w->click_state = (1 << 3) << _opt_mod_temp.diff_level;
+	switch (e->event) {
+		case WE_CREATE: /* Setup disabled buttons when creating window */
+		// disable all other difficulty buttons during gameplay except for 'custom'
 		w->disabled_state = (_game_mode != GM_NORMAL) ? 0 : (1 << 3) | (1 << 4) | (1 << 5) | (1 << 6);
+
+		if (_game_mode == GM_EDITOR) SETBIT(w->disabled_state, 7);
+
 		if (_networking) {
 			SETBIT(w->disabled_state, 7); // disable highscore chart in multiplayer
 			if (!_network_server)
 				SETBIT(w->disabled_state, 10); // Disable save-button in multiplayer (and if client)
 		}
+		break;
+	case WE_PAINT: {
+		uint32 click_a, click_b, disabled;
+		int i;
+		int y, value;
 
+		w->click_state = (1 << 3) << _opt_mod_temp.diff_level; // have current difficulty button clicked
 		DrawWindowWidgets(w);
 
 		click_a = _difficulty_click_a;
 		click_b = _difficulty_click_b;
 
-		/* XXX - This is most likely the worst way I have ever seen
-		     to disable some buttons and to enable others.
-		     What the value means, is this:
-		       if bit1 is enabled, setting 1 is disabled
-		       then it is shifted to the left, and the story
-		       repeats....
-		   -- TrueLight */
-		disabled = _game_mode == GM_NORMAL ? 0x383E : 0;
+		/* XXX - Disabled buttons in normal gameplay. Bitshifted for each button to see if
+		 * that bit is set. If it is set, the button is disabled */
+		disabled = (_game_mode == GM_NORMAL) ? DIFF_INGAME_DISABLED_BUTTONS : 0;
 
-		x = 0;
 		y = GAMEDIFF_WND_TOP_OFFSET;
 		for (i = 0; i != GAME_DIFFICULTY_NUM; i++) {
-			DrawFrameRect(x+5, y, x+5+8, y+8, 3, GetBitAndShift(&click_a)?0x20:0);
-			DrawFrameRect(x+15, y, x+15+8, y+8, 3, GetBitAndShift(&click_b)?0x20:0);
+			DrawFrameRect( 5, y,  5 + 8, y + 8, 3, GetBitAndShift(&click_a) ? (1 << 5) : 0);
+			DrawFrameRect(15, y, 15 + 8, y + 8, 3, GetBitAndShift(&click_b) ? (1 << 5) : 0);
 			if (GetBitAndShift(&disabled) || (_networking && !_network_server)) {
-				int color = 0x8000 | _color_list[3].unk2;
-				GfxFillRect(x+6, y+1, x+6+8, y+8, color);
-				GfxFillRect(x+16, y+1, x+16+8, y+8, color);
+				int color = PALETTE_MODIFIER_COLOR | _color_list[3].unk2;
+				GfxFillRect( 6, y + 1,  6 + 8, y + 8, color);
+				GfxFillRect(16, y + 1, 16 + 8, y + 8, color);
 			}
 
-			DrawStringCentered(x+10, y, STR_6819, 0);
-			DrawStringCentered(x+20, y, STR_681A, 0);
+			DrawStringCentered(10, y, STR_6819, 0);
+			DrawStringCentered(20, y, STR_681A, 0);
 
 
 			value = _game_setting_info[i].str + ((int*)&_opt_mod_temp.diff)[i];
-			if (i == 4) value *= 1000; // handle currency option
+			if (i == 4) value *= 1000; // XXX - handle currency option
 			SetDParam(0, value);
-			DrawString(x+30, y, STR_6805_MAXIMUM_NO_COMPETITORS + i, 0);
+			DrawString(30, y, STR_6805_MAXIMUM_NO_COMPETITORS + i, 0);
 
 			y += GAMEDIFF_WND_ROWSIZE + 2; // space items apart a bit
 		}
 	} break;
 
 	case WE_CLICK:
-		switch(e->click.widget) {
-		case 8: {
-			int x,y;
+		switch (e->click.widget) {
+		case 8: { /* Difficulty settings widget, decode click */
+			const GameSettingData *info;
+			int x, y;
 			uint btn, dis;
 			int val;
-			const GameSettingData *info;
 
 			// Don't allow clients to make any changes
 			if  (_networking && !_network_server)
 				return;
 
 			x = e->click.pt.x - 5;
-			if (!IS_INT_INSIDE(x, 0, 21))
+			if (!IS_INT_INSIDE(x, 0, 21)) // Button area
 				return;
 
 			y = e->click.pt.y - GAMEDIFF_WND_TOP_OFFSET;
@@ -417,7 +446,7 @@ static void GameDifficultyWndProc(Window *w, WindowEvent *e)
 				return;
 
 			// Clicked disabled button?
-			dis = (_game_mode == GM_NORMAL) ? 0x383E : 0;
+			dis = (_game_mode == GM_NORMAL) ? DIFF_INGAME_DISABLED_BUTTONS : 0;
 
 			if (HASBIT(dis, btn))
 				return;
@@ -426,7 +455,7 @@ static void GameDifficultyWndProc(Window *w, WindowEvent *e)
 
 			val = ((int*)&_opt_mod_temp.diff)[btn];
 
-			info = &_game_setting_info[btn];
+			info = &_game_setting_info[btn]; // get information about the difficulty setting
 			if (x >= 10) {
 				// Increase button clicked
 				val = min(val + info->step, info->max);
@@ -441,8 +470,7 @@ static void GameDifficultyWndProc(Window *w, WindowEvent *e)
 			((int*)&_opt_mod_temp.diff)[btn] = val;
 			SetDifficultyLevel(3, &_opt_mod_temp); // set difficulty level to custom
 			SetWindowDirty(w);
-			break;
-		}
+		}	break;
 		case 3: case 4: case 5: case 6: /* Easy / Medium / Hard / Custom */
 			// temporarily change difficulty level
 			SetDifficultyLevel(e->click.widget - 3, &_opt_mod_temp);
@@ -456,7 +484,7 @@ static void GameDifficultyWndProc(Window *w, WindowEvent *e)
 			for (btn = 0; btn != GAME_DIFFICULTY_NUM; btn++) {
 				val = ((int*)&_opt_mod_temp.diff)[btn];
 				// if setting has changed, change it
-				if (val != ((int*)&_opt_mod_ptr->diff)[btn])
+				if (val != ((int*)&_opt_ptr->diff)[btn])
 					DoCommandP(0, btn, val, NULL, CMD_CHANGE_DIFFICULTY_LEVEL);
 			}
 			DoCommandP(0, -1, _opt_mod_temp.diff_level, NULL, CMD_CHANGE_DIFFICULTY_LEVEL);
@@ -468,13 +496,12 @@ static void GameDifficultyWndProc(Window *w, WindowEvent *e)
 				StartupEconomy();
 			break;
 		}
-		case 11: // Cancel button - close window
+		case 11: /* Cancel button - close window, abandon changes */
 			DeleteWindow(w);
 			break;
-		}
-		break;
+	} break;
 
-	case WE_MOUSELOOP:
+	case WE_MOUSELOOP: /* Handle the visual 'clicking' of the buttons */
 		if (_difficulty_timeout != 0 && !--_difficulty_timeout) {
 			_difficulty_click_a = 0;
 			_difficulty_click_b = 0;
@@ -484,6 +511,8 @@ static void GameDifficultyWndProc(Window *w, WindowEvent *e)
 	}
 }
 
+#undef DIFF_INGAME_DISABLED_BUTTONS
+
 static const Widget _game_difficulty_widgets[] = {
 {   WWT_CLOSEBOX,   RESIZE_NONE,    10,     0,    10,     0,    13, STR_00C5,									STR_018B_CLOSE_WINDOW},
 {    WWT_CAPTION,   RESIZE_NONE,    10,    11,   369,     0,    13, STR_6800_DIFFICULTY_LEVEL,	STR_018C_WINDOW_TITLE_DRAG_THIS},
@@ -511,10 +540,9 @@ static const WindowDesc _game_difficulty_desc = {
 void ShowGameDifficulty(void)
 {
 	DeleteWindowById(WC_GAME_OPTIONS, 0);
-	/*	copy current settings to temporary holding place
-	 *	change that when setting stuff, copy back on clicking 'OK'
-	 */
-	memcpy(&_opt_mod_temp, _opt_mod_ptr, sizeof(GameOptions));
+	/* Copy current settings (ingame or in intro) to temporary holding place
+	 * change that when setting stuff, copy back on clicking 'OK' */
+	memcpy(&_opt_mod_temp, _opt_ptr, sizeof(GameOptions));
 	AllocateWindowDesc(&_game_difficulty_desc);
 }
 
@@ -560,6 +588,12 @@ static int32 InValidateDetailsWindow(int32 p1)
 	return 0;
 }
 
+static int32 InvalidateStationBuildWindow(int32 p1)
+{
+	InvalidateWindow(WC_BUILD_STATION, 0);
+	return 0;
+}
+
 /* Check service intervals of vehicles, p1 is value of % or day based servicing */
 static int32 CheckInterval(int32 p1)
 {
@@ -585,14 +619,14 @@ static int32 CheckInterval(int32 p1)
 typedef int32 PatchButtonClick(int32);
 
 typedef struct PatchEntry {
-	byte type;										// type of selector
-	byte flags;										// selector flags
-	StringID str;									// string with descriptive text
-	char console_name[40];				// the name this patch has in console
-	void *variable;								// pointer to the variable
-	int32 min,max;								// range for spinbox setting
-	uint32 step;									// step for spinbox
-	PatchButtonClick *click_proc;	// callback procedure
+  byte type;                    // type of selector
+  byte flags;                   // selector flags
+  StringID str;                 // string with descriptive text
+  char console_name[40];        // the name this patch has in console
+  void *variable;               // pointer to the variable
+  int32 min, max;               // range for spinbox setting
+  uint32 step;                  // step for spinbox
+  PatchButtonClick *click_proc; // callback procedure
 } PatchEntry;
 
 enum {
@@ -602,7 +636,7 @@ enum {
 	PE_UINT16		= 3,
 	PE_INT32		= 4,
 	PE_CURRENCY	= 5,
-
+	// selector flags
 	PF_0ISDIS				= 1 << 0,
 	PF_NOCOMMA			= 1 << 1,
 	PF_MULTISTRING	= 1 << 2,
@@ -622,6 +656,9 @@ static const PatchEntry _patches_ui[] = {
 	{PE_UINT8,	PF_0ISDIS | PF_PLAYERBASED, STR_CONFIG_PATCHES_SNAP_RADIUS, "window_snap_radius", &_patches.window_snap_radius,     1, 32,  1, NULL},
 	{PE_BOOL,		PF_PLAYERBASED, STR_CONFIG_PATCHES_INVISIBLE_TREES,	"invisible_trees", &_patches.invisible_trees,					0,  1,  1, &InvisibleTreesActive},
 	{PE_BOOL,		PF_PLAYERBASED, STR_CONFIG_PATCHES_POPULATION_IN_LABEL, "population_in_label", &_patches.population_in_label, 0, 1, 1, &PopulationInLabelActive},
+
+	{PE_INT32, 0, STR_CONFIG_PATCHES_MAP_X, "map_x", &_patches.map_x, 6, 11, 1, NULL},
+	{PE_INT32, 0, STR_CONFIG_PATCHES_MAP_Y, "map_y", &_patches.map_y, 6, 11, 1, NULL},
 };
 
 static const PatchEntry _patches_construction[] = {
@@ -637,11 +674,13 @@ static const PatchEntry _patches_construction[] = {
 
 static const PatchEntry _patches_vehicles[] = {
 	{PE_BOOL,		0, STR_CONFIG_PATCHES_REALISTICACCEL,		"realistic_acceleration", &_patches.realistic_acceleration,		0,  0,  0, NULL},
-	{PE_BOOL,		0, STR_CONFIG_PATCHES_MAMMOTHTRAINS,		"mammoth_trains", &_patches.mammoth_trains,						0,  0,  0, NULL},
-	{PE_BOOL,		0, STR_CONFIG_PATCHES_GOTODEPOT,				"goto_depot", &_patches.gotodepot,								0,  0,  0, NULL},
-	{PE_BOOL,		0, STR_CONFIG_PATCHES_ROADVEH_QUEUE,		"roadveh_queue", &_patches.roadveh_queue,						0,  0,  0, NULL},
-	{PE_BOOL,		0, STR_CONFIG_PATCHES_NEW_DEPOT_FINDING,"depot_finding", &_patches.new_depot_finding,				0,  0,  0, NULL},
+	{PE_BOOL,		0, STR_CONFIG_PATCHES_FORBID_90_DEG,		"forbid_90_deg", 		&_patches.forbid_90_deg,						0,  0,  0, NULL},
+	{PE_BOOL,		0, STR_CONFIG_PATCHES_MAMMOTHTRAINS,		"mammoth_trains", 	&_patches.mammoth_trains,						0,  0,  0, NULL},
+	{PE_BOOL,		0, STR_CONFIG_PATCHES_GOTODEPOT,				"goto_depot", 			&_patches.gotodepot,								0,  0,  0, NULL},
+	{PE_BOOL,		0, STR_CONFIG_PATCHES_ROADVEH_QUEUE,		"roadveh_queue", 		&_patches.roadveh_queue,						0,  0,  0, NULL},
+	{PE_BOOL,		0, STR_CONFIG_PATCHES_NEW_DEPOT_FINDING,"depot_finding", 		&_patches.new_depot_finding,				0,  0,  0, NULL},
 	{PE_BOOL,		0, STR_CONFIG_PATCHES_NEW_TRAIN_PATHFIND,"new_pathfinding", &_patches.new_pathfinding,	0,  0,  0, NULL},
+	{PE_BOOL,		0, STR_CONFIG_PATCHES_NEW_PATHFINDING_ALL, "new_pathfinding_all", &_patches.new_pathfinding_all,		0,  0,  0, NULL},
 
 	{PE_BOOL,		PF_PLAYERBASED, STR_CONFIG_PATCHES_WARN_INCOME_LESS, "train_income_warn", &_patches.train_income_warn,				0,  0,  0, NULL},
 	{PE_UINT8,	PF_MULTISTRING | PF_PLAYERBASED, STR_CONFIG_PATCHES_ORDER_REVIEW, "order_review_system", &_patches.order_review_system,0,2,  1, NULL},
@@ -652,10 +691,10 @@ static const PatchEntry _patches_vehicles[] = {
 	{PE_INT16,	PF_PLAYERBASED, STR_CONFIG_PATCHES_AUTORENEW_MONTHS, "autorenew_months", &_patches.autorenew_months,				-12, 12,  1, NULL},
 	{PE_CURRENCY, PF_PLAYERBASED, STR_CONFIG_PATCHES_AUTORENEW_MONEY,"autorenew_money", &_patches.autorenew_money,					0, 2000000, 100000, NULL},
 
-	{PE_UINT8,	0, STR_CONFIG_PATCHES_MAX_TRAINS,				"max_trains", &_patches.max_trains,								0,240, 10, NULL},
-	{PE_UINT8,	0, STR_CONFIG_PATCHES_MAX_ROADVEH,			"max_roadveh", &_patches.max_roadveh,							0,240, 10, NULL},
-	{PE_UINT8,	0, STR_CONFIG_PATCHES_MAX_AIRCRAFT,			"max_aircraft", &_patches.max_aircraft,							0,240, 10, NULL},
-	{PE_UINT8,	0, STR_CONFIG_PATCHES_MAX_SHIPS,				"max_ships", &_patches.max_ships,								0,240, 10, NULL},
+	{PE_UINT16,	0, STR_CONFIG_PATCHES_MAX_TRAINS,				"max_trains", &_patches.max_trains,								0,5000, 50, NULL},
+	{PE_UINT16,	0, STR_CONFIG_PATCHES_MAX_ROADVEH,			"max_roadveh", &_patches.max_roadveh,							0,5000, 50, NULL},
+	{PE_UINT16,	0, STR_CONFIG_PATCHES_MAX_AIRCRAFT,			"max_aircraft", &_patches.max_aircraft,						0,5000, 50, NULL},
+	{PE_UINT16,	0, STR_CONFIG_PATCHES_MAX_SHIPS,				"max_ships", &_patches.max_ships,									0,5000, 50, NULL},
 
 	{PE_BOOL,		0, STR_CONFIG_PATCHES_SERVINT_ISPERCENT,"servint_isperfect",&_patches.servint_ispercent,				0,  0,  0, &CheckInterval},
 	{PE_UINT16, PF_0ISDIS, STR_CONFIG_PATCHES_SERVINT_TRAINS,		"servint_trains",   &_patches.servint_trains,		5,800,  5, &InValidateDetailsWindow},
@@ -672,7 +711,7 @@ static const PatchEntry _patches_stations[] = {
 	{PE_BOOL,		0, STR_CONFIG_PATCHES_SELECTGOODS,			"select_goods",  &_patches.selectgoods,							0,  0,  0, NULL},
 	{PE_BOOL,		0, STR_CONFIG_PATCHES_NEW_NONSTOP,			"new_nonstop", &_patches.new_nonstop,							0,  0,  0, NULL},
 	{PE_BOOL,		0, STR_CONFIG_PATCHES_NONUNIFORM_STATIONS, "nonuniform_stations", &_patches.nonuniform_stations,		0,  0,  0, NULL},
-	{PE_UINT8,	0, STR_CONFIG_PATCHES_STATION_SPREAD,		"station_spread", &_patches.station_spread,						4, 64,  1, NULL},
+	{PE_UINT8,	0, STR_CONFIG_PATCHES_STATION_SPREAD,		"station_spread", &_patches.station_spread,						4, 64,  1, &InvalidateStationBuildWindow},
 	{PE_BOOL,		0, STR_CONFIG_PATCHES_SERVICEATHELIPAD, "service_at_helipad", &_patches.serviceathelipad,					0,  0,  0, NULL},
 	{PE_BOOL, 0, STR_CONFIG_PATCHES_CATCHMENT, "modified_catchment", &_patches.modified_catchment, 0, 0, 0, NULL},
 
@@ -717,19 +756,17 @@ static const PatchPage _patches_page[] = {
 	{_patches_ai,						lengthof(_patches_ai) },
 };
 
-extern uint GetCurrentCurrencyRate(void);
 
 static int32 ReadPE(const PatchEntry*pe)
 {
-	switch(pe->type) {
+	switch (pe->type) {
 	case PE_BOOL:   return *(bool*)pe->variable;
 	case PE_UINT8:  return *(uint8*)pe->variable;
 	case PE_INT16:  return *(int16*)pe->variable;
 	case PE_UINT16: return *(uint16*)pe->variable;
 	case PE_INT32:  return *(int32*)pe->variable;
 	case PE_CURRENCY:  return (*(int32*)pe->variable) * GetCurrentCurrencyRate();
-	default:
-		NOT_REACHED();
+	default: NOT_REACHED();
 	}
 
 	/* useless, but avoids compiler warning this way */
@@ -738,9 +775,7 @@ static int32 ReadPE(const PatchEntry*pe)
 
 static void WritePE(const PatchEntry *pe, int32 val)
 {
-
 	if ((pe->flags & PF_0ISDIS) && val <= 0) {
-		// "clamp" 'disabled' value to smallest type
 		switch (pe->type) {
 			case PE_BOOL: case PE_UINT8:
 				*(bool*)pe->variable = 0;
@@ -755,50 +790,31 @@ static void WritePE(const PatchEntry *pe, int32 val)
 		return;
 	}
 
-	switch(pe->type) {
-	case PE_BOOL: *(bool*)pe->variable = (bool)val; break;
-
-	case PE_UINT8: if ((uint8)val > (uint8)pe->max)
-									*(uint8*)pe->variable = (uint8)pe->max;
-								else if ((uint8)val < (uint8)pe->min)
-									*(uint8*)pe->variable = (uint8)pe->min;
-								else
-									*(uint8*)pe->variable = (uint8)val;
-								break;
-
-	case PE_INT16: if ((int16)val > (int16)pe->max)
-									*(int16*)pe->variable = (int16)pe->max;
-								else if ((int16)val < (int16)pe->min)
-									*(int16*)pe->variable = (int16)pe->min;
-								else
-									*(int16*)pe->variable = (int16)val;
-								break;
-
-	case PE_UINT16: if ((uint16)val > (uint16)pe->max)
-									*(uint16*)pe->variable = (uint16)pe->max;
-								else if ((uint16)val < (uint16)pe->min)
-									*(uint16*)pe->variable = (uint16)pe->min;
-								else
-									*(uint16*)pe->variable = (uint16)val;
-								break;
-
-	case PE_CURRENCY:
-	case PE_INT32: if ((int32)val > (int32)pe->max)
-									*(int32*)pe->variable = (int32)pe->max;
-								else if ((int32)val < (int32)pe->min)
-									*(int32*)pe->variable = (int32)pe->min;
-								else
-									*(int32*)pe->variable = val;
-								break;
-	default:
-		NOT_REACHED();
+	// "clamp" 'disabled' value to smallest type
+	switch (pe->type) {
+		case PE_BOOL:
+			*(bool*)pe->variable = (bool)val;
+			break;
+		case PE_UINT8:
+			*(uint8*)pe->variable = (uint8)clamp((uint8)val, (uint8)pe->min, (uint8)pe->max);
+			break;
+		case PE_INT16:
+			*(int16*)pe->variable = (int16)clamp((int16)val, (int16)pe->min, (int16)pe->max);
+			break;
+		case PE_UINT16:
+			*(uint16*)pe->variable = (uint16)clamp((uint16)val, (uint16)pe->min, (uint16)pe->max);
+			break;
+		case PE_CURRENCY: case PE_INT32:
+			*(int32*)pe->variable = (int32)clamp((int32)val, (int32)pe->min, (int32)pe->max);
+			break;
+		default: NOT_REACHED();
 	}
 }
 
 static void PatchesSelectionWndProc(Window *w, WindowEvent *e)
 {
 	uint i;
-	switch(e->event) {
+	switch (e->event) {
 	case WE_PAINT: {
 		int x,y;
 		const PatchEntry *pe;
@@ -1002,17 +1018,24 @@ static void PatchesSelectionWndProc(Window *w, WindowEvent *e)
 	}
 }
 
+/** Network-safe changing of patch-settings.
+ * @param p1 various bitstuffed elements
+ * - p1 = (bit 0- 7) - the patches type (page) that is being changed (construction, network, ai) (p1 & 0xFF)
+ * - p2 = (bit 8-15) - the actual patch (entry) being set inside the category ((p1>>8) & 0xFF)
+ * @param p2 the new value for the patch
+ * @todo check that the new value is a valid one. Awful lot of work, but since only
+ * the server is allowed to do this, we trust it on this one :)
+ */
 int32 CmdChangePatchSetting(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
-	const PatchPage *page;
-	const PatchEntry *pe;
+	byte pcat = p1 & 0xFF;
+	byte pel = (p1 >> 8) & 0xFF;
+
+	if (pcat >= lengthof(_patches_page)) return CMD_ERROR;
+	if (pel >= _patches_page[pcat].num) return CMD_ERROR;
 
 	if (flags & DC_EXEC) {
-		page = &_patches_page[(byte)p1];
-		if (page == NULL) return 0;
-		pe = &page->entries[(byte)(p1 >> 8)];
-		if (pe == NULL) return 0;
-
+		const PatchEntry *pe = &_patches_page[pcat].entries[pel];
 		WritePE(pe, (int32)p2);
 
 		InvalidateWindow(WC_GAME_OPTIONS, 0);
@@ -1021,116 +1044,79 @@ int32 CmdChangePatchSetting(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	return 0;
 }
 
-/* Those 2 functions need to be here, else we have to make some stuff non-static
-    and besides, it is also better to keep stuff like this at the same place */
-void ConsoleSetPatchSetting(char *name, char *value)
+static const PatchEntry *IConsoleGetPatch(const char *name, uint *page, uint *entry)
 {
-	const PatchPage *page;
-	const PatchEntry *pe = NULL;
-	bool found = false;
-	uint i;
-	unsigned int j;
-	int val;
+	const PatchPage *pp;
+	const PatchEntry *pe;
 
-	/* Search for the name in the patch-settings */
-	for (i = 0; i < lengthof(_patches_page); i++) {
-		page = &_patches_page[i];
-		for (j = 0; j < page->num; j++) {
-			pe = &page->entries[j];
-			if (strncmp(pe->console_name, name, sizeof(pe->console_name)) == 0) {
-				/* We found the name */
-				found = true;
-				break;
-			}
+	for (*page = 0; *page < lengthof(_patches_page); (*page)++) {
+		pp = &_patches_page[*page];
+		for (*entry = 0; *entry < pp->num; (*entry)++) {
+			pe = &pp->entries[*entry];
+			if (strncmp(pe->console_name, name, sizeof(pe->console_name)) == 0)
+				return pe;
 		}
-		if (found)
-			break;
 	}
 
-	/* We did not found the patch setting */
-	if (!found || pe == NULL) {
-		IConsolePrintF(_iconsole_color_warning, "'%s' is an unkown patch setting", name);
+	return NULL;
+}
+
+/* Those 2 functions need to be here, else we have to make some stuff non-static
+    and besides, it is also better to keep stuff like this at the same place */
+void IConsoleSetPatchSetting(char *name, const char *value)
+{
+	const PatchEntry *pe;
+	uint page, entry;
+	int val;
+
+	pe = IConsoleGetPatch(name, &page, &entry);
+
+	if (pe == NULL) {
+		IConsolePrintF(_icolour_warn, "'%s' is an unknown patch setting.", name);
 		return;
 	}
 
-	val = atoi(value);
+	sscanf(value, "%d", &val);
 
-	if (pe->type == PE_CURRENCY) {
+	if (pe->type == PE_CURRENCY) // currency can be different on each client
 		val /= GetCurrentCurrencyRate();
-	}
 
 	// If an item is playerbased, we do not send it over the network (if any)
 	if (pe->flags & PF_PLAYERBASED) {
 		WritePE(pe, val);
-	} else {
-		// Else we do
-		DoCommandP(0, i + (j << 8), val, NULL, CMD_CHANGE_PATCH_SETTING);
-	}
+	} else // Else we do
+		DoCommandP(0, page + (entry << 8), val, NULL, CMD_CHANGE_PATCH_SETTING);
 
-	switch(pe->type) {
-		case PE_BOOL:
-			if (val == 1)
-				snprintf(value, sizeof(value), "enabled");
-			else
-				snprintf(value, sizeof(value), "disabled");
-			break;
-		default:
-			break;
-	}
+	{
+		char tval[20];
+		const char *tval2 = value;
+		if (pe->type == PE_BOOL) {
+			snprintf(tval, sizeof(tval), (val == 1) ? "on" : "off");
+			tval2 = tval;
+		}
 
-	IConsolePrintF(_iconsole_color_warning, "'%s' changed in:", name);
-	IConsolePrintF(_iconsole_color_warning, "  '%s'", value);
+		IConsolePrintF(_icolour_warn, "'%s' changed to:  %s", name, tval2);
+	}
 }
 
-void ConsoleGetPatchSetting(char *name)
+void IConsoleGetPatchSetting(const char *name)
 {
-	const PatchPage *page;
-	const PatchEntry *pe = NULL;
-	char value[50];
-	bool found = false;
-	uint i;
-	unsigned int j;
+	char value[20];
+	uint page, entry;
+	const PatchEntry *pe = IConsoleGetPatch(name, &page, &entry);
 
-	/* Search for the name in the patch-settings */
-	for (i = 0; i < lengthof(_patches_page); i++) {
-		page = &_patches_page[i];
-		for (j = 0; j < page->num; j++) {
-			pe = &page->entries[j];
-			if (strncmp(pe->console_name, name, sizeof(pe->console_name)) == 0) {
-				/* We found the name */
-				found = true;
-				break;
-			}
-		}
-		if (found)
-			break;
-	}
-
-	/* We did not found the patch setting */
-	if (!found || pe == NULL) {
-		IConsolePrintF(_iconsole_color_warning, "'%s' is an unkown patch setting", name);
+	/* We did not find the patch setting */
+	if (pe == NULL) {
+		IConsolePrintF(_icolour_warn, "'%s' is an unknown patch setting.", name);
 		return;
 	}
 
-	/* 'pe' is now the correct patch setting */
-	switch(pe->type) {
-		case PE_BOOL:
-			if (ReadPE(pe) == 1)
-				snprintf(value, sizeof(value), "enabled");
-			else
-				snprintf(value, sizeof(value), "disabled");
-			break;
-		case PE_UINT8:
-		case PE_INT16:
-		case PE_UINT16:
-		case PE_INT32:
-		case PE_CURRENCY:
-			snprintf(value, sizeof(value), "%d", ReadPE(pe));
-			break;
-	}
+	if (pe->type == PE_BOOL) {
+		snprintf(value, sizeof(value), (ReadPE(pe) == 1) ? "on" : "off");
+	} else
+		snprintf(value, sizeof(value), "%d", ReadPE(pe));
 
-	IConsolePrintF(_iconsole_color_warning, "Current value for '%s' is:", name);
-	IConsolePrintF(_iconsole_color_warning, "  '%s'", value);
+	IConsolePrintF(_icolour_warn, "Current value for '%s' is: '%s'", name, value);
 }
 
 static const Widget _patches_selection_widgets[] = {
@@ -1162,8 +1148,6 @@ void ShowPatchesSelection(void)
 	AllocateWindowDesc(&_patches_selection_desc);
 }
 
-struct GRFFile *_sel_grffile;
-
 enum {
 	NEWGRF_WND_PROC_OFFSET_TOP_WIDGET = 14,
 	NEWGRF_WND_PROC_ROWSIZE = 14
@@ -1171,11 +1155,12 @@ enum {
 
 static void NewgrfWndProc(Window *w, WindowEvent *e)
 {
+	static GRFFile *_sel_grffile;
 	switch (e->event) {
 	case WE_PAINT: {
 		int x, y = NEWGRF_WND_PROC_OFFSET_TOP_WIDGET;
 		uint16 i = 0;
-		struct GRFFile *c = _first_grffile;
+		GRFFile *c = _first_grffile;
 
 		DrawWindowWidgets(w);
 
@@ -1291,7 +1276,8 @@ void ShowNewgrf(void)
 
 	{ // little helper function to calculate _grffile_count
 	  // should be REMOVED once _grffile_count is calculated at loading
-		struct GRFFile *c = _first_grffile;
+		GRFFile *c = _first_grffile;
+
 		_grffile_count = 0;
 		while (c != NULL) {
 			_grffile_count++;
@@ -1314,7 +1300,7 @@ void DrawArrowButtons(int x, int y, int state)
 	DrawStringCentered(x+15, y+1, STR_681A, 0);
 }
 
-char str_separator[2];
+char _str_separator[2];
 
 static void CustCurrencyWndProc(Window *w, WindowEvent *e)
 {
@@ -1336,7 +1322,7 @@ static void CustCurrencyWndProc(Window *w, WindowEvent *e)
 		// separator
 		DrawFrameRect(10, y+1, 29, y+9, 0, ((clk >> (i*2)) & 0x03)?0x20:0x00);
 		x = DrawString(x, y + 1, STR_CURRENCY_SEPARATOR, 0);
-		DoDrawString(str_separator, x + 4, y + 1, 6);
+		DoDrawString(_str_separator, x + 4, y + 1, 6);
 		x = 35;
 		y+=12;
 		i++;
@@ -1398,7 +1384,7 @@ static void CustCurrencyWndProc(Window *w, WindowEvent *e)
 			case 1: // separator
 				if ( IS_INT_INSIDE(x, 10, 30) )  // clicked button
 					WP(w,def_d).data_1 =  (1 << (line * 2 + 1));
-				str = AllocateName(str_separator, 0);
+				str = AllocateName(_str_separator, 0);
 				len = 1;
 				edittext = true;
 			break;
@@ -1437,12 +1423,12 @@ static void CustCurrencyWndProc(Window *w, WindowEvent *e)
 			break;
 		}
 
-		if(edittext) {
+		if (edittext) {
 			WP(w,def_d).data_2 = line;
 			ShowQueryString(
 			str,
 			STR_CURRENCY_CHANGE_PARAMETER,
-			len, // maximum number of characters OR
+			len + 1, // maximum number of characters OR
 			250, // characters up to this width pixels, whichever is satisfied first
 			w->window_class,
 			w->window_number);
@@ -1455,24 +1441,24 @@ static void CustCurrencyWndProc(Window *w, WindowEvent *e)
 
 	case WE_ON_EDIT_TEXT: {
 			int val;
-			byte *b = e->edittext.str;
+			const char *b = e->edittext.str;
 			switch (WP(w,def_d).data_2) {
-				case 0:
+				case 0: /* Exchange rate */
 					val = atoi(b);
 					val = clamp(val, 1, 5000);
 					_currency_specs[23].rate = val;
 				break;
-				case 1:
-					_currency_specs[23].separator = b[0];
-					ttd_strlcpy(str_separator, b, 16);
+				case 1: /* Thousands seperator */
+					_currency_specs[23].separator = (b[0] == '\0') ? ' ' : b[0];
+					ttd_strlcpy(_str_separator, b, 16);
 				break;
-				case 2:
+				case 2: /* Currency prefix */
 					ttd_strlcpy(_currency_specs[23].prefix, b, 16);
 				break;
-				case 3:
+				case 3: /* Currency suffix */
 					ttd_strlcpy(_currency_specs[23].suffix, b, 16);
 				break;
-				case 4:
+				case 4: /* Year to switch to euro */
 					val = atoi(b);
 					val = clamp(val, 1999, MAX_YEAR_END_REAL);
 					if (val == 1999) val = 0;
@@ -1515,8 +1501,8 @@ void ShowCustCurrency(void)
 {
 	Window *w;
 
-	str_separator[0] = _currency_specs[23].separator;
-	str_separator[1] = '\0';
+	_str_separator[0] = _currency_specs[23].separator;
+	_str_separator[1] = '\0';
 
 	DeleteWindowById(WC_CUSTOM_CURRENCY, 0);
 	w = AllocateWindowDesc(&_cust_currency_desc);
