@@ -1,7 +1,9 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "spritecache.h"
 #include "table/strings.h"
 #include "map.h"
+#include "tile.h"
 #include "gui.h"
 #include "window.h"
 #include "gfx.h"
@@ -248,7 +250,17 @@ static const uint32 _map_height_bits[16] = {
 	MKCOLOR(0x27272727),
 };
 
-static const uint32 _smallmap_contours_andor[12][2] = {
+typedef struct AndOr {
+	uint32 mor;
+	uint32 mand;
+} AndOr;
+
+static inline uint32 ApplyMask(uint32 colour, const AndOr* mask)
+{
+	return (colour & mask->mand) | mask->mor;
+}
+
+static const AndOr _smallmap_contours_andor[] = {
 	{MKCOLOR(0x00000000),MKCOLOR(0xFFFFFFFF)},
 	{MKCOLOR(0x000A0A00),MKCOLOR(0xFF0000FF)},
 	{MKCOLOR(0x00D7D700),MKCOLOR(0xFF0000FF)},
@@ -263,7 +275,7 @@ static const uint32 _smallmap_contours_andor[12][2] = {
 	{MKCOLOR(0x000A0A00),MKCOLOR(0xFF0000FF)},
 };
 
-static const uint32 _smallmap_vehicles_andor[12][2] = {
+static const AndOr _smallmap_vehicles_andor[] = {
 	{MKCOLOR(0x00000000),MKCOLOR(0xFFFFFFFF)},
 	{MKCOLOR(0x00D7D700),MKCOLOR(0xFF0000FF)},
 	{MKCOLOR(0x00D7D700),MKCOLOR(0xFF0000FF)},
@@ -278,7 +290,7 @@ static const uint32 _smallmap_vehicles_andor[12][2] = {
 	{MKCOLOR(0x00D7D700),MKCOLOR(0xFF0000FF)},
 };
 
-static const uint32 _smallmap_vegetation_andor[12][2] = {
+static const AndOr _smallmap_vegetation_andor[] = {
 	{MKCOLOR(0x00000000),MKCOLOR(0xFFFFFFFF)},
 	{MKCOLOR(0x00D7D700),MKCOLOR(0xFF0000FF)},
 	{MKCOLOR(0x00D7D700),MKCOLOR(0xFF0000FF)},
@@ -293,14 +305,14 @@ static const uint32 _smallmap_vegetation_andor[12][2] = {
 	{MKCOLOR(0x00D7D700),MKCOLOR(0xFF0000FF)},
 };
 
-static inline uint32 GetSmallMapCountoursPixels(uint tile)
-{
-	uint t;
 
-	t = TileType(tile);
+static inline TileType GetEffectiveTileType(TileIndex tile)
+{
+	TileType t = GetTileType(tile);
+
 	if (t == MP_TUNNELBRIDGE) {
 		t = _map5[tile];
-		if ((t & 0x80) == 0) t>>=1;
+		if ((t & 0x80) == 0) t >>= 1;
 		if ((t & 6) == 0) {
 			t = MP_RAILWAY;
 		} else if ((t & 6) == 2) {
@@ -309,37 +321,33 @@ static inline uint32 GetSmallMapCountoursPixels(uint tile)
 			t = MP_WATER;
 		}
 	}
+	return t;
+}
 
-	return (_map_height_bits[TileHeight(tile)] & _smallmap_contours_andor[t][1]) | _smallmap_contours_andor[t][0];
+
+static inline uint32 GetSmallMapContoursPixels(TileIndex tile)
+{
+	TileType t = GetEffectiveTileType(tile);
+
+	return
+		ApplyMask(_map_height_bits[TileHeight(tile)], &_smallmap_contours_andor[t]);
 }
 
 static void DrawSmallMapContours(byte *dst, uint xc, uint yc, int pitch, int reps, uint32 mask)
 {
 	do {
 		if (xc < MapMaxX() && yc < MapMaxY())
-		    if (dst > _screen.dst_ptr && dst < (_screen.dst_ptr + _screen.width * _screen.height - _screen.width) )
-		        WRITE_PIXELS_OR( dst, GetSmallMapCountoursPixels(TILE_XY(xc,yc)) & mask );
+			if (dst > _screen.dst_ptr && dst < (_screen.dst_ptr + _screen.width * _screen.height - _screen.width))
+				WRITE_PIXELS_OR(dst, GetSmallMapContoursPixels(TILE_XY(xc,yc)) & mask);
 	} while (xc++,yc++,dst+=pitch,--reps != 0);
 }
 
 
-static inline uint32 GetSmallMapVehiclesPixels(uint tile)
+static inline uint32 GetSmallMapVehiclesPixels(TileIndex tile)
 {
-	uint t;
+	TileType t = GetEffectiveTileType(tile);
 
-	t = TileType(tile);
-	if (t == MP_TUNNELBRIDGE) {
-		t = _map5[tile];
-		if ((t & 0x80) == 0) t>>=1;
-		if ((t & 6) == 0) {
-			t = MP_RAILWAY;
-		} else if ((t & 6) == 2) {
-			t = MP_STREET;
-		} else {
-			t = MP_WATER;
-		}
-	}
-	return (MKCOLOR(0x54545454) & _smallmap_vehicles_andor[t][1]) | _smallmap_vehicles_andor[t][0];
+	return ApplyMask(MKCOLOR(0x54545454), &_smallmap_vehicles_andor[t]);
 }
 
 
@@ -376,27 +384,15 @@ static const byte _industry_smallmap_colors[175] = {
 	 15, 15, 15, 15, 15, 15, 15,
 };
 
-static inline uint32 GetSmallMapIndustriesPixels(uint tile)
+static inline uint32 GetSmallMapIndustriesPixels(TileIndex tile)
 {
-	int t;
+	TileType t = GetEffectiveTileType(tile);
 
-	t = TileType(tile);
 	if (t == MP_INDUSTRY) {
 		byte color = _industry_smallmap_colors[_map5[tile]];
 		return color + (color << 8) + (color << 16) + (color << 24);
 	} else {
-		if (t == MP_TUNNELBRIDGE) {
-			t = _map5[tile];
-			if ((t & 0x80) == 0) t>>=1;
-			if ((t & 6) == 0) {
-				t = MP_RAILWAY;
-			} else if ((t & 6) == 2) {
-				t = MP_STREET;
-			} else {
-				t = MP_WATER;
-			}
-		}
-		return ((MKCOLOR(0x54545454) & _smallmap_vehicles_andor[t][1]) | _smallmap_vehicles_andor[t][0]);
+		return ApplyMask(MKCOLOR(0x54545454), &_smallmap_vehicles_andor[t]);
 	}
 }
 
@@ -408,12 +404,11 @@ static void DrawSmallMapIndustries(byte *dst, uint xc, uint yc, int pitch, int r
 	} while (xc++,yc++,dst+=pitch,--reps != 0);
 }
 
-static inline uint32 GetSmallMapRoutesPixels(uint tile)
+static inline uint32 GetSmallMapRoutesPixels(TileIndex tile)
 {
-	int t;
+	TileType t = GetEffectiveTileType(tile);
 	uint32 bits;
 
-	t = TileType(tile);
 	if (t == MP_STATION) {
 		byte m5 = _map5[tile];
 		(bits = MKCOLOR(0x56565656), m5 < 8) ||			//   8 - railroad station (green)
@@ -424,19 +419,8 @@ static inline uint32 GetSmallMapRoutesPixels(uint tile)
 		(bits = MKCOLOR(0xB8B8B8B8), m5 < 0x73) ||	// 115 - airport (red) (new airports)
 		(bits = MKCOLOR(0xFFFFFFFF), true);					// all others
 	} else {
-		if (t == MP_TUNNELBRIDGE) {
-			t = _map5[tile];
-			if ((t & 0x80) == 0) t>>=1;
-			if ((t & 6) == 0) {
-				t = MP_RAILWAY;
-			} else if ((t & 6) == 2) {
-				t = MP_STREET;
-			} else {
-				t = MP_WATER;
-			}
-		}
 		// ground color
-		bits = ((MKCOLOR(0x54545454) & _smallmap_contours_andor[t][1]) | _smallmap_contours_andor[t][0]);
+		bits = ApplyMask(MKCOLOR(0x54545454), &_smallmap_contours_andor[t]);
 	}
 	return bits;
 }
@@ -464,35 +448,33 @@ static const uint32 _vegetation_clear_bits[4 + 7] = {
 	MKCOLOR(0x54545454),
 };
 
-static inline uint32 GetSmallMapVegetationPixels(uint tile)
+static inline uint32 GetSmallMapVegetationPixels(TileIndex tile)
 {
-	int i,t;
+	TileType t = GetEffectiveTileType(tile);
+	int i;
 	uint32 bits;
 
-	t = TileType(tile);
-	if (t == MP_CLEAR) {
-		i = (_map5[tile] & 0x1F) - 4;
-		if (i >= 0) i = (i >> 2);
-		bits = _vegetation_clear_bits[i + 4];
-	} else if (t == MP_INDUSTRY) {
-		bits = IS_BYTE_INSIDE(_map5[tile], 0x10, 0x12) ? MKCOLOR(0xD0D0D0D0) : MKCOLOR(0xB5B5B5B5);
-	} else if (t == MP_TREES) {
-		bits = MKCOLOR(0x54575754);
-		if ((_map2[tile] & 0x30) == 0x20)
-			bits = (_opt.landscape == LT_HILLY) ? MKCOLOR(0x98575798) : MKCOLOR(0xC25757C2);
-	} else {
-		if (t == MP_TUNNELBRIDGE) {
-			t = _map5[tile];
-			if ((t & 0x80) == 0) t>>=1;
-			if ((t & 6) == 0) {
-				t = MP_RAILWAY;
-			} else if ((t & 6) == 2) {
-				t = MP_STREET;
-			} else {
-				t = MP_WATER;
-			}
-		}
-		bits = ((MKCOLOR(0x54545454) & _smallmap_vehicles_andor[t][1]) | _smallmap_vehicles_andor[t][0]);
+	switch (t) {
+		case MP_CLEAR:
+			i = (_map5[tile] & 0x1F) - 4;
+			if (i >= 0) i >>= 2;
+			bits = _vegetation_clear_bits[i + 4];
+			break;
+
+		case MP_INDUSTRY:
+			bits = IS_BYTE_INSIDE(_map5[tile], 0x10, 0x12) ? MKCOLOR(0xD0D0D0D0) : MKCOLOR(0xB5B5B5B5);
+			break;
+
+		case MP_TREES:
+			if ((_map2[tile] & 0x30) == 0x20)
+				bits = (_opt.landscape == LT_HILLY) ? MKCOLOR(0x98575798) : MKCOLOR(0xC25757C2);
+			else
+				bits = MKCOLOR(0x54575754);
+			break;
+
+		default:
+			bits = ApplyMask(MKCOLOR(0x54545454), &_smallmap_vehicles_andor[t]);
+			break;
 	}
 
 	return bits;
@@ -510,11 +492,10 @@ static void DrawSmallMapVegetation(byte *dst, uint xc, uint yc, int pitch, int r
 
 static uint32 _owner_colors[256];
 
-static inline uint32 GetSmallMapOwnerPixels(uint tile)
+static inline uint32 GetSmallMapOwnerPixels(TileIndex tile)
 {
-	int t;
+	TileType t = GetTileType(tile);
 
-	t = TileType(tile);
 	if (t == MP_HOUSE || _map_owner[tile] == OWNER_TOWN) {
 		t = 0x80;
 	} else if (t == MP_INDUSTRY) {
@@ -542,8 +523,7 @@ static const uint32 _smallmap_mask_left[3] = {
 	MKCOLOR(0xFFFFFF00),
 };
 
-static const uint32 _smallmap_mask_right[4] = {
-	MKCOLOR(0x00000000),
+static const uint32 _smallmap_mask_right[] = {
 	MKCOLOR(0x000000FF),
 	MKCOLOR(0x0000FFFF),
 	MKCOLOR(0x00FFFFFF),
@@ -587,10 +567,8 @@ static void DrawSmallMap(DrawPixelInfo *dpi, Window *w, int type, bool show_town
 	DrawPixelInfo *old_dpi;
 	int dx,dy, x, y, x2, y2;
 	byte *ptr;
-	uint tile_x, tile_y;
-	uint32 mask;
-	int t;
-	int reps;
+	int tile_x;
+	int tile_y;
 	SmallmapDrawProc *proc;
 	ViewPort *vp;
 
@@ -616,21 +594,21 @@ static void DrawSmallMap(DrawPixelInfo *dpi, Window *w, int type, bool show_town
 		FOR_ALL_PLAYERS(p) {
 			if (p->is_active)
 				_owner_colors[p->index] =
-					dup_byte32(GetSpritePtr(0x307 + p->player_color)[0xCB]);
+					dup_byte32(GetNonSprite(0x307 + p->player_color)[0xCB]);
 		}
 	}
 
-	tile_x = (int)WP(w,smallmap_d).scroll_x >> 4;
-	tile_y = (int)WP(w,smallmap_d).scroll_y >> 4;
+	tile_x = WP(w,smallmap_d).scroll_x / 16;
+	tile_y = WP(w,smallmap_d).scroll_y / 16;
 
 	dx = dpi->left + WP(w,smallmap_d).subscroll;
-	tile_x -= (dx >> 2);
-	tile_y += (dx >> 2);
+	tile_x -= dx / 4;
+	tile_y += dx / 4;
 	dx &= 3;
 
 	dy = dpi->top;
-	tile_x += (dy >> 1);
-	tile_y += (dy >> 1);
+	tile_x += dy / 2;
+	tile_y += dy / 2;
 
 	if (dy & 1) {
 		tile_x++;
@@ -649,7 +627,11 @@ static void DrawSmallMap(DrawPixelInfo *dpi, Window *w, int type, bool show_town
 	y = 0;
 
 	for(;;) {
-		mask = (uint32)-1;
+		uint32 mask;
+		int reps;
+		int t;
+
+		mask = 0xFFFFFFFF;
 
 		/* distance from left edge */
 		if (x < 0) {
@@ -661,14 +643,14 @@ static void DrawSmallMap(DrawPixelInfo *dpi, Window *w, int type, bool show_town
 		/* distance from right edge */
 		t = dpi->width - x;
 		if (t < 4) {
-			if (t < 0)
+			if (t <= 0)
 				break; /* exit loop */
 			/* mask to use at the right edge */
-			mask &= _smallmap_mask_right[t];
+			mask &= _smallmap_mask_right[t - 1];
 		}
 
 		/* number of lines */
-		reps = ((dpi->height - y + 1) >> 1);
+		reps = (dpi->height - y + 1) / 2;
 		if (reps > 0) {
 //			assert(ptr >= dpi->dst_ptr);
 			proc(ptr, tile_x, tile_y, dpi->pitch*2, reps, mask);
@@ -695,19 +677,19 @@ skip_column:
 		byte color;
 
 		FOR_ALL_VEHICLES(v) {
-			if (v->type != 0 && v->type != 0x14 && (v->vehstatus & (VS_HIDDEN|VS_UNCLICKABLE)) == 0) {
+			if (v->type != 0 && v->type != VEH_Special &&
+					(v->vehstatus & (VS_HIDDEN | VS_UNCLICKABLE)) == 0) {
 				// Remap into flat coordinates.
 				Point pt = RemapCoords(
-					(int)(v->x_pos - WP(w,smallmap_d).scroll_x) >> 4,
-					(int)(v->y_pos - WP(w,smallmap_d).scroll_y) >> 4,
+					(v->x_pos - WP(w,smallmap_d).scroll_x) / 16,
+					(v->y_pos - WP(w,smallmap_d).scroll_y) / 16,
 					0);
 				x = pt.x;
 				y = pt.y;
 
 				// Check if y is out of bounds?
 				y -= dpi->top;
-				if ((uint)y >= (uint)dpi->height)
-					continue;
+				if (!IS_INT_INSIDE(y, 0, dpi->height)) continue;
 
 				// Default is to draw both pixels.
 				skip = false;
@@ -742,12 +724,13 @@ skip_column:
 
 	if (show_towns) {
 		Town *t;
+
 		FOR_ALL_TOWNS(t) {
 			if (t->xy != 0) {
 				// Remap the town coordinate
 				Point pt = RemapCoords(
-					(int)(TileX(t->xy) * 16 - WP(w,smallmap_d).scroll_x) >> 4,
-					(int)(TileY(t->xy) * 16 - WP(w,smallmap_d).scroll_y) >> 4,
+					(int)(TileX(t->xy) * 16 - WP(w, smallmap_d).scroll_x) / 16,
+					(int)(TileY(t->xy) * 16 - WP(w, smallmap_d).scroll_y) / 16,
 					0);
 				x = pt.x - WP(w,smallmap_d).subscroll + 3 - (t->sign.width_2 >> 1);
 				y = pt.y;
@@ -772,17 +755,14 @@ skip_column:
 		// Find main viewport.
 		vp = FindWindowById(WC_MAIN_WINDOW,0)->viewport;
 
-		pt = RemapCoords(
-			WP(w,smallmap_d).scroll_x,
-			WP(w,smallmap_d).scroll_y,
-			0);
+		pt = RemapCoords(WP(w, smallmap_d).scroll_x, WP(w, smallmap_d).scroll_y, 0);
 
 		x = vp->virtual_left - pt.x;
 		y = vp->virtual_top - pt.y;
-		x2 = (x + vp->virtual_width) >> 4;
-		y2 = (y + vp->virtual_height) >> 4;
-		x >>= 4;
-		y >>= 4;
+		x2 = (x + vp->virtual_width) / 16;
+		y2 = (y + vp->virtual_height) / 16;
+		x /= 16;
+		y /= 16;
 
 		x -= WP(w,smallmap_d).subscroll;
 		x2 -= WP(w,smallmap_d).subscroll;
@@ -940,52 +920,53 @@ static const Widget _extra_view_port_widgets[] = {
 
 static void ExtraViewPortWndProc(Window *w, WindowEvent *e)
 {
-	ViewPort *vp = w->viewport;
-
-	switch(e->event) {
-	case WE_PAINT: {
+	switch (e->event) {
+	case WE_CREATE: /* Disable zoom in button */
+		w->disabled_state = (1 << 5);
+		break;
+	case WE_PAINT:
 		// set the number in the title bar
 		SetDParam(0, (w->window_number+1));
 
 		DrawWindowWidgets(w);
 		DrawWindowViewport(w);
-	}	break;
+		break;
+
 	case WE_CLICK: {
-		switch(e->click.widget) {
-		case 5: { /* zoom in */
+		switch (e->click.widget) {
+		case 5: /* zoom in */
 			DoZoomInOutWindow(ZOOM_IN, w);
-		} break;
-
-		case 6: { /* zoom out */
+			break;
+		case 6: /* zoom out */
 			DoZoomInOutWindow(ZOOM_OUT, w);
-		} break;
-
-		case 7: { /* location button (move main view to same spot as this view) */
-			Window * w2 = FindWindowById(WC_MAIN_WINDOW, 0);
-			int x = WP(w,vp_d).scrollpos_x; // Where is the main looking at
-			int y = WP(w,vp_d).scrollpos_y;
+			break;
+		case 7: { /* location button (move main view to same spot as this view) 'Paste Location' */
+			Window *w2 = FindWindowById(WC_MAIN_WINDOW, 0);
+			int x = WP(w, vp_d).scrollpos_x; // Where is the main looking at
+			int y = WP(w, vp_d).scrollpos_y;
 
 			// set this view to same location. Based on the center, adjusting for zoom
-			WP(w2,vp_d).scrollpos_x =  x - (w2->viewport->virtual_width -  (w->viewport->virtual_width  << vp->zoom)) / 2;
-			WP(w2,vp_d).scrollpos_y =  y - (w2->viewport->virtual_height - (w->viewport->virtual_height << vp->zoom)) / 2;
+			WP(w2, vp_d).scrollpos_x =  x - (w2->viewport->virtual_width -  w->viewport->virtual_width) / 2;
+			WP(w2, vp_d).scrollpos_y =  y - (w2->viewport->virtual_height - w->viewport->virtual_height) / 2;
 		} break;
-		case 8: { /* inverse location button (move this view to same spot as main view) */
-			Window * w2 = FindWindowById(WC_MAIN_WINDOW, 0);
-			int x = WP(w2,vp_d).scrollpos_x;
-			int y = WP(w2,vp_d).scrollpos_y;
+		case 8: { /* inverse location button (move this view to same spot as main view) 'Copy Location' */
+			const Window *w2 = FindWindowById(WC_MAIN_WINDOW, 0);
+			int x = WP(w2, vp_d).scrollpos_x;
+			int y = WP(w2, vp_d).scrollpos_y;
 
-			WP(w,vp_d).scrollpos_x =  x + (w2->viewport->virtual_width -  (w->viewport->virtual_width  << vp->zoom)) / 2;
-			WP(w,vp_d).scrollpos_y =  y + (w2->viewport->virtual_height - (w->viewport->virtual_height << vp->zoom)) / 2;
+			WP(w, vp_d).scrollpos_x =  x + (w2->viewport->virtual_width -  w->viewport->virtual_width) / 2;
+			WP(w, vp_d).scrollpos_y =  y + (w2->viewport->virtual_height - w->viewport->virtual_height) / 2;
 		} break;
 		}
 	} break;
-	case WE_RESIZE: {
+
+	case WE_RESIZE:
 		w->viewport->width  += e->sizing.diff.x;
 		w->viewport->height += e->sizing.diff.y;
 
 		w->viewport->virtual_width  += e->sizing.diff.x;
 		w->viewport->virtual_height += e->sizing.diff.y;
-	} break;
+		break;
 	}
 }
 
@@ -1010,17 +991,15 @@ void ShowExtraViewPortWindow(void)
 	w = AllocateWindowDescFront(&_extra_view_port_desc, i);
 	if (w) {
 		int x, y;
-		// disable zoom in button
-		w->disabled_state = (1 << 4);
 		// the main window with the main view
 		v = FindWindowById(WC_MAIN_WINDOW, 0);
 		// New viewport start ats (zero,zero)
 		AssignWindowViewport(w, 3, 17, 294, 214, 0 , 0);
 
 		// center on same place as main window (zoom is maximum, no adjustment needed)
-		x = WP(v,vp_d).scrollpos_x;
-		y = WP(v,vp_d).scrollpos_y;
-		WP(w,vp_d).scrollpos_x = x + (v->viewport->virtual_width  - (294)) / 2;
-		WP(w,vp_d).scrollpos_y = y + (v->viewport->virtual_height - (214)) / 2;
+		x = WP(v, vp_d).scrollpos_x;
+		y = WP(v, vp_d).scrollpos_y;
+		WP(w, vp_d).scrollpos_x = x + (v->viewport->virtual_width  - (294)) / 2;
+		WP(w, vp_d).scrollpos_y = y + (v->viewport->virtual_height - (214)) / 2;
 	}
 }
