@@ -1,5 +1,7 @@
 #include "stdafx.h"
 #include "ttd.h"
+#include "debug.h"
+#include "strings.h"
 #include "table/strings.h"
 #include "window.h"
 #include "gui.h"
@@ -80,11 +82,10 @@ static void GlobalSortStationList(void)
 	uint16 *i;
 
 	// reset #-of stations to 0 because ++ is used for value-assignment
-	for (i = _num_station_sort; i != endof(_num_station_sort); i++)
-		*i = 0;
+	memset(_num_station_sort, 0, sizeof(_num_station_sort));
 
 	/* Create array for sorting */
-	_station_sort = realloc(_station_sort, _stations_size * sizeof(_station_sort[0]));
+	_station_sort = realloc(_station_sort, GetStationPoolSize() * sizeof(_station_sort[0]));
 	if (_station_sort == NULL)
 		error("Could not allocate memory for the station-sorting-list");
 
@@ -315,10 +316,8 @@ static void DrawStationViewWindow(Window *w)
 	int pos;
 	StringID str;
 	uint16 station_id;
-	byte *b;
 
-
-	station_id = (byte)w->window_number;
+	station_id = (uint16)w->window_number;
 
 	st = GetStation(w->window_number);
 
@@ -399,30 +398,24 @@ static void DrawStationViewWindow(Window *w)
 	} while (pos > -5 && ++i != 12);
 
 	if (IsWindowOfPrototype(w, _station_view_widgets)) {
-		b = _userstring;
-		b[0] = 0x81;
-		b[1] = STR_000C_ACCEPTS;
-		b[2] = STR_000C_ACCEPTS >> 8;
-		b += 3;
+		char *b = _userstring;
 
-		for(i=0; i!=NUM_CARGO; i++) {
-			if ((b - (byte *) &_userstring) + 5 > USERSTRING_LEN - 1)
-				break;
+		b = InlineString(b, STR_000C_ACCEPTS);
+
+		for (i = 0; i != NUM_CARGO; i++) {
+			if (b >= endof(_userstring) - 5 - 1) break;
 			if (st->goods[i].waiting_acceptance & 0x8000) {
-				b[0] = 0x81;
-				WRITE_LE_UINT16(b+1, _cargoc.names_s[i]);
-				WRITE_LE_UINT16(b+3, 0x202C);
-				b += 5;
+				b = InlineString(b, _cargoc.names_s[i]);
+				*b++ = ',';
+				*b++ = ' ';
 			}
 		}
 
-		if (b == (byte*)&_userstring[3]) {
-			b[0] = 0x81;
-			b[1] = STR_00D0_NOTHING;
-			b[2] = STR_00D0_NOTHING >> 8;
-			b[3] = 0;
+		if (b == &_userstring[3]) {
+			b = InlineString(b, STR_00D0_NOTHING);
+			*b++ = '\0';
 		} else {
-			b[-2] = 0;
+			b[-2] = '\0';
 		}
 
 		DrawStringMultiLine(2, 67, STR_SPEC_USERSTRING, 245);
@@ -432,7 +425,7 @@ static void DrawStationViewWindow(Window *w)
 
 		y = 77;
 		for(i=0; i!=NUM_CARGO; i++) {
-			if (st->goods[i].enroute_from != 0xFF) {
+			if (st->goods[i].enroute_from != INVALID_STATION) {
 				SetDParam(0, _cargoc.names_s[i]);
 				SetDParam(2, st->goods[i].rating * 101 >> 8);
 				SetDParam(1, STR_3035_APPALLING + (st->goods[i].rating >> 5));
@@ -479,41 +472,44 @@ static void StationViewWndProc(Window *w, WindowEvent *e)
 			ShowQueryString(st->string_id, STR_3030_RENAME_STATION_LOADING, 31, 180, w->window_class, w->window_number);
 		}	break;
 
-		case 10: {
+		case 10: { /* Show a list of scheduled trains to this station */
 			const Station *st = GetStation(w->window_number);
 			ShowPlayerTrains(st->owner, w->window_number);
 			break;
 		}
 
-		case 11: {
+		case 11: { /* Show a list of scheduled road-vehicles to this station */
 			const Station *st = GetStation(w->window_number);
 			ShowPlayerRoadVehicles(st->owner, w->window_number);
 			break;
 		}
 
-		case 12: {
+		case 12: { /* Show a list of scheduled aircraft to this station */
 			const Station *st = GetStation(w->window_number);
-			ShowPlayerAircraft(st->owner, w->window_number);
+			/* Since oilrigs have no owners, show the scheduled aircraft of current player */
+			PlayerID owner = (st->owner == OWNER_NONE) ? _current_player : st->owner;
+			ShowPlayerAircraft(owner, w->window_number);
 			break;
 		}
 
-		case 13: {
+		case 13: { /* Show a list of scheduled ships to this station */
 			const Station *st = GetStation(w->window_number);
-			ShowPlayerShips(st->owner, w->window_number);
+			/* Since oilrigs/bouys have no owners, show the scheduled ships of current player */
+			PlayerID owner = (st->owner == OWNER_NONE) ? _current_player : st->owner;
+			ShowPlayerShips(owner, w->window_number);
 			break;
 		}
 		}
 		break;
 
 	case WE_ON_EDIT_TEXT: {
-		Station *st;
-		byte *b = e->edittext.str;
-		if (*b == 0)
-			return;
-		memcpy(_decode_parameters, b, 32);
+		if (e->edittext.str[0] != '\0') {
+			Station* st = GetStation(w->window_number);
 
-		st = GetStation(w->window_number);
-		DoCommandP(st->xy, w->window_number, 0, NULL, CMD_RENAME_STATION | CMD_MSG(STR_3031_CAN_T_RENAME_STATION));
+			_cmd_text = e->edittext.str;
+			DoCommandP(st->xy, w->window_number, 0, NULL,
+				CMD_RENAME_STATION | CMD_MSG(STR_3031_CAN_T_RENAME_STATION));
+		}
 	} break;
 
 	case WE_DESTROY: {
